@@ -14,93 +14,6 @@
 
 #   include	"docBuf.h"
 
-/************************************************************************/
-/*									*/
-/*  Style administration.						*/
-/*									*/
-/************************************************************************/
-
-void docInitStyle(	DocumentStyle *	ds )
-    {
-    ds->dsStyleNumber= -1;
-    ds->dsBasedOn= -1;
-    ds->dsIsCharacterStyle= 0;
-    ds->dsBusy= 0;
-
-    docInitParagraphProperties( &(ds->dsParagraphProperties) );
-
-    docInitTextAttribute( &(ds->dsTextAttribute) );
-
-    ds->dsName= (char *)0;
-    }
-
-void docCleanStyle(	DocumentStyle *	ds )
-    {
-    docCleanParagraphProperties( &(ds->dsParagraphProperties) );
-
-    if  ( ds->dsName )
-	{ free( ds->dsName );	}
-    }
-
-DocumentStyle * docInsertStyle(	BufferDocument *	bd,
-				int			n,
-				const char *		name )
-    {
-    DocumentStyle *	ds;
-    char *		s= (char *)0;
-
-    if  ( n > bd->bdStyleCount )
-	{
-	ds= (DocumentStyle *)realloc( bd->bdStyles,
-		    ( n + 1 ) * sizeof( DocumentStyle ) );
-	}
-    else{
-	ds= (DocumentStyle *)realloc( bd->bdStyles,
-		    ( bd->bdStyleCount + 1 ) * sizeof( DocumentStyle ) );
-	}
-    if  ( ! ds )
-	{ LLDEB(bd->bdStyleCount,ds); return ds; }
-    bd->bdStyles= ds;
-
-    if  ( name && name[0] )
-	{
-	s= strdup( name );
-	if  ( ! s )
-	    { XDEB(s); return (DocumentStyle *)0;	}
-	}
-
-    if  ( n == -1 )
-	{
-	for ( n= 0; n < bd->bdStyleCount; n++ )
-	    {
-	    if  (  ds[n].dsStyleNumber < 0 )
-		{ break;	}
-	    }
-	}
-    else{
-	if  ( n < bd->bdStyleCount && ds[n].dsStyleNumber >= 0 )
-	    { LLDEB(n,ds[n].dsStyleNumber);	}
-	/* No!!
-	for ( i= bd->bdStyleCount; i > n; i-- )
-	    { ds[i]= ds[i-1];	}
-	*/
-	}
-
-    while( bd->bdStyleCount < n )
-	{ docInitStyle( ds+ bd->bdStyleCount ); bd->bdStyleCount++;	}
-
-    ds += n;
-    docInitStyle( ds );
-
-    ds->dsStyleNumber= n;
-    ds->dsName= s;
-
-    if  ( n == bd->bdStyleCount )
-	{ bd->bdStyleCount++;	}
-
-    return ds;
-    }
-
 /************************************************************************/
 /*									*/
 /*  Free a BufferItem.							*/
@@ -203,23 +116,9 @@ void docFreeItem(	BufferDocument *	bd,
 
 void docFreeDocument( BufferDocument *	bd )
     {
-    DocumentStyle *	ds;
-    int			i;
-
     docCleanItem( bd, &(bd->bdItem) );
 
-    ds= bd->bdStyles;
-    for ( i= 0; i < bd->bdStyleCount; ds++, i++ )
-	{
-	if  ( ds->dsStyleNumber < 0 )
-	    { continue;	}
-
-	docCleanStyle( ds );
-	}
-
-    if  ( bd->bdStyles )
-	{ free( bd->bdStyles );	}
-
+    docCleanStyleSheet( &(bd->bdStyleSheet) );
     docCleanFieldList( bd, &(bd->bdFieldList) );
 
     docCleanDocumentProperties( &(bd->bdProperties) );
@@ -238,6 +137,11 @@ void docInitHeaderFooter(	HeaderFooter *	hf )
     hf->hfItem= (BufferItem *)0;
     hf->hfPageUsedFor= -1;
 
+    hf->hfY0UsedTwips= 0;
+    hf->hfY1UsedTwips= 0;
+    hf->hfY0ReservedTwips= 0;
+    hf->hfY1ReservedTwips= 0;
+
     return;
     }
 
@@ -254,7 +158,8 @@ void docInitItem(	BufferItem *		bi,
 			BufferItem *		parent,
 			const BufferDocument *	bd,
 			int			numberInParent,
-			int			level )
+			int			level,
+			int			inHeaderFooter )
     {
     bi->biChildren= (BufferItem **)0;
     bi->biChildCount= 0;
@@ -322,7 +227,8 @@ void docInitItem(	BufferItem *		bi,
 	    LDEB(level); return;
 	}
 
-    bi->biLevel= (ItemLevel)level;
+    bi->biLevel= level;
+    bi->biInHeaderFooter= inHeaderFooter;
     bi->biParent= parent;
     bi->biNumberInParent= numberInParent;
 
@@ -334,9 +240,10 @@ void docInitItem(	BufferItem *		bi,
 
 void docInitLayoutPosition(	LayoutPosition *	lp )
     {
-    lp->lpPage= 0;
     lp->lpPageYTwips= 0;
-    lp->lpYPixels= 0;
+    lp->lpPage= 0;
+    lp->lpColumn= 0;
+    lp->lpAtTopOfColumn= 0;
     }
 
 /************************************************************************/
@@ -349,11 +256,9 @@ void docInitDocument(	BufferDocument *	bd )
     {
     docInitDocumentProperties( &(bd->bdProperties) );
 
-    docInitItem( &(bd->bdItem), (BufferItem *)0, bd, 0, DOClevDOC );
-
-    bd->bdStyles= (DocumentStyle *)0;
-    bd->bdStyleCount= 0;
+    docInitItem( &(bd->bdItem), (BufferItem *)0, bd, 0, DOClevDOC, DOCinBODY );
 
+    docInitStyleSheet( &(bd->bdStyleSheet) );
     docInitFieldList( &(bd->bdFieldList) );
 
     return;
@@ -406,34 +311,6 @@ BufferDocument * docNewFile(	const char *			fontFamilyName,
     return bd;
     }
 
-/************************************************************************/
-/*									*/
-/*  Make sure there is enough space in the string of a paragraph.	*/
-/*									*/
-/************************************************************************/
-
-int docInflateTextString(	BufferItem *    bi,
-				int		by	)
-    {
-    unsigned char *	freshString;
-    int			newSize;
-
-    if  ( bi->biLevel != DOClevPARA )
-	{ LLDEB(bi->biLevel,DOClevPARA); return -1;	}
-
-    if  ( bi->biParaStrlen % 50 )
-	{ newSize= bi->biParaStrlen+ by+  1; }
-    else{ newSize= bi->biParaStrlen+ by+ 51; }
-
-    freshString= (unsigned char *)realloc( bi->biParaString, newSize );
-    if  ( ! freshString )
-	{ LXDEB(bi->biParaStrlen,freshString); return -1; }
-
-    bi->biParaString= freshString;
-
-    return 0;
-    }
-
 /************************************************************************/
 /*									*/
 /*  Add a Particule to a paragraph.					*/
@@ -536,7 +413,7 @@ TextParticule *	docInsertTextParticule(	BufferItem *	bi,
 
 void docDeleteParticules(	BufferItem *	bi,
 				int		first,
-				int		count	)
+				int		count )
     {
     if  ( first > bi->biParaParticuleCount )
 	{
@@ -677,10 +554,6 @@ void docInitTextLine(		TextLine *	tl )
     tl->tlLineHeightTwips= 0;
     tl->tlLineSpacingTwips= 0;
 
-    tl->tlLineAscentPixels= 0;
-    tl->tlLineHeightTwips= 0;
-    tl->tlLineSpacingTwips= 0;
-
     tl->tlHasPageBreak= 0;
 
     return;
@@ -781,10 +654,10 @@ BufferItem * docInsertItem(	const BufferDocument *	bd,
 	{
 	n= *countTarget;
 
-	docInitItem( bi, parent, bd, n, level );
+	docInitItem( bi, parent, bd, n, level, parent->biInHeaderFooter );
 	}
     else{
-	docInitItem( bi, parent, bd, n, level );
+	docInitItem( bi, parent, bd, n, level, parent->biInHeaderFooter );
 
 	for ( i= *countTarget; i > n; i-- )
 	    {
@@ -809,14 +682,15 @@ BufferItem * docInsertItem(	const BufferDocument *	bd,
 /************************************************************************/
 
 BufferItem *	docMakeExternalItem(	BufferDocument *		bd,
-					const SectionProperties *	sp )
+					const SectionProperties *	sp,
+					int				inHdFt )
     {
     BufferItem *	bi= (BufferItem *)malloc( sizeof(BufferItem) );
 
     if  ( ! bi )
 	{ XDEB(bi); return (BufferItem *)0;	}
 
-    docInitItem( bi, (BufferItem *)0, bd, 0, DOClevSECT );
+    docInitItem( bi, (BufferItem *)0, bd, 0, DOClevSECT, inHdFt );
 
     if  ( docCopySectionProperties( &(bi->biSectProperties), sp ) )
 	{
@@ -1137,93 +1011,6 @@ int docFindLineOfParticule(	const BufferItem *	bi,
     return 0;
     }
 
-/************************************************************************/
-/*									*/
-/*  Derive the frame for a paragraph from the page rectangle and the	*/
-/*  paragraph properties.						*/
-/*									*/
-/*  For paragraphs inside a table cell, geometry is derived from the	*/
-/*  table column.							*/
-/*									*/
-/************************************************************************/
-
-void docParagraphFrameTwips(	FormattingFrame *		ff,
-				int				bottom,
-				int				stripHigh,
-				const BufferItem *		paraBi )
-    {
-    const BufferItem *		sectBi;
-    const SectionProperties *	sp;
-    const DocumentGeometry *	dg;
-
-    sectBi= paraBi;
-    while( sectBi				&&
-	   sectBi->biLevel != DOClevSECT	)
-	{ sectBi= sectBi->biParent;	}
-
-    if  ( ! sectBi )
-	{ SXDEB(docLevelStr(paraBi->biLevel),sectBi); return;	}
-
-    sp= &(sectBi->biSectProperties);
-    dg= &(sp->spDocumentGeometry);
-
-    ff->ffX0Geometry= dg->dgLeftMarginTwips;
-    ff->ffX1Geometry= dg->dgPageWideTwips- dg->dgRightMarginTwips;
-
-    ff->ffPageHigh= dg->dgPageHighTwips-
-			    dg->dgTopMarginTwips- dg->dgBottomMarginTwips;
-    ff->ffStripHigh= stripHigh;
-
-    if  ( bottom > 0							&&
-	  bottom <= dg->dgPageHighTwips- dg->dgBottomMarginTwips	)
-	{ ff->ffY1= bottom;						}
-    else{ ff->ffY1= dg->dgPageHighTwips- dg->dgBottomMarginTwips;	}
-
-    if  ( paraBi->biParaInTable )
-	{
-	int			col0;
-	int			col1;
-	const BufferItem *	cellBi= paraBi->biParent;
-	const BufferItem *	rowBi= cellBi->biParent;
-	const RowProperties *	rp= &(rowBi->biRowProperties);
-
-	col0= col1= cellBi->biNumberInParent;
-
-	if  ( col1 >= rowBi->biRowCellCount )
-	    { LLDEB(col1,rowBi->biRowCellCount);	}
-	else{
-	    const CellProperties *	cp1= rp->rpCells+ col1;
-
-	    if  ( cp1->cpLeftInMergedRow )
-		{
-		while( col1 < rowBi->biRowCellCount- 1	&&
-		       cp1[1].cpMergedWithLeft		)
-		    { col1++; cp1++;	}
-		}
-
-	    ff->ffX1Geometry= dg->dgLeftMarginTwips+ cp1->cpRightBoundaryTwips;
-
-	    if  ( col0 == 0 )
-		{ ff->ffX0Geometry += rowBi->biRowLeftIndentTwips;	}
-	    else{
-		CellProperties *	cpp;
-
-		cpp= rp->rpCells+ col0- 1;
-		ff->ffX0Geometry += cpp->cpRightBoundaryTwips;
-		}
-	    }
-
-	ff->ffX0Geometry += rowBi->biRowHalfGapWidthTwips;
-	}
-
-    ff->ffX0TextLines= ff->ffX0Geometry+ paraBi->biParaLeftIndentTwips;
-    ff->ffX1TextLines= ff->ffX1Geometry- paraBi->biParaRightIndentTwips;
-
-    ff->ffX0FirstLine= ff->ffX0TextLines+ paraBi->biParaFirstIndentTwips;
-
-    return;
-    }
-
 /************************************************************************/
 /*									*/
 /*  Copy particules plus contents from one paragraph to another.	*/
@@ -1256,6 +1043,7 @@ int docCopyParticules(	BufferDocument *	bdTo,
 
     int				size;
     int				stroffTo;
+    int				stroffShift;
 
     int				i;
 
@@ -1281,9 +1069,6 @@ int docCopyParticules(	BufferDocument *	bdTo,
     size= tpFrom[countFrom-1].tpStroff+ tpFrom[countFrom-1].tpStrlen-
 							    tpFrom->tpStroff;
 
-    if  ( docInflateTextString( biTo, size ) )
-	{ LLDEB(biTo->biParaStrlen,size); return -1;	}
-
     if  ( partTo >= biTo->biParaParticuleCount )
 	{
 	partTo= biTo->biParaParticuleCount;
@@ -1294,15 +1079,9 @@ int docCopyParticules(	BufferDocument *	bdTo,
 	stroffTo= tp->tpStroff;
 	}
 
-    if  ( biTo->biParaStrlen > 0 )
-	{
-	memmove( biTo->biParaString+ stroffTo+ size,
-			    biTo->biParaString+ stroffTo,
-			    biTo->biParaStrlen- stroffTo );
-	}
-    memcpy( biTo->biParaString+ stroffTo,
-			    biFrom->biParaString+ tpFrom->tpStroff, size );
-    biTo->biParaString[biTo->biParaStrlen+ size]= '\0';
+    if  ( docParaStringReplace( &stroffShift, biTo, stroffTo, stroffTo,
+			    biFrom->biParaString+ tpFrom->tpStroff, size ) )
+	{ LDEB(size); return -1;	}
 			    
     /*  3  */
     tp= biTo->biParaParticules;
@@ -1337,10 +1116,9 @@ int docCopyParticules(	BufferDocument *	bdTo,
 	}
 
     biTo->biParaParticuleCount += countFrom;
-    biTo->biParaStrlen += size;
 
     *pParticulesInserted += countFrom- replacedEmpty;
-    *pCharactersCopied += size;
+    *pCharactersCopied += stroffShift;
 
     return 0;
     }
