@@ -14,6 +14,14 @@
 #   include	"docShape.h"
 #   include	<appGeo.h>
 #   include	<sioGeneral.h>
+#   include	<utilMemoryBuffer.h>
+
+#   include	"docParaProperties.h"
+#   include	"docSectProperties.h"
+#   include	"docStyleSheet.h"
+#   include	"docDocumentProperties.h"
+#   include	"docField.h"
+#   include	"docObject.h"
 
 /************************************************************************/
 /*									*/
@@ -59,17 +67,25 @@ typedef struct TextParticule
 
 /************************************************************************/
 /*									*/
-/*  Used to layout lines of text.					*/
+/*  The geometry bounds inside which a layout job operates.		*/
 /*									*/
 /************************************************************************/
 
 typedef struct FormattingFrame
     {
-    int		ffX0Geometry;
-    int		ffX0TextLines;
-    int		ffX0FirstLine;
-    int		ffX1Geometry;
-    int		ffX1TextLines;
+    int		ffX0GeometryTwips;
+    int		ffX0TextLinesTwips;
+    int		ffX0FirstLineTwips;
+
+    int		ffX0TextLinesPixels;
+    int		ffX0FirstLinePixels;
+
+    int		ffX1GeometryTwips;
+    int		ffX1TextLinesTwips;
+
+    int		ffX1TextLinesPixels;
+
+    int		ffPageFrameBottomTwips;
 
     int		ffY1;
 
@@ -77,49 +93,23 @@ typedef struct FormattingFrame
     int		ffStripHigh;
     } FormattingFrame;
 
+/************************************************************************/
+/*									*/
+/*  Used to layout lines of text.					*/
+/*									*/
+/*  A0 paper is 4* 30 cm high: 28.3465* 120* 20= 68031.6 Twips.		*/
+/*  So an unsigned short limits us to A1 paper.				*/
+/*									*/
+/************************************************************************/
+
 typedef struct LayoutPosition
     {
-    int			lpYPixels;
-    short int		lpPageYTwips;
-    short int		lpPage;
+    unsigned short	lpPageYTwips;
+    unsigned short	lpPage;
+    unsigned char	lpColumn;
+    unsigned char	lpAtTopOfColumn;
     } LayoutPosition;
 
-/************************************************************************/
-/*									*/
-/*  Describes tabs.							*/
-/*									*/
-/*  !)	RS6000/AIX compler only supports unsigned bitfields.		*/
-/*									*/
-/************************************************************************/
-
-typedef enum TabKind
-    {
-    DOCtkLEFT= 0,
-
-    DOCtkRIGHT,
-    DOCtkCENTRE,
-    DOCtkDECIMAL
-    } TabKind;
-
-typedef enum TabLeader
-    {
-    DOCtlNONE= 0,
-
-    DOCtlDOTS,
-    DOCtlHYPH,
-    DOCtlUNDERLINE,
-    DOCtlTHICK,
-    DOCtlEQUAL
-    } TabLeader;
-
-typedef struct TabStop
-    {
-    int			tsTwips;
-    int			tsPixels;
-    unsigned int	tsKind:2;	/*  (enum)	*/
-    unsigned int	tsLeader:3;	/*  (enum)	*/
-    } TabStop;
-
 /************************************************************************/
 /*									*/
 /*  Used to lay out the text on the page.				*/
@@ -150,7 +140,6 @@ typedef struct TextLine
     short int		tlLineHeightTwips;
     short int		tlLineSpacingTwips;
 								/*  5  */
-    short int		tlLineAscentPixels;
     short int		tlHasPageBreak;
     } TextLine;
 
@@ -169,136 +158,6 @@ typedef struct DocumentList
     char *		dlListName;
     } DocumentList;
 
-/************************************************************************/
-/*									*/
-/*  Information on Paragraph Numbering.					*/
-/*									*/
-/************************************************************************/
-
-typedef struct ParagraphNumber
-    {
-    short		pnLevel;
-    short		pnStyle;
-#			define	DOCpnCARD	0
-#			define	DOCpnDEC	1
-#			define	DOCpnUCLTR	2
-#			define	DOCpnUCRM	3
-#			define	DOCpnLCLTR	4
-#			define	DOCpnLCRM	5
-#			define	DOCpnORD	6
-#			define	DOCpnORDT	7
-
-    short		pnUseHangingIndent;
-
-    int			pnStartNumber;
-    int			pnIndentTwips;
-
-    TextAttribute	pnTextAttribute;
-
-    unsigned char	pnTextBefore[32+1];
-    unsigned char	pnTextAfter[32+1];
-
-    } ParagraphNumber;
-
-/************************************************************************/
-/*									*/
-/*  A paragraph in a document.						*/
-/*									*/
-/*  !)	RS6000/AIX compler only supports unsigned bitfields.		*/
-/*									*/
-/************************************************************************/
-
-typedef enum BorderStyle
-    {
-    DOCbsSH= 0,
-    DOCbsDB,
-    DOCbsDOT,
-    DOCbsDASH,
-    DOCbsHAIR,
-    DOCbsDASHSM,
-    DOCbsDASHD,
-    DOCbsDASHDD,
-    DOCbsTRIPLE,
-    DOCbsTNTHSG,
-    DOCbsTHTNSG,
-    DOCbsTNTHTNSG,
-    DOCbsTNTHMG,
-    DOCbsTHTNMG,
-    DOCbsTNTHTNMG,
-    DOCbsTNTHLG,
-    DOCbsTHTNLG,
-    DOCbsTNTHTNLG,
-    DOCbsWAVY,
-    DOCbsWAVYDB,
-    DOCbsDASHDOTSTR,
-    DOCbsEMBOSS,
-    DOCbsENGRAVE,
-
-    DOCbs_COUNT
-    } BorderStyle;
-
-typedef struct BorderProperties
-    {
-    unsigned int	bpColor:8;
-    unsigned int	bpSpacingTwips:16;
-    unsigned int	bpWidthTwips:8;
-    unsigned int	bpStyle:5;		/*  (enum)	*/
-    unsigned int	bpThickness:2;		/*  0, 1 or 2	*/
-    unsigned int	bpIsSet:1;
-    } BorderProperties;
-
-#   define	docCopyBorderProperties( bp1, bp2 ) *(bp1)= *(bp2)
-
-typedef enum ItemAlignment
-    {
-    DOCiaLEFT= 0,
-    DOCiaRIGHT,
-    DOCiaCENTERED,
-    DOCiaJUSTIFIED,
-
-    DOCia_COUNT
-    } ItemAlignment;
-
-typedef enum ObjectKind
-    {
-    DOCokUNKNOWN= 0,
-    DOCokPICTWMETAFILE,
-    DOCokPICTPNGBLIP,
-    DOCokPICTJPEGBLIP,
-    DOCokPICTEMFBLIP,
-    DOCokMACPICT,
-    DOCokOLEOBJECT
-    } ObjectKind;
-
-typedef enum VerticalTextAlignment
-    {
-    DOCvtaTOP= 0,
-    DOCvtaCENTERED,
-    DOCvtaBOTTOM,
-    DOCvtaVERT_LEFT,
-    DOCvtaVERT_RIGHT
-    } VerticalTextAlignment;
-
-typedef enum ShadingPattern
-    {
-    DOCspSOLID= 0,
-
-    DOCspBGHORIZ,
-    DOCspBGVERT,
-    DOCspBGFDIAG,
-    DOCspBGBDIAG,
-    DOCspBGCROSS,
-    DOCspBGDCROSS,
-    DOCspBGDKHORIZ,
-    DOCspBGDKVERT,
-    DOCspBGDKFDIAG,
-    DOCspBGDKBDIAG,
-    DOCspBGDKCROSS,
-    DOCspBGDKDCROSS,
-
-    DOCsp_COUNT
-    } ShadingPattern;
-
 typedef struct CellProperties
     {
     int			cpRightBoundaryTwips;
@@ -362,282 +221,12 @@ typedef struct RowProperties
     unsigned int	rpKeepOnPage:1;
     } RowProperties;
 
-typedef enum SectionBreak
-    {
-    DOCsbkNONE= 0,
-    DOCsbkCOL,
-    DOCsbkPAGE,
-    DOCsbkEVEN,
-    DOCsbkODD,
-
-    DOCsbk_COUNT
-    } SectionBreak;
-
-typedef enum PageNumberStyle
-    {
-    DOCpgnDEC= 0,
-    DOCpgnUCRM,
-    DOCpgnLCRM,
-    DOCpgnUCLTR,
-    DOCpgnLCLTR,
-
-    DOCpgn_COUNT
-    } PageNumberStyle;
-
-typedef struct SectionProperties
-    {
-    DocumentGeometry	spDocumentGeometry;
-
-    int			spColumnCount;
-    int			spColumnSpacingTwips;
-    int			spColumnWidthTwips;
-
-    unsigned int	spHasTitlePage:1;
-    unsigned int	spBreakKind:3;
-    unsigned int	spPageNumberStyle:3;
-    unsigned int	spRestartPageNumbers:1;
-
-    int			spStartPageNumber;
-
-    ParagraphNumber *	spParagraphNumbers;
-    int			spParagraphNumberCount;
-    } SectionProperties;
-
-#   define	SPupdNONE		(0)
-
-#   define	SPupdDOCUMENT_GEOMETRY	DGupdALL
-
-#   define	SPupdTITLEPG		(1<<9)
-
-#   define	SPupdNUMBER_STYLE	(1<<10)
-#   define	SPupdBREAK_KIND		(1<<11)
-#   define	SPupdPAGE_RESTART	(1<<12)
-#   define	SPupdSTART_PAGE		(1<<13)
-
-#   define	SPupdCOLUMN_COUNT	(1<<14)
-#   define	SPupdCOLUMN_SPACING	(1<<15)
-#   define	SPupdCOLUMN_WIDTH	(1<<16)
-#   define	SPupdCOLUMN_PROPS	( SPupdCOLUMN_COUNT	|\
-					  SPupdCOLUMN_SPACING	|\
-					  SPupdCOLUMN_WIDTH	)
-
-#   define	SPupdALL		( SPupdDOCUMENT_GEOMETRY	|\
-					  SPupdTITLEPG			|\
-					  SPupdNUMBER_STYLE		|\
-					  SPupdBREAK_KIND		|\
-					  SPupdPAGE_RESTART		|\
-					  SPupdSTART_PAGE		|\
-					  SPupdCOLUMN_PROPS		)
-
 /************************************************************************/
 /*									*/
-/*  Document Properties.						*/
+/*  Primitives for shapes.						*/
 /*									*/
 /************************************************************************/
 
-#   define	DPfetFOOT_ONLY		0
-#   define	DPfetEND_ONLY		1
-#   define	DPfetFOOT_END_BOTH	2
-
-#   define	DPftnPOS_SECT_END	0
-#   define	DPftnPOS_DOC_END	1
-#   define	DPftnPOS_BELOW_TEXT	2
-#   define	DPftnPOS_PAGE_BOTTOM	3
-#   define	DPftnPOS__COUNT		4
-
-#   define	DPftnRST_CONTINUOUS	0
-#   define	DPftnRST_PER_SECTION	1
-#   define	DPftnRST_PER_PAGE	2
-#   define	DPftnRST__COUNT		3
-
-typedef enum NoteNumberStyle
-    {
-    DPftnNAR= 0,
-    DPftnNALC,
-    DPftnNAUC,
-    DPftnNRLC,
-    DPftnNRUC,
-    DPftnNCHI,
-
-    DPftn_NCOUNT
-    } NoteNumberStyle;
-
-#   define	DPupdNONE		(0)
-
-#   define	DPupdDOCUMENT_GEOMETRY	DGupdALL
-
-#   define	DPupdSTART_PAGE		(1<<9)
-#   define	DPupdFACING_PAGES	(1<<10)
-
-#   define	DPupdFOOTNOTE_POSITION	(1<<11)
-#   define	DPupdFOOTNOTE_RESTART	(1<<12)
-#   define	DPupdFOOTNOTE_STYLE	(1<<13)
-#   define	DPupdFOOTNOTE_STARTNR	(1<<14)
-#   define	DPupdFOOTNOTE_PROPS	( DPupdFOOTNOTE_POSITION |\
-					  DPupdFOOTNOTE_RESTART |\
-					  DPupdFOOTNOTE_STYLE |\
-					  DPupdFOOTNOTE_STARTNR )
-
-#   define	DPupdENDNOTE_POSITION	(1<<15)
-#   define	DPupdENDNOTE_RESTART	(1<<16)
-#   define	DPupdENDNOTE_STYLE	(1<<17)
-#   define	DPupdENDNOTE_STARTNR	(1<<18)
-#   define	DPupdENDNOTE_PROPS	( DPupdENDNOTE_POSITION |\
-					  DPupdENDNOTE_RESTART |\
-					  DPupdENDNOTE_STYLE |\
-					  DPupdENDNOTE_STARTNR )
-
-#   define	DPupdNOTE_PROPS		( DPupdFOOTNOTE_PROPS |\
-					  DPupdENDNOTE_PROPS )
-
-#   define	DPupdDEFTAB		(1<<19)
-#   define	DPupdANSICPG		(1<<20)
-
-#   define	DPupdALL		( DPupdDOCUMENT_GEOMETRY |\
-					  DPupdSTART_PAGE |\
-					  DPupdFACING_PAGES |\
-					  DPupdNOTE_PROPS |\
-					  DPupdDEFTAB |\
-					  DPupdANSICPG )
-
-typedef struct DocumentProperties
-    {
-    int			dpContainsTables;
-    int			dpTabIntervalTwips;
-    int			dpDefaultColor;
-    int			dpDefaultFont;
-    int			dpAnsiCodePage;
-
-    int			dpStartPageNumber;
-    int			dpStartFootnoteNumber;
-    int			dpStartEndnoteNumber;
-
-    unsigned int	dpHasFacingPages:1;
-
-    unsigned int	dpFootEndNoteType:2;
-    unsigned int	dpFootnotePosition:2;
-    unsigned int	dpFootnotesRestart:2;
-    unsigned int	dpFootnoteNumberStyle:3;
-    unsigned int	dpEndnotePosition:2;
-    unsigned int	dpEndnotesRestart:2;
-    unsigned int	dpEndnoteNumberStyle:3;
-
-    DocumentGeometry	dpGeometry;
-    DocumentFontList	dpFontList;
-
-    RGB8Color *		dpColors;
-    int			dpColorCount;
-
-    unsigned char **	dpAuthors;
-    int			dpAuthorCount;
-
-    struct tm		dpCreatim;
-    struct tm		dpRevtim;
-    struct tm		dpPrintim;
-
-    unsigned char *	dpTitle;
-    unsigned char *	dpSubject;
-    unsigned char *	dpKeywords;
-    unsigned char *	dpComment;
-    unsigned char *	dpAuthor;
-    unsigned char *	dpHlinkbase;
-    } DocumentProperties;
-
-#   define	docHasDocumentInfo( dp )		\
-		( (dp)->dpTitle			||	\
-		  (dp)->dpSubject		||	\
-		  (dp)->dpKeywords		||	\
-		  (dp)->dpComment		||	\
-		  (dp)->dpHlinkbase		||	\
-		  (dp)->dpAuthor		||	\
-		  (dp)->dpCreatim.tm_mday != 0	||	\
-		  (dp)->dpRevtim.tm_mday != 0	||	\
-		  (dp)->dpPrintim.tm_mday != 0	)
-
-/************************************************************************/
-/*									*/
-/*  Paragraph properties.						*/
-/*									*/
-/*  Unless explicitly stated, all properties except numbers in pixels	*/
-/*  are properties of the paragraph.					*/
-/*									*/
-/*  1)  Tab stops.							*/
-/*  2)  Indentations.							*/
-/*  3)  White space above/after the paragraph.				*/
-/*  4)  Style.								*/
-/*  5)  Borders.							*/
-/*									*/
-/************************************************************************/
-
-typedef struct ParagraphProperties
-    {
-    /*  1  */
-    TabStop *		ppTabStops;
-    short int		ppTabCount;
-
-    /*  2  */
-    short int		ppFirstIndentTwips;
-    short int		ppLeftIndentTwips;
-    short int		ppRightIndentTwips;
-
-    /*  3  */
-    short int		ppSpaceBeforeTwips;
-    short int		ppSpaceAfterTwips;
-
-    short int		ppLineSpacingTwips;
-
-    /*  4  */
-    short int		ppStyle;
-
-    /*  5  */
-    BorderProperties	ppTopBorder;
-    BorderProperties	ppBottomBorder;
-    BorderProperties	ppLeftBorder;
-    BorderProperties	ppRightBorder;
-    BorderProperties	ppBetweenBorder;
-    BorderProperties	ppBoxBorder;
-    BorderProperties	ppBar;
-
-    unsigned int	ppOutlineLevel:4;
-
-    unsigned int	ppShadingLevel:10;
-    unsigned int	ppShadingPattern:4;		/*  (enum)	*/
-
-    unsigned int	ppAlignment:3;			/*  (enum)	*/
-
-    unsigned int	ppStartsOnNewPage:1;
-    unsigned int	ppInTable:1;
-    unsigned int	ppLineSpacingIsMultiple:1;
-
-    unsigned int	ppKeepOnPage:1;
-    unsigned int	ppKeepWithNext:1;
-    } ParagraphProperties;
-
-/************************************************************************/
-/*									*/
-/*  Masks for updating paragraph properies.				*/
-/*									*/
-/************************************************************************/
-
-#   define	PPupdNONE		0x0000
-
-#   define	PPupdLEFT_INDENT	0x0001
-#   define	PPupdFIRST_INDENT	0x0002
-#   define	PPupdRIGHT_INDENT	0x0004
-#   define	PPupdTAB_STOPS		0x0008
-
-#   define	PPupdNEWPAGE		0x0010
-#   define	PPupdSPACE_BEFORE	0x0020
-#   define	PPupdSPACE_AFTER	0x0040
-#   define	PPupdTOP_BORDER		0x0080
-#   define	PPupdBOTTOM_BORDER	0x0100
-#   define	PPupdALIGNMENT		0x0200
-#   define	PPupdLINE_SPACING	0x0400
-
-#   define	PPupdALL		0x7fff
-
-#   define	PPupdCOLUMNS		0x8000		/*  not real	*/
-
 typedef enum ShapeHorizontalAttachment
     {
     SHPshaPAGE= 0,
@@ -683,161 +272,6 @@ typedef struct ShapeProperties
     unsigned int	spLockAnchor:1;
     } ShapeProperties;
 
-typedef struct ObjectData
-    {
-    unsigned char *	odBytes;
-    int			odSize;
-    } ObjectData;
-
-typedef struct InsertedObject
-    {
-    int			ioKind;		/*  Kind of object.		*/
-    int			ioResultKind;	/*  Kind of object.		*/
-    int			ioTwipsWide;	/*  Width of object.		*/
-    int			ioTwipsHigh;	/*  Height of object.		*/
-    int			ioScaleX;	/*  In %.			*/
-    int			ioScaleY;	/*  In %.			*/
-    int			ioPixelsWide;	/*  Width of object on screen	*/
-    int			ioPixelsHigh;	/*  Height of object on screen	*/
-    int			ioXExtent;	/*  Of metafile picture.	*/
-    int			ioYExtent;	/*  Of metafile picture.	*/
-    int			ioUnitsPerInch;	/*  Only use if > 0		*/
-
-    int			ioTopCropTwips;
-    int			ioBottomCropTwips;
-    int			ioLeftCropTwips;
-    int			ioRightCropTwips;
-
-    int			ioDragWide;	/*  PixelsWide during resize.	*/
-    int			ioDragHigh;	/*  PixelsHigh during resize.	*/
-
-    ObjectData		ioObjectData;
-    ObjectData		ioResultData;
-
-    unsigned char *	ioObjectName;
-    unsigned char *	ioObjectClass;
-    int			ioBliptag;
-
-#   ifdef USE_MOTIF
-    unsigned long	ioPixmap;
-#   endif
-
-#   ifdef USE_GTK
-    void *		ioPixmap;
-#   endif
-
-    void *		ioPrivate;
-    } InsertedObject;
-
-/************************************************************************/
-/*									*/
-/*  Kind of field. Is an index in a descriptive array of field kinds.	*/
-/*									*/
-/*  Fields in the 'Ted' sense are not necessarily fileds in the 'Word'	*/
-/*  sense. Everything inside the text, that can possibly result in	*/
-/*  a piece of document, but is possibly under the control of the	*/
-/*  application is a field in the sense of 'Ted'.			*/
-/*									*/
-/*  FieldKindInformation:						*/
-/*  a)  The (case insensitive) string that identifies the field in the	*/
-/*	field instructions. NOTE that the notion of a field is abused	*/
-/*	by 'Ted' for a few things that are not fields in MS-Word.	*/
-/*  b)  Some things that are a destination in RTF are stored as a field	*/
-/*	in Ted. This is the control word of the destination.		*/
-/*  c)  The level at which this kind of field occurs.			*/
-/*  d)  Tells whether this kind of field is a field in the RTF file.	*/
-/*  e)  Tells whether this kind of field is a destination in the RTF	*/
-/*	file.								*/
-/*  f)  Recalculate the field when it is a TEXT level field.		*/
-/*  g)  When to recalculate fields in the text.				*/
-/*  h)  Is the field result editable such as with a HYPERLINK or	*/
-/*	readonly such as with a PAGEREF?				*/
-/*									*/
-/************************************************************************/
-
-/*  f  */
-typedef int (*CALCULATE_TEXT_FIELD)(
-				int *				pCalculated,
-				int *				pNewSize,
-				char *				target,
-				int				targetSize,
-				struct BufferDocument *		bd,
-				const struct BufferItem *	paraBi,
-				const struct DocumentField *	df );
-
-/*  g  */
-#   define	FIELDdoNOTHING		0
-#   define	FIELDdoNEVER		0
-#   define	FIELDdoDOC_FORMATTED	(1<<0)
-#   define	FIELDdoPAGE_NUMBER	(1<<1)
-#   define	FIELDdoDOC_INFO		(1<<2)
-#   define	FIELDdoDOC_COMPLETE	(1<<3)
-
-typedef struct FieldKindInformation
-    {
-    const char *		fkiLabel;			/*  a	*/
-    const char *		fkiDestinationLabel;		/*  b	*/
-    int				fkiLevel;			/*  c	*/
-    int				fkiIsFieldInRtf;		/*  d	*/
-    int				fkiIsDestinationInRtf;		/*  e	*/
-    CALCULATE_TEXT_FIELD	fkiCalculateText;		/*  f	*/
-    unsigned int		fkiCalculateWhen;		/*  g	*/
-    unsigned int		fkiResultEditable;		/*  h	*/
-    } FieldKindInformation;
-
-typedef enum FieldKind
-    {
-    DOCfkFREE= -1,
-    DOCfkUNKNOWN= 0,
-    DOCfkHYPERLINK,
-    DOCfkXE,
-    DOCfkTC,
-    DOCfkTCN,
-    DOCfkBOOKMARK,
-    DOCfkPAGE,
-    DOCfkPAGEREF,
-    DOCfkNUMPAGES,
-    DOCfkREF,
-
-    DOCfkCREATEDATE,
-    DOCfkSAVEDATE,
-    DOCfkPRINTDATE,
-
-    DOCfkAUTHOR,
-    DOCfkCOMMENTS,
-    DOCfkKEYWORDS,
-    DOCfkSUBJECT,
-    DOCfkTITLE,
-
-    DOCfkFOOTNOTE,
-    DOCfkCHFTN,
-
-    DOCfk_COUNT
-    } FieldKind;
-
-typedef struct DocumentField
-    {
-    int			dfPage;
-    FieldKind		dfKind;
-    ObjectData		dfData;
-    ObjectData		dfInstructions;
-    struct BufferItem *	dfReferencedItem;
-    } DocumentField;
-
-typedef struct FieldInstructionsComponent
-    {
-    int		ficOffset;
-    int		ficSize;
-    } FieldInstructionsComponent;
-
-#   define	DOCmaxBOOKMARK	39
-
-typedef struct DocumentFieldList
-    {
-    DocumentField *	dflFields;
-    int			dflFieldCount;
-    } DocumentFieldList;
-
 typedef struct BufferPara
     {
     unsigned char *		btString;
@@ -872,6 +306,10 @@ typedef struct HeaderFooter
     {
     struct BufferItem *	hfItem;
     int			hfPageUsedFor;
+    int			hfY0UsedTwips;
+    int			hfY1UsedTwips;
+    int			hfY0ReservedTwips;
+    int			hfY1ReservedTwips;
     } HeaderFooter;
 
 typedef struct BufferSect
@@ -1056,6 +494,7 @@ typedef enum ItemLevel
 #	define	biParaBetweenBorder	biParaProperties.ppBetweenBorder
 #	define	biParaBar		biParaProperties.ppBar
 #	define	biParaStyle		biParaProperties.ppStyle
+#	define	biParaWidowControl	biParaProperties.ppWidowControl
 
 #	define	biParaAscentTwips	BIU.biuPara.btAscentTwips
 #	define	biParaDescentTwips	BIU.biuPara.btDescentTwips
@@ -1078,7 +517,8 @@ typedef enum ItemLevel
 
 typedef struct BufferItem
     {
-    ItemLevel			biLevel;
+    short int			biLevel;
+    char			biInHeaderFooter;
     struct BufferItem *		biParent;
     int				biNumberInParent;
     struct BufferItem **	biChildren;
@@ -1119,18 +559,23 @@ typedef struct BufferPosition
     int			bpBaselinePixels;
     } BufferPosition;
 
-typedef struct SelectionScope
-    {
-    int			ssInHeaderFooter;
 #			define	DOCinBODY		0
 #			define	DOCinSECT_HEADER	1
 #			define	DOCinFIRST_HEADER	2
 #			define	DOCinLEFT_HEADER	3
 #			define	DOCinRIGHT_HEADER	4
-#			define	DOCinSECT_FOOTER	(-1)
-#			define	DOCinFIRST_FOOTER	(-2)
-#			define	DOCinLEFT_FOOTER	(-3)
-#			define	DOCinRIGHT_FOOTER	(-4)
+#			define	DOCinSECT_FOOTER	5
+#			define	DOCinFIRST_FOOTER	6
+#			define	DOCinLEFT_FOOTER	7
+#			define	DOCinRIGHT_FOOTER	8
+#			define	DOCinFOOTNOTE		9
+			/* ?
+#			define	DOCinENDNOTE		10
+			*/
+
+typedef struct SelectionScope
+    {
+    int			ssInHeaderFooter;
     int			ssHeaderFooterSection;
     int			ssHeaderFooterPage;
     } SelectionScope;
@@ -1169,25 +614,6 @@ typedef struct TableRectangle
 #   define	docSelectionInsideRow(bs) \
 		docPositionsInsideRow(&(bs)->bsBegin,&(bs)->bsEnd)
 
-/************************************************************************/
-/*									*/
-/*  Styles in a document.						*/
-/*									*/
-/*  dsBusy is used to prevent recursion when styles are expanded.	*/
-/*									*/
-/************************************************************************/
-
-typedef struct DocumentStyle
-    {
-    int			dsStyleNumber;
-    int			dsBasedOn;
-    int			dsIsCharacterStyle;
-    int			dsBusy;
-    ParagraphProperties	dsParagraphProperties;
-    TextAttribute	dsTextAttribute;
-    char *		dsName;
-    } DocumentStyle;
-
 /************************************************************************/
 /*									*/
 /*  The document as a whole.						*/
@@ -1199,17 +625,10 @@ typedef struct BufferDocument
     BufferItem		bdItem;
     DocumentProperties	bdProperties;
 
-    DocumentStyle *	bdStyles;
-    int			bdStyleCount;
-
+    DocumentStyleSheet	bdStyleSheet;
     DocumentFieldList	bdFieldList;
     } BufferDocument;
 
-typedef void (*DOC_CLOSE_OBJECT)(	BufferDocument *	bd,
-					BufferItem *		bi,
-					TextParticule *		tp,
-					void *			voiddis );
-
 /************************************************************************/
 /*									*/
 /*  Statistics about a document. Used in the 'Document Properties'	*/
@@ -1226,15 +645,6 @@ typedef struct DocumentStatistics
     int		dsCharacterCount;
     } DocumentStatistics;
 
-/************************************************************************/
-/*									*/
-/*  Field Kind Information.						*/
-/*									*/
-/************************************************************************/
-
-extern const FieldKindInformation	DOC_FieldKinds[];
-extern const int			DOC_FieldKindCount;
-
 /************************************************************************/
 /*									*/
 /*  Routine declarations.						*/
@@ -1242,9 +652,12 @@ extern const int			DOC_FieldKindCount;
 /************************************************************************/
 
 extern BufferDocument *	docPlainReadFile( SimpleInputStream *	sis,
+				    int *			pMxL,
 				    const DocumentGeometry *	dg );
 
-extern BufferDocument *	docRtfReadFile(		SimpleInputStream *	sis );
+extern BufferDocument *	docRtfReadFile(	SimpleInputStream *	sis,
+					int			defAnsigpg );
+
 extern BufferDocument *	docLeafReadFile(	SimpleInputStream *	sis );
 extern BufferDocument *	docWordReadFile(	SimpleInputStream *	sis );
 
@@ -1271,6 +684,8 @@ extern int docRtfSaveSelectionAsLink(	SimpleOutputStream *	sos,
 
 extern int docHtmlSaveDocument(		SimpleOutputStream *	sos,
 					const BufferDocument *	bd,
+					int			asMimeAggr,
+					const char *		mimeBoundary,
 					const char *		filename );
 
 extern int docPlainSaveDocument(	SimpleOutputStream *	sos,
@@ -1304,26 +719,11 @@ extern void docFreeDocument( BufferDocument *	bd );
 
 extern void docInitDocument(	BufferDocument *	bd	);
 
-extern void docInitParagraphProperties(	ParagraphProperties *	pp );
-extern void docInitDocumentProperties(	DocumentProperties *	dp );
-extern void docInitSectionProperties(	SectionProperties *	sp );
-
 extern void docInitTabStop(	TabStop *	ts );
 
-extern void docCleanParagraphProperties(	ParagraphProperties *	pp );
-extern void docCleanSectionProperties(		SectionProperties *	sp );
-extern void docCleanDocumentProperties(		DocumentProperties *	dp );
-
-extern int docCopyParagraphProperties(	ParagraphProperties *		to,
-					const ParagraphProperties *	from );
-extern int docCopySectionProperties(	SectionProperties *		to,
-					const SectionProperties *	from );
 extern int docCopyRowProperties(	RowProperties *			to,
 					const RowProperties *		from );
 
-extern int docCopyParagraphRuler(	ParagraphProperties *		to,
-					const ParagraphProperties *	from );
-
 extern BufferItem * docInsertItem(	const BufferDocument *	bd,
 					BufferItem *		parent,
 					int			n,
@@ -1391,7 +791,7 @@ extern int docParaReplaceText(	BufferDocument *	bd,
 				const unsigned char *	addedText,
 				unsigned int		addedLength,
 				TextAttribute		addedAttribute,
-				void *			voiddisplay,
+				void *			voidadd,
 				DOC_CLOSE_OBJECT	closeObject );
 
 extern int docSplitParaItem(	BufferDocument *	bd,
@@ -1497,17 +897,19 @@ extern int docReplaceSelection(	BufferDocument *	bd,
 				const unsigned char *	addedText,
 				int			addedLength,
 				TextAttribute		addedAttribute,
-				void *			voiddisplay,
+				void *			voidadd,
 				DOC_CLOSE_OBJECT	closeObject );
 
 extern void docInitItem(	BufferItem *		bi,
 				BufferItem *		parent,
 				const BufferDocument *	bd,
 				int			numberInParent,
-				int			level );
+				int			level,
+				int			inHeaderFooter );
 
 extern const char * docKindStr(		int			kind );
 extern const char * docLevelStr(	int			level );
+extern const char * docExternalKindStr(	int			level );
 extern const char * docFieldKindStr(	int			kind );
 extern const char * docAttributeStr(	TextAttribute		ta );
 
@@ -1521,9 +923,6 @@ extern int docObjectSetData(	InsertedObject *	io,
 				const unsigned char *	bytes,
 				int			size );
 
-extern int docCopyObjectData(	ObjectData *		odTo,
-				const ObjectData *	odFrom );
-
 extern int docSetResultData(	InsertedObject *	io,
 				const unsigned char *	bytes,
 				int			size );
@@ -1580,8 +979,6 @@ extern int docInsertRowColumn(	RowProperties *			rp,
 				int				n,
 				const CellProperties *	cp );
 
-extern void docInitBorderProperties(	BorderProperties *	bp );
-
 extern int docAlignedColumns(	const RowProperties *	rp1,
 				const RowProperties *	rp2 );
 
@@ -1595,11 +992,6 @@ extern int docDelimitTable(	BufferItem *		paraBi,
 				int *			pRow,
 				int *			pRow1 );
 
-extern void docParagraphFrameTwips(	FormattingFrame *	ff,
-					int			bottom,
-					int			stripHigh,
-					const BufferItem *	bi );
-
 extern void docInitShapeProperties(	ShapeProperties *	sp );
 
 
@@ -1610,28 +1002,6 @@ extern int docGetBookmarkForPosition(	const BufferDocument *	bd,
 					const char **		pMarkName,
 					int *			pMarkSize );
 
-extern DocumentField *	docFindBookmarkField( const DocumentFieldList * dfl,
-					const char *		markName,
-					int			markSize );
-
-extern int docFieldGetHyperlink(		const DocumentField *	df,
-					const char **		pFileName,
-					int *			pFileSize,
-					const char **		pMarkName,
-					int *			pMarkSize );
-
-extern int docFieldGetBookmark(		const DocumentField *	df,
-					const char **		pMarkName,
-					int *			pMarkSize );
-
-extern int docFieldGetPageref(		const DocumentField *	df,
-					const char **		pMarkName,
-					int *			pMarkSize );
-
-extern int docFieldGetRef(		const DocumentField *	df,
-					const char **		pMarkName,
-					int *			pMarkSize );
-
 extern int docGetHyperlinkForPosition(	const BufferDocument *	bd,
 					const BufferPosition *	bp,
 					int *			pStartPart,
@@ -1647,6 +1017,32 @@ extern int docFieldSetHyperlink(	DocumentField *		df,
 					const unsigned char *	mark,
 					int			markSize );
 
+extern DocumentField *	docFindBookmarkField( const DocumentFieldList * dfl,
+					const char *		markName,
+					int			markSize );
+
+extern int docFieldGetHyperlink(	const DocumentField *	df,
+					const char **		pFileName,
+					int *			pFileSize,
+					const char **		pMarkName,
+					int *			pMarkSize );
+
+extern int docFieldGetBookmark(		const DocumentField *	df,
+					const char **		pMarkName,
+					int *			pMarkSize );
+
+extern int docFieldGetPageref(		const DocumentField *	df,
+					const char **		pMarkName,
+					int *			pMarkSize );
+
+extern int docFieldGetIncludePicture(	const DocumentField *	df,
+					const char **		pFileName,
+					int *			pFileSize );
+
+extern int docFieldGetRef(		const DocumentField *	df,
+					const char **		pMarkName,
+					int *			pMarkSize );
+
 extern int docFieldSetBookmark(		DocumentField *		df,
 					const unsigned char *	mark,
 					int			markSize );
@@ -1694,24 +1090,15 @@ extern int docCopyParticules(	BufferDocument *	bdTo,
 				int *			pCharactersCopied,
 				const char *		refFileName );
 
-extern int docEqualBorder(	const BorderProperties *	bp1,
-				const BorderProperties *	bp2 );
-
 extern void docInitShape(	DrawingShape *	ds );
 extern void docCleanShape(	DrawingShape *	ds );
 
 extern DrawingShape * docClaimShape(	int *			pNr,
 					BufferItem *		bi );
 
-extern void docInitStyle(	DocumentStyle *	ds );
-extern void docCleanStyle(	DocumentStyle *	ds );
-
-extern DocumentStyle * docInsertStyle(	BufferDocument *	bd,
-					int			n,
-					const char *		name );
-
 extern int docParticuleInField(		BufferItem *	bi,
 					int		part );
+
 extern int docSaveParticules(	BufferItem *		bi,
 				TextAttribute		ta,
 				const unsigned char	inputMapping[256],
@@ -1722,11 +1109,6 @@ extern int docSaveSpecialParticule(	BufferItem *		bi,
 					TextAttribute		ta,
 					int			kind );
 
-extern unsigned int docParaPropertyDifference(
-				const ParagraphProperties *	pp1,
-				const ParagraphProperties *	pp2,
-				unsigned int			updMask );
-
 extern void docLogRectangle(	const char *			label,
 				const DocumentRectangle *	dr );
 
@@ -1747,16 +1129,6 @@ extern void docInitDocumentStatistics(		DocumentStatistics *	ds );
 extern void docCollectDocumentStatistics(	DocumentStatistics *	ds,
 						const BufferDocument *	bd );
 
-extern int docUpdParaProperties( unsigned int *		pPpChgPask,
-				BufferItem *		paraBi,
-				unsigned int		ppUpdMask,
-				const ParagraphProperties *	ppNew );
-
-extern int docUpdSectProperties( unsigned int *			pSpChgMask,
-				SectionProperties *		to,
-				unsigned int			spUpdMask,
-				const SectionProperties *	from );
-
 extern int docWhatPageHeader(	HeaderFooter **			pHf,
 				BufferItem *			sectBi,
 				int				page,
@@ -1799,35 +1171,17 @@ extern void docFieldRefreshFlags(	BufferItem *		bi,
 extern void docAdaptBookmarkName(	int *		pLength,
 					char *		mark );
 
-extern int docParaSaveTabs(	ParagraphProperties *		to,
-				const ParagraphProperties *	from );
-
 extern void docInitDocumentList(	DocumentList *	dl );
 extern void docCleanDocumentList(	DocumentList *	dl );
 
-extern int docUpdDocProperties(	unsigned int *			pChgPask,
-				DocumentProperties *		dp,
-				unsigned int			updMask,
-				const DocumentProperties *	dpNew );
 
-extern int docCopyDocumentProperties(	DocumentProperties *		to,
-					const DocumentProperties *	from );
-
-extern int docAddToObjectData(		ObjectData *		od,
-					const unsigned char *	bytes,
-					int			size );
-
-extern int docSetObjectData(		ObjectData *		od,
-					const unsigned char *	bytes,
-					int			size );
-
-extern void docInitObjectData(		ObjectData *	od );
-extern void docCleanObjectData(		ObjectData *	od );
-
-extern int docRecalculateTextFields(	int *			pChanged,
+extern int docRecalculateTextLevelFields(
+					int *			pChanged,
 					BufferDocument *	bd,
 					BufferItem *		bi,
-					unsigned int		whenMask );
+					unsigned int		whenMask,
+					void *			voidadd,
+					DOC_CLOSE_OBJECT	closeObject );
 
 extern int docRedivideStringInParticules(	BufferItem *	bi,
 						int		strOff,
@@ -1844,7 +1198,7 @@ extern int docBalanceFieldSelection(	int *			pBeginMoved,
 
 extern void docInitSelectionScope(	SelectionScope *	ss );
 
-extern int docSplitFieldInstructions(	const ObjectData *	od,
+extern int docSplitFieldInstructions(	const MemoryBuffer *	mb,
 				FieldInstructionsComponent *	fic,
 				int				count );
 
@@ -1859,8 +1213,10 @@ extern void docInitHeaderFooter(	HeaderFooter *		hf );
 extern void docCleanHeaderFooter(	BufferDocument *	bd,
 					HeaderFooter *		hf );
 
-extern BufferItem * docMakeExternalItem( BufferDocument *		bd,
-					const SectionProperties *	sp );
+extern BufferItem * docMakeExternalItem(
+				BufferDocument *		bd,
+				const SectionProperties *	sp,
+				int				inHdFt );
 
 extern BufferItem * docInsertEmptyParagraph(	BufferDocument *	bd,
 						BufferItem *		bi,
@@ -1908,4 +1264,84 @@ extern void docUnionParaSelections(	BufferSelection *	bs,
 
 extern int *	docAllocateFieldMap(	const BufferDocument *	bdFrom );
 
+extern int docCalculatePagerefFieldString(
+					int *			pCalculated,
+					int *			pNewSize,
+					unsigned char *		target,
+					int			targetSize,
+					BufferDocument *	bd,
+					const BufferItem *	paraBi,
+					const DocumentField *	dfRef );
+
+extern int docCalculatePageFieldString(	int *			pCalculated,
+					int *			pNewSize,
+					unsigned char *		target,
+					int			targetSize,
+					BufferDocument *	bd,
+					const BufferItem *	paraBi,
+					const DocumentField *	df );
+
+extern int docCalculateNumpagesFieldString(
+					int *			pCalculated,
+					int *			pNewSize,
+					unsigned char *		target,
+					int			targetSize,
+					BufferDocument *	bd,
+					const BufferItem *	paraBi,
+					const DocumentField *	df );
+
+extern int docCalculateDocDateFieldString(
+					int *			pCalculated,
+					int *			pNewSize,
+					unsigned char *		target,
+					int			targetSize,
+					BufferDocument *	bd,
+					const BufferItem *	paraBi,
+					const DocumentField *	df );
+
+extern int docCalculateDocStringFieldString(
+					int *			pCalculated,
+					int *			pNewSize,
+					unsigned char *		target,
+					int			targetSize,
+					BufferDocument *	bd,
+					const BufferItem *	paraBi,
+					const DocumentField *	df );
+
+extern int docCalculateFilenameFieldString(
+					int *			pCalculated,
+					int *			pNewSize,
+					unsigned char *		target,
+					int			targetSize,
+					BufferDocument *	bd,
+					const BufferItem *	paraBi,
+					const DocumentField *	df );
+
+extern int docCalculateRefFieldString(	int *			pCalculated,
+					int *			pNewSize,
+					unsigned char *		target,
+					int			targetSize,
+					BufferDocument *	bd,
+					const BufferItem *	paraBi,
+					const DocumentField *	dfRef );
+
+extern int docRecalculateIncludePictureField(
+				int *				pCalculated,
+				BufferDocument *		bd,
+				int *				pPartShift,
+				int *				pStroffShift,
+				BufferItem *			bi,
+				int				part,
+				int				partCount,
+				DocumentField *			df,
+				void *				voidadd,
+				DOC_CLOSE_OBJECT		closeObject );
+
+extern int docParaStringReplace(	int *			pStroffShift,
+					BufferItem *		bi,
+					int			stroffBegin,
+					int			stroffEnd,
+					const unsigned char *	addedString,
+					int			addedStrlen );
+
 #   endif
