@@ -27,8 +27,7 @@ typedef struct CellDrawingProgress
     int			cdpX1Pixels;
     int			cdpX11Pixels;	/*  Includes right border	*/
 
-    FormattingFrame	cdpFormattingFrameTwips;
-    FormattingFrame	cdpFormattingFramePixels;
+    FormattingFrame	cdpFormattingFrame;
     } CellDrawingProgress;
 
 /************************************************************************/
@@ -60,10 +59,10 @@ void docInitDrawingContext(	DrawingContext *	dc )
     dc->dcDrawCellRight= (DRAW_CELL_RIGHT)0;
     dc->dcFinishPage= (FINISH_PAGE)0;
     dc->dcStartPage= (START_PAGE)0;
-    dc->dcParaSidesPixels= (PARA_SIDES_PIXELS)0;
+    dc->dcParaFramePixels= (PARA_SIDES_PIXELS)0;
 
-    dc->dcLayoutHeader= (LAYOUT_EXTERNAL)0;
-    dc->dcLayoutFooter= (LAYOUT_EXTERNAL)0;
+    dc->dcLayoutExternal= (LAYOUT_EXTERNAL)0;
+    dc->dcCloseObject= (DOC_CLOSE_OBJECT)0;
 
     return;
     }
@@ -77,8 +76,7 @@ void docInitDrawingContext(	DrawingContext *	dc )
 static int docDrawOneLine(	void *				through,
 				const BufferItem *		bi,
 				int				part,
-				const FormattingFrame *		ffTwips,
-				const FormattingFrame *		ffPixels,
+				const FormattingFrame *		ff,
 				int				line,
 				DrawingContext *		dc )
     {
@@ -91,14 +89,13 @@ static int docDrawOneLine(	void *				through,
 	  dc->dcDrawParaTop			)
 	{
 	if  ( (*dc->dcDrawParaTop)( bi, bi->biParaBorderAboveParagraph,
-					ffTwips, ffPixels, through, dc ) )
+							ff, through, dc ) )
 	    { LDEB(1); return -1;	}
 	}
 
     if  ( dc->dcDrawTextLine )
 	{
-	accepted= (*dc->dcDrawTextLine)( bi, line,
-				    ffTwips, ffPixels, through, dc );
+	accepted= (*dc->dcDrawTextLine)( bi, line, ff, through, dc );
 	}
     else{ accepted= tl->tlParticuleCount;	}
 
@@ -112,7 +109,7 @@ static int docDrawOneLine(	void *				through,
 	    {
 	    if  ( (*dc->dcDrawParaBottom)( bi,
 					bi->biParaBorderBelowParagraph,
-					ffTwips, ffPixels, through, dc ) )
+							ff, through, dc ) )
 		{ LDEB(1); return -1;	}
 	    }
 	}
@@ -123,8 +120,7 @@ static int docDrawOneLine(	void *				through,
 static int docDrawTextLines(	void *				through,
 				const BufferItem *		bi,
 				int				page,
-				const FormattingFrame *		ffTwips,
-				const FormattingFrame *		ffPixels,
+				const FormattingFrame *		ff,
 				int *				pLinesDone,
 				DrawingContext *		dc )
     {
@@ -148,12 +144,10 @@ static int docDrawTextLines(	void *				through,
 	if  ( dc->dcClipRect )
 	    {
 	    AppDrawingData *		add= dc->dcDrawingData;
-	    double			xfac= add->addMagnifiedPixelsPerTwip;
 
-	    int	y1= tl->tlTopPosition.lpYPixels+
-				TWIPStoPIXELS( xfac, tl->tlLineSpacingTwips );
+	    int	y1= TL_BELOW_PIXELS( add, tl );
 
-	    if  ( tl->tlTopPosition.lpYPixels > dc->dcClipRect->drY1 )
+	    if  ( TL_TOP_PIXELS( add, tl ) > dc->dcClipRect->drY1 )
 		{ pastSelectionEnd= 1;	}
 	    if  ( y1 < dc->dcClipRect->drY0 )
 		{ beforeSelectionBegin= 1;	}
@@ -192,7 +186,7 @@ static int docDrawTextLines(	void *				through,
 	    { accepted= tl->tlParticuleCount;		}
 	else{
 	    accepted= docDrawOneLine( through, bi, tl->tlFirstParticule,
-						ffTwips, ffPixels, line, dc );
+								ff, line, dc );
 	    }
 
 	if  ( accepted < 1 )
@@ -212,40 +206,36 @@ static int docDrawTextLines(	void *				through,
 /*									*/
 /************************************************************************/
 
-static int docDrawHeaderItem(	HeaderFooter *			hf,
-				int				pageNumber,
-				int				inHeaderFooter,
-				const BufferItem *		sectBi,
-				DrawingContext *		dc,
-				void *				through )
+static int docDrawHeaderFooter(	BufferItem *		sectBi,
+				void *			through,
+				HeaderFooter *		hf,
+				DrawingContext *	dc,
+				int			page )
     {
-    if  ( pageNumber != hf->hfPageUsedFor				&&
-	  docLayoutHeaderFooter( hf, pageNumber, inHeaderFooter,
-					    dc->dcDocument, sectBi,
+    if  ( dc->dcClipRect )
+	{
+	DocumentRectangle		drExtern;
+	AppDrawingData *		add= dc->dcDrawingData;
+	double				xfac= add->addMagnifiedPixelsPerTwip;
+	int				pageY0;
+
+	drExtern.drX0= add->addBackRect.drX0;
+	drExtern.drX1= add->addBackRect.drX1;
+
+	pageY0= page* add->addPageStepPixels;
+	drExtern.drY0= pageY0+ TWIPStoPIXELS( xfac, hf->hfY0UsedTwips );
+	drExtern.drY1= pageY0+ TWIPStoPIXELS( xfac, hf->hfY1UsedTwips );
+
+	if  ( ! docIntersectRectangle( &drExtern, &drExtern, dc->dcClipRect ) )
+	    { return 0;	}
+	}
+
+    if  ( page != hf->hfPageUsedFor					&&
+	  docLayoutHeaderFooter( hf, page, dc->dcDocument, sectBi,
 					    dc->dcDrawingData,
-					    dc->dcLayoutHeader,
-					    (DocumentRectangle *)0 )	)
-	{ LDEB(1); return -1;	}
-
-    if  ( docDrawItem( hf->hfItem, through, dc ) )
-	{ LDEB(1); return -1;	}
-
-    return 0;
-    }
-
-static int docDrawFooterItem(	HeaderFooter *			hf,
-				int				pageNumber,
-				int				inHeaderFooter,
-				const BufferItem *		sectBi,
-				DrawingContext *		dc,
-				void *				through )
-    {
-    if  ( pageNumber != hf->hfPageUsedFor				&&
-	  docLayoutHeaderFooter( hf, pageNumber, inHeaderFooter,
-					    dc->dcDocument, sectBi,
-					    dc->dcDrawingData,
-					    dc->dcLayoutFooter,
-					    (DocumentRectangle *)0 )	)
+					    dc->dcLayoutExternal,
+					    (DocumentRectangle *)0,
+					    dc->dcCloseObject )		)
 	{ LDEB(1); return -1;	}
 
     if  ( docDrawItem( hf->hfItem, through, dc ) )
@@ -264,33 +254,13 @@ int docDrawPageHeader(	BufferItem *			sectBi,
     BufferDocument *		bd= dc->dcDocument;
     DocumentProperties *	dp= &(bd->bdProperties);
 
-    if  ( dc->dcClipRect )
-	{
-	DocumentRectangle		drHeader;
-	AppDrawingData *		add= dc->dcDrawingData;
-	double				xfac= add->addMagnifiedPixelsPerTwip;
-	const SectionProperties *	sp= &(sectBi->biSectProperties);
-	const DocumentGeometry *	dg= &(sp->spDocumentGeometry);
-	int				pageY0;
-
-	drHeader.drX0= add->addBackRect.drX0;
-	drHeader.drX1= add->addBackRect.drX1;
-
-	pageY0= page* add->addPageStepPixels;
-	drHeader.drY0= pageY0;
-	drHeader.drY1= pageY0+ TWIPStoPIXELS( xfac, dg->dgTopMarginTwips );
-
-	if  ( ! docIntersectRectangle( &drHeader, &drHeader, dc->dcClipRect ) )
-	    { return 0;	}
-	}
-
     inHeaderFooter= docWhatPageHeader( &hf, sectBi, page, dp );
 
-    if  ( hf								&&
-	  hf->hfItem							&&
-	  docDrawHeaderItem( hf, page, inHeaderFooter,
-					    sectBi, dc, through )	)
-	{ XLDEB(hf,page); return -1;	}
+    if  ( ! hf || ! hf->hfItem )
+	{ return 0;	}
+
+    if  ( docDrawHeaderFooter( sectBi, through, hf, dc, page ) )
+	{ LDEB(page); return 1;	}
 
     return 0;
     }
@@ -305,34 +275,13 @@ int docDrawPageFooter(	BufferItem *			sectBi,
     BufferDocument *		bd= dc->dcDocument;
     DocumentProperties *	dp= &(bd->bdProperties);
 
-    if  ( dc->dcClipRect )
-	{
-	DocumentRectangle		drFooter;
-	AppDrawingData *		add= dc->dcDrawingData;
-	double				xfac= add->addMagnifiedPixelsPerTwip;
-	const SectionProperties *	sp= &(sectBi->biSectProperties);
-	const DocumentGeometry *	dg= &(sp->spDocumentGeometry);
-	int				pageY0;
-
-	drFooter.drX0= add->addBackRect.drX0;
-	drFooter.drX1= add->addBackRect.drX1;
-
-	pageY0= page* add->addPageStepPixels;
-	drFooter.drY0= pageY0+ TWIPStoPIXELS( xfac,
-			    dg->dgPageHighTwips- dg->dgBottomMarginTwips );
-	drFooter.drY1= pageY0+ TWIPStoPIXELS( xfac, dg->dgPageHighTwips );
-
-	if  ( ! docIntersectRectangle( &drFooter, &drFooter, dc->dcClipRect ) )
-	    { return 0;	}
-	}
-
     inHeaderFooter= docWhatPageFooter( &hf, sectBi, page, dp );
 
-    if  ( hf								&&
-	  hf->hfItem							&&
-	  docDrawFooterItem( hf, page, inHeaderFooter,
-					    sectBi, dc, through )	)
-	{ XLDEB(hf,page); return -1;	}
+    if  ( ! hf || ! hf->hfItem )
+	{ return 0;	}
+
+    if  ( docDrawHeaderFooter( sectBi, through, hf, dc, page ) )
+	{ LDEB(page); return 1;	}
 
     return 0;
     }
@@ -406,24 +355,28 @@ static int docDrawParaItem(	BufferItem *			paraBi,
 				void *				through,
 				DrawingContext *		dc )
     {
-    FormattingFrame		ffTwips;
-    FormattingFrame		ffPixels;
+    FormattingFrame		ff;
     int				page;
     int				line= 0;
 
     int				bottomTwips= -1;
     int				stripHigh= -1;
 
-    docParagraphFrameTwips( &ffTwips, bottomTwips, stripHigh, paraBi );
-
-    if  ( dc->dcParaSidesPixels )
-	{ (*dc->dcParaSidesPixels) ( &ffPixels, dc->dcDrawingData, paraBi ); }
+    BufferDocument *		bd= dc->dcDocument;
+    const DocumentProperties *	dp= &(bd->bdProperties);
 
     page= paraBi->biTopPosition.lpPage;
+
+    docParagraphFrameTwips( &ff, bottomTwips, stripHigh, paraBi, dp,
+					    paraBi->biTopPosition.lpPage,
+					    paraBi->biTopPosition.lpColumn );
+
+    if  ( dc->dcParaFramePixels )
+	{ (*dc->dcParaFramePixels) ( &ff, dc->dcDrawingData, paraBi ); }
+
     while( line < paraBi->biParaLineCount )
 	{
-	if  ( docDrawTextLines( through, paraBi, page,
-					    &ffTwips, &ffPixels, &line, dc ) )
+	if  ( docDrawTextLines( through, paraBi, page, &ff, &line, dc ) )
 	    { LDEB(page); return -1;	}
 
 	/*  1  */
@@ -443,89 +396,11 @@ static int docDrawParaItem(	BufferItem *			paraBi,
     return 0;
     }
 
-static int psAdvanceCell(	void *				through,
-				int *				pAdvanced,
-				BufferItem *			cellBi,
-				int				page,
-				CellDrawingProgress *		cdp,
-				DrawingContext *		dc,
-				LayoutPosition *		lpBelow )
-				
-    {
-    int				bottomTwips= -1;
-    int				stripHigh= -1;
-
-    int				advanced= 0;
-
-    LayoutPosition		lp;
-
-    while( cdp->cdpPara < cellBi->biChildCount )
-	{
-	BufferItem *		paraBi= cellBi->biChildren[cdp->cdpPara];
-	int			linesDone= cdp->cdpLinesDone;
-
-	if  ( cdp->cdpLinesDone == 0 )
-	    {
-	    docParagraphFrameTwips( &(cdp->cdpFormattingFrameTwips),
-					    bottomTwips, stripHigh, paraBi );
-
-	    if  ( dc->dcParaSidesPixels )
-		{
-		(*dc->dcParaSidesPixels) ( &(cdp->cdpFormattingFramePixels),
-						dc->dcDrawingData, paraBi );
-		}
-	    }
-
-	if  ( docDrawTextLines( through, paraBi, page,
-					&(cdp->cdpFormattingFrameTwips),
-					&(cdp->cdpFormattingFramePixels),
-					&linesDone, dc ) )
-	    { LDEB(page); return -1;	}
-
-	if  ( linesDone > cdp->cdpLinesDone )
-	    { cdp->cdpAdvanced= advanced= 1;	}
-
-	cdp->cdpLinesDone= linesDone;
-
-	if  ( cdp->cdpLinesDone < paraBi->biParaLineCount )
-	    {
-	    if  ( cdp->cdpLinesDone > 0 )
-		{
-		AppDrawingData *	add= dc->dcDrawingData;
-		double			xfac= add->addMagnifiedPixelsPerTwip;
-		TextLine *		tl;
-
-		tl= paraBi->biParaLines+ cdp->cdpLinesDone- 1;
-
-		lp.lpPage= tl->tlTopPosition.lpPage;
-		lp.lpPageYTwips= tl->tlTopPosition.lpPageYTwips+
-						    tl->tlLineSpacingTwips;
-		lp.lpYPixels= tl->tlTopPosition.lpYPixels+
-			    TWIPStoPIXELS( xfac, tl->tlLineSpacingTwips );
-		}
-	    else{ lp= paraBi->biTopPosition;	}
-	    }
-	else{ lp= paraBi->biBelowPosition; }
-
-	if  ( lpBelow->lpPage < lp.lpPage )
-	    { lpBelow->lpPage=  lp.lpPage; }
-	if  ( lpBelow->lpPageYTwips < lp.lpPageYTwips )
-	    { lpBelow->lpPageYTwips=  lp.lpPageYTwips;	}
-	if  ( lpBelow->lpYPixels < lp.lpYPixels )
-	    { lpBelow->lpYPixels=  lp.lpYPixels;	}
-
-	if  ( cdp->cdpLinesDone < paraBi->biParaLineCount )
-	    { break;	}
-
-	cdp->cdpLinesDone= 0;
-	cdp->cdpPara++;
-	}
-
-    if  ( advanced )
-	{ *pAdvanced= 1;	}
-
-    return 0;
-    }
+/************************************************************************/
+/*									*/
+/*  Draw part of the grid originating from a table row.			*/
+/*									*/
+/************************************************************************/
 
 static int docDrawCellSides(	const BufferItem *		rowBi,
 				const CellDrawingProgress *	cdp,
@@ -592,32 +467,34 @@ static int docDrawCellSides(	const BufferItem *		rowBi,
 
 	if  ( dc->dcDrawTableGrid )
 	    {
-	    if  ( ! cp->cpTopBorder.bpIsSet				&&
-		  ( ! prevRow || ! prevCp->cpBottomBorder.bpIsSet )	)
+	    if  ( cp->cpTopBorder.bpStyle == DOCbsNONE			&&
+		  ( ! prevRow					||
+		    prevCp->cpBottomBorder.bpStyle == DOCbsNONE	)	)
 		{ topAsGrid= 1;	}
 
-	    if  ( col == 0 && ! cp->cpLeftBorder.bpIsSet )
+	    if  ( col == 0 && cp->cpLeftBorder.bpStyle == DOCbsNONE )
 		{ leftAsGrid= 1;	}
 
 	    if  ( col == rowBi->biChildCount- 1 )
 		{
-		if  ( ! cp->cpRightBorder.bpIsSet )
+		if  ( cp->cpRightBorder.bpStyle == DOCbsNONE )
 		    { rightAsGrid= 1;	}
 		}
 	    else{
-		if  ( ! cp->cpRightBorder.bpIsSet	&&
-		      ! cp[1].cpLeftBorder.bpIsSet	)
+		if  ( cp->cpRightBorder.bpStyle == DOCbsNONE	&&
+		      cp[1].cpLeftBorder.bpStyle == DOCbsNONE	)
 		    { rightAsGrid= 1;	}
 		}
 
-	    if  ( ! cp->cpBottomBorder.bpIsSet				&&
-		  ( ! nextRow || ! nextCp->cpTopBorder.bpIsSet )	)
+	    if  ( cp->cpBottomBorder.bpStyle == DOCbsNONE		&&
+		  ( ! nextRow					||
+		    nextCp->cpTopBorder.bpStyle == DOCbsNONE	)	)
 		{ bottomAsGrid= 1;	}
 	    }
 
 	if  ( drawTop							&&
 	      dc->dcDrawCellTop						&&
-	      ( cp->cpTopBorder.bpIsSet || topAsGrid )			)
+	      ( cp->cpTopBorder.bpStyle != DOCbsNONE || topAsGrid )	)
 	    {
 	    if  ( (*dc->dcDrawCellTop)(		&(cp->cpTopBorder),
 						topAsGrid,
@@ -632,7 +509,7 @@ static int docDrawCellSides(	const BufferItem *		rowBi,
 
 	if  ( drawLeft							&&
 	      dc->dcDrawCellLeft					&&
-	      ( cp->cpLeftBorder.bpIsSet || leftAsGrid )		)
+	      ( cp->cpLeftBorder.bpStyle != DOCbsNONE || leftAsGrid )	)
 	    {
 	    if  ( (*dc->dcDrawCellLeft)(	&(cp->cpLeftBorder),
 						leftAsGrid,
@@ -645,7 +522,7 @@ static int docDrawCellSides(	const BufferItem *		rowBi,
 
 	if  ( drawRight							&&
 	      dc->dcDrawCellRight					&&
-	      ( cp->cpRightBorder.bpIsSet || rightAsGrid )		)
+	      ( cp->cpRightBorder.bpStyle != DOCbsNONE || rightAsGrid )	)
 	    {
 	    if  ( (*dc->dcDrawCellRight)(	&(cp->cpRightBorder),
 						rightAsGrid,
@@ -658,7 +535,7 @@ static int docDrawCellSides(	const BufferItem *		rowBi,
 
 	if  ( drawBottom						&&
 	      dc->dcDrawCellBottom					&&
-	      ( cp->cpBottomBorder.bpIsSet || bottomAsGrid )		)
+	      ( cp->cpBottomBorder.bpStyle != DOCbsNONE || bottomAsGrid ) )
 	    {
 	    if  ( (*dc->dcDrawCellBottom)(	&(cp->cpBottomBorder),
 						bottomAsGrid,
@@ -680,6 +557,94 @@ static int docDrawCellSides(	const BufferItem *		rowBi,
     return 0;
     }
 
+
+/************************************************************************/
+/*									*/
+/*  Draw the lines in a table cell that appear on one printed page.	*/
+/*									*/
+/************************************************************************/
+
+static int doDrawAdvanceCell(	void *				through,
+				int *				pAdvanced,
+				BufferItem *			cellBi,
+				int				page,
+				int				column,
+				CellDrawingProgress *		cdp,
+				DrawingContext *		dc,
+				LayoutPosition *		lpBelow )
+				
+    {
+    BufferDocument *		bd= dc->dcDocument;
+    const DocumentProperties *	dp= &(bd->bdProperties);
+
+    int				bottomTwips= -1;
+    int				stripHigh= -1;
+
+    int				advanced= 0;
+
+    LayoutPosition		lp;
+
+    while( cdp->cdpPara < cellBi->biChildCount )
+	{
+	BufferItem *		paraBi= cellBi->biChildren[cdp->cdpPara];
+	int			linesDone= cdp->cdpLinesDone;
+
+	if  ( paraBi->biTopPosition.lpPage > page )
+	    { break;	}
+
+	if  ( cdp->cdpLinesDone == 0 )
+	    {
+	    docParagraphFrameTwips( &(cdp->cdpFormattingFrame),
+			    bottomTwips, stripHigh, paraBi, dp, page, column );
+
+	    if  ( dc->dcParaFramePixels )
+		{
+		(*dc->dcParaFramePixels) ( &(cdp->cdpFormattingFrame),
+						dc->dcDrawingData, paraBi );
+		}
+	    }
+
+	if  ( docDrawTextLines( through, paraBi, page,
+				&(cdp->cdpFormattingFrame), &linesDone, dc ) )
+	    { LDEB(page); return -1;	}
+
+	if  ( linesDone > cdp->cdpLinesDone )
+	    { cdp->cdpAdvanced= advanced= 1;	}
+
+	cdp->cdpLinesDone= linesDone;
+
+	if  ( cdp->cdpLinesDone < paraBi->biParaLineCount )
+	    {
+	    if  ( cdp->cdpLinesDone > 0 )
+		{
+		TextLine *		tl;
+
+		tl= paraBi->biParaLines+ cdp->cdpLinesDone- 1;
+
+		lp.lpPage= tl->tlTopPosition.lpPage;
+		lp.lpPageYTwips= tl->tlTopPosition.lpPageYTwips+
+						    tl->tlLineSpacingTwips;
+		lp.lpAtTopOfColumn= 0;
+		}
+	    else{ lp= paraBi->biTopPosition;	}
+	    }
+	else{ lp= paraBi->biBelowPosition;	}
+
+	docPsLayoutPushBottomDown( lpBelow, &lp );
+
+	if  ( cdp->cdpLinesDone < paraBi->biParaLineCount )
+	    { break;	}
+
+	cdp->cdpLinesDone= 0;
+	cdp->cdpPara++;
+	}
+
+    if  ( advanced )
+	{ *pAdvanced= 1;	}
+
+    return 0;
+    }
+
 /************************************************************************/
 /*									*/
 /*  Print as much of a table row as fits on the current page.		*/
@@ -688,8 +653,9 @@ static int docDrawCellSides(	const BufferItem *		rowBi,
 
 static int docDrawRowPageStrip(	const BufferItem *		rowBi,
 				int				page,
+				int				column,
 				int *				pToNextPage,
-				CellDrawingProgress *		cdp,
+				CellDrawingProgress *		cdpRow,
 				void *				through,
 				DrawingContext *		dc,
 				const LayoutPosition *		lpTop )
@@ -703,13 +669,16 @@ static int docDrawRowPageStrip(	const BufferItem *		rowBi,
 
     LayoutPosition		lpBottom;
 
+    CellDrawingProgress *	cdp;
+
+    lpBottom= *lpTop;
+
     advanced= 0;
-    for ( col= 0; col < rowBi->biChildCount; col++ )
+    cdp= cdpRow;
+    for ( col= 0; col < rowBi->biChildCount; cdp++, col++ )
 	{
 	BufferItem *	cellBi= rowBi->biChildren[col];
 
-	lpBottom= *lpTop;
-
 	if  ( dc->dcSelection )
 	    {
 	    const BufferSelection *	bs= dc->dcSelection;
@@ -725,14 +694,14 @@ static int docDrawRowPageStrip(	const BufferItem *		rowBi,
 		{ continue;	}
 	    }
 
-	if  ( cdp[col].cdpAdvanced )
+	if  ( cdp->cdpAdvanced )
 	    { atTop= 0;	}
 
-	if  ( psAdvanceCell( through, &advanced, cellBi, page,
-						    cdp+ col, dc, &lpBottom ) )
+	if  ( doDrawAdvanceCell( through, &advanced, cellBi, page, column,
+						    cdp, dc, &lpBottom ) )
 	    { LDEB(1); return -1;	}
 
-	if  ( cdp[col].cdpPara < cellBi->biChildCount	)
+	if  ( cdp->cdpPara < cellBi->biChildCount	)
 	    { toNextPage= 1;	}
 	}
 
@@ -744,7 +713,7 @@ static int docDrawRowPageStrip(	const BufferItem *		rowBi,
 
 	if  ( drawTop || drawSides || drawBottom )
 	    {
-	    if  ( docDrawCellSides( rowBi, cdp,
+	    if  ( docDrawCellSides( rowBi, cdpRow,
 					drawTop, drawSides, drawBottom,
 					through, dc, lpTop, &lpBottom ) )
 		{ LDEB(1); return -1;	}
@@ -773,12 +742,13 @@ static int docDrawRowItem(	BufferItem *			rowBi,
     int				col;
 
     const RowProperties *	rp= &(rowBi->biRowProperties);
-    const SectionProperties *	sp= &(rowBi->biParent->biSectProperties);
-    const DocumentGeometry *	dg= &(sp->spDocumentGeometry);
+    BufferItem *		sectBi= rowBi->biParent;
+    const DocumentGeometry *	dg= &(sectBi->biSectDocumentGeometry);
     CellProperties *		cp;
 
     int				toNextPage;
     int				page= rowBi->biTopPosition.lpPage;
+    int				column= rowBi->biTopPosition.lpColumn;
 
     const int			drawTop= 0;
     const int			drawSides= 1;
@@ -835,8 +805,8 @@ static int docDrawRowItem(	BufferItem *			rowBi,
 	cdp[col].cdpX1Pixels= x1Pixels;
 	cdp[col].cdpX11Pixels= x1Pixels;
 
-	if  ( cp->cpRightBorder.bpIsSet )
-	    { cdp[col].cdpX11Twips +=  cp->cpRightBorder.bpWidthTwips; }
+	if  ( cp->cpRightBorder.bpStyle != DOCbsNONE )
+	    { cdp[col].cdpX11Twips +=  cp->cpRightBorder.bpPenWideTwips; }
 
 	x0Twips= x1Twips;
 	x0Pixels= x1Pixels;
@@ -844,12 +814,15 @@ static int docDrawRowItem(	BufferItem *			rowBi,
 
     lpTop= rowBi->biTopPosition;
 
-    if  ( docDrawRowPageStrip( rowBi, page, &toNextPage, cdp,
+    if  ( docDrawRowPageStrip( rowBi, page, column, &toNextPage, cdp,
 						    through, dc, &lpTop ) )
 	{ LDEB(1); return -1;	}
 
     while( toNextPage )
 	{
+	BufferDocument *		bd= dc->dcDocument;
+	const DocumentProperties *	dp= &(bd->bdProperties);
+
 	/*  1  */
 	if  ( dc->dcLastPage >= 0	&&
 	      page >= dc->dcLastPage	)
@@ -859,9 +832,9 @@ static int docDrawRowItem(	BufferItem *			rowBi,
 	    { SDEB(docLevelStr(rowBi->biLevel)); return -1;	}
 
 	page++;
-	docPsStartPageLayout( page, add, sp, &lpTop );
+	docPsStartPageLayout( page, sectBi, dp, add, &lpTop );
 
-	if  ( docDrawRowPageStrip( rowBi, page, &toNextPage, cdp,
+	if  ( docDrawRowPageStrip( rowBi, page, column, &toNextPage, cdp,
 						    through, dc, &lpTop ) )
 	    { LDEB(1); return -1;	}
 	}
@@ -884,10 +857,12 @@ static int docDrawGroupItem(	const BufferItem *		bi,
 				void *				through,
 				DrawingContext *		dc )
     {
-    int			i;
-    int			page;
+    AppDrawingData *		add= dc->dcDrawingData;
 
-    BufferItem *	prevBi= (BufferItem *)0;
+    int				i;
+    int				page;
+
+    BufferItem *		prevBi= (BufferItem *)0;
 
     if  ( bi->biChildCount > 0 )
 	{ prevBi= bi->biChildren[0];	}
@@ -898,7 +873,7 @@ static int docDrawGroupItem(	const BufferItem *		bi,
 	BufferItem *	thisBi= bi->biChildren[i];
 
 	if  ( dc->dcClipRect						&&
-	      thisBi->biTopPosition.lpYPixels > dc->dcClipRect->drY1	)
+	      BI_TOP_PIXELS( add, thisBi ) > dc->dcClipRect->drY1	)
 	    { break;	}
 
 	if  ( dc->dcSelection					&&
@@ -935,12 +910,14 @@ int docDrawItem(	BufferItem *			bi,
 			void *				through,
 			DrawingContext *		dc )
     {
+    AppDrawingData *		add= dc->dcDrawingData;
+
     if  ( dc->dcClipRect )
 	{
-	if  ( bi->biTopPosition.lpYPixels > dc->dcClipRect->drY1 )
+	if  ( BI_TOP_PIXELS( add, bi ) > dc->dcClipRect->drY1 )
 	    { return 0;	}
 
-	if  ( bi->biBelowPosition.lpYPixels < dc->dcClipRect->drY0 )
+	if  ( BI_BELOW_PIXELS( add, bi ) < dc->dcClipRect->drY0 )
 	    { return 0; }
 	}
 
