@@ -427,7 +427,7 @@ int docParaReplaceText(	BufferDocument *	bd,
 			const unsigned char *	addedString,
 			unsigned int		addedStrlen,
 			TextAttribute		addedAttr,
-			void *			voiddisplay,
+			void *			voidadd,
 			DOC_CLOSE_OBJECT	closeObject )
     {
     int				stroffShift;
@@ -465,11 +465,8 @@ int docParaReplaceText(	BufferDocument *	bd,
 	{ LDEB(bi->biParaParticuleCount); return -1;	}
 
     /*  2  */
-    stroffShift= addedStrlen- stroffEnd+ stroffBegin;
+    stroffShift= 0;
     partShift= 0;
-    if  ( stroffShift > 0				&&
-	  docInflateTextString( bi, stroffShift )	)
-	{ LLDEB(bi->biParaStrlen,stroffShift); return -1;	}
 
     /*  3  */
     if  ( addedStrlen == 0 )
@@ -557,8 +554,6 @@ int docParaReplaceText(	BufferDocument *	bd,
 #	endif
 	}
 
-    newEnd += stroffShift;
-
     /*  10  */
     oldCount= 0;
     if  ( excludeBegin )
@@ -585,7 +580,7 @@ int docParaReplaceText(	BufferDocument *	bd,
 		    bi->biParaString+ tp->tpStroff );
 #	endif
 
-	(*closeObject)( bd, bi, tp, voiddisplay );
+	(*closeObject)( bd, bi, tp, voidadd );
 	docCleanParticuleObject( bi, tp );
 
 	if  ( tp->tpKind == DOCkindFIELDSTART	||
@@ -598,19 +593,9 @@ int docParaReplaceText(	BufferDocument *	bd,
 	}
 
     /*  12  */
-    if  ( bi->biParaStrlen > 0 )
-	{
-	memmove( bi->biParaString+ stroffEnd+ stroffShift,
-		bi->biParaString+ stroffEnd, bi->biParaStrlen- stroffEnd+ 1 );
-	}
-
-    memcpy( bi->biParaString+ stroffBegin, addedString, addedStrlen );
-
-    if  ( stroffShift != 0 )
-	{
-	bi->biParaString[bi->biParaStrlen+ stroffShift]= '\0';
-	bi->biParaStrlen += stroffShift;
-	}
+    if  ( docParaStringReplace( &stroffShift, bi, stroffBegin, stroffEnd,
+						addedString, addedStrlen ) )
+	{ LDEB(addedStrlen); return -1;	}
 
     /*  13  */
     if  ( fieldCount > 0 )
@@ -619,6 +604,7 @@ int docParaReplaceText(	BufferDocument *	bd,
 	{ LDEB(fieldCount); return -1;	}
 
     /*  14  */
+    newEnd += stroffShift;
     newCount= 0;
 
     /*  15  */
@@ -758,14 +744,14 @@ static int docParaAppend(	BufferDocument *	bdTo,
 				BufferItem *		biFrom,
 				int			part,
 				unsigned int		stroffFrom,
-				void *			voiddisplay,
+				void *			voidadd,
 				DOC_CLOSE_OBJECT	closeObject )
     {
     TextParticule *	tpFrom= biFrom->biParaParticules+ part;
     TextParticule *	newTp;
 
-    int			partShift= 0;
     int			stroffShift= 0;
+    int			partShift= 0;
     int			particuleCount;
 
     int			stroffTo;
@@ -774,8 +760,6 @@ static int docParaAppend(	BufferDocument *	bdTo,
 	  stroffFrom == tpFrom->tpStroff+ tpFrom->tpStrlen	)
 	{ tpFrom++; part++;	}
 
-    stroffShift= biFrom->biParaStrlen- stroffFrom;
-
     /*  2  */
     particuleCount= biTo->biParaParticuleCount+
 				    biFrom->biParaParticuleCount- part+ 1;
@@ -803,7 +787,7 @@ static int docParaAppend(	BufferDocument *	bdTo,
 				biFrom->biParaString+ stroffFrom,
 				tpFrom->tpStroff+ tpFrom->tpStrlen- stroffFrom,
 				tpFrom->tpTextAttribute,
-				voiddisplay, closeObject ) )
+				voidadd, closeObject ) )
 		{ LDEB(tpFrom->tpStrlen); return -1;	}
 
 	    stroffFrom= tpFrom->tpStroff+ tpFrom->tpStrlen;
@@ -811,20 +795,14 @@ static int docParaAppend(	BufferDocument *	bdTo,
 	    }
 	}
 
-    /*  1  */
-    if  ( docInflateTextString( biTo, biFrom->biParaStrlen- stroffFrom ) )
-	{ LDEB(biFrom->biParaStrlen- stroffFrom); return -1;	}
-
-    /*  4  */
-    memcpy( biTo->biParaString+ biTo->biParaStrlen,
-	    biFrom->biParaString+ stroffFrom,
-	    biFrom->biParaStrlen- stroffFrom );
-    stroffTo= biTo->biParaStrlen+biFrom->biParaStrlen- stroffFrom;
-    biTo->biParaString[stroffTo]= '\0';
-
-    /*  5  */
     stroffTo= biTo->biParaStrlen;
 
+    /*  1,4,5  */
+    if  ( docParaStringReplace( &stroffShift, biTo, stroffTo, stroffTo,
+					biFrom->biParaString+ stroffFrom,
+					biFrom->biParaStrlen- stroffFrom ) )
+	{ LDEB(biFrom->biParaStrlen); return -1;	}
+
     while( part < biFrom->biParaParticuleCount )
 	{
 	int	len= tpFrom->tpStroff+ tpFrom->tpStrlen- stroffFrom;
@@ -853,8 +831,6 @@ static int docParaAppend(	BufferDocument *	bdTo,
 	tpFrom++; part++; partShift++;
 	}
 
-    biTo->biParaStrlen= stroffTo;
-
     return 0;
     }
 
@@ -983,6 +959,7 @@ static int docSplitParaItemLow(	BufferDocument *	bd,
 
     int				splittingField= 0;
 
+    int				stroffShift= 0;
     int				partShift= 0;
     int				newStrlen= 0;
 
@@ -1015,18 +992,15 @@ static int docSplitParaItemLow(	BufferDocument *	bd,
 	{ LDEB(oldBi->biParaTabCount); return -1;	}
 
     /*  2  */
-    if  ( docInflateTextString( newBi, oldBi->biParaStrlen- stroff ) )
+    if  ( docParaStringReplace( &stroffShift, newBi, 0, 0,
+						oldBi->biParaString+ stroff,
+						oldBi->biParaStrlen- stroff ) )
 	{
 	LLDEB(oldBi->biParaStrlen,stroff);
 	docDeleteItems( bd, oldBi->biParent, oldBi->biNumberInParent+ 1, 1 );
 	return -1;
 	}
 
-    memcpy( newBi->biParaString, oldBi->biParaString+ stroff,
-						oldBi->biParaStrlen- stroff );
-    newBi->biParaStrlen= oldBi->biParaStrlen- stroff;
-    newBi->biParaString[newBi->biParaStrlen]= '\0';
-
     splittingField= docParticuleInField( oldBi, part );
 
     /*  3  */
@@ -1276,15 +1250,12 @@ TextParticule * docParaSpecialParticule(BufferItem *		bi,
     {
     TextParticule *	tp;
     TextParticule *	tpRet;
+    int			stroffShift= 0;
 
     /*  -1  */
-    if  ( docInflateTextString( bi, 1 )	)
-	{ LLDEB(bi->biParaStrlen,1); return (TextParticule *)0;	}
-
-    memmove( bi->biParaString+ stroff+ 1, bi->biParaString+ stroff,
-						bi->biParaStrlen- stroff+ 1 );
-    bi->biParaString[stroff]= ' ';
-    bi->biParaStrlen++;
+    if  ( docParaStringReplace( &stroffShift, bi, stroff, stroff,
+						(unsigned char *)" ", 1 ) )
+	{ LDEB(stroff); return (TextParticule *)0;	}
 
     if  ( part == bi->biParaParticuleCount		&&
 	  stroff == bi->biParaStrlen- 1			)
@@ -1297,7 +1268,8 @@ TextParticule * docParaSpecialParticule(BufferItem *		bi,
 
 	tpRet->tpObjectNumber= -1; /* set later on */
 
-	*pPartShift= 1; *pStroffShift= 1; return tpRet;
+	*pPartShift= 1; *pStroffShift= stroffShift;
+	return tpRet;
 	}
 
     if  ( part < 0 || part >= bi->biParaParticuleCount )
@@ -1313,7 +1285,8 @@ TextParticule * docParaSpecialParticule(BufferItem *		bi,
 	bi->biParaParticules->tpKind= kind;
 	bi->biParaParticules->tpStrlen= 1;
 
-	*pPartShift= 0; *pStroffShift= 1; return bi->biParaParticules;
+	*pPartShift= 0; *pStroffShift= stroffShift;
+	return bi->biParaParticules;
 	}
 
     /*  1  */
@@ -1366,7 +1339,7 @@ TextParticule * docParaSpecialParticule(BufferItem *		bi,
     while( part < bi->biParaParticuleCount )
 	{ tp->tpStroff++; part++; tp++;	}
 
-    *pStroffShift= 1;
+    *pStroffShift= stroffShift;
 
     return tpRet;
     }
@@ -1379,7 +1352,7 @@ TextParticule * docParaSpecialParticule(BufferItem *		bi,
 
 static void docCloseParaObjects(	BufferDocument *	bd,
 					BufferItem *		bi,
-					void *			voiddisplay,
+					void *			voidadd,
 					DOC_CLOSE_OBJECT	closeObject )
     {
     int			part;
@@ -1392,7 +1365,7 @@ static void docCloseParaObjects(	BufferDocument *	bd,
 	      tp->tpPhysicalFont != 0		)
 	    { continue;	}
 
-	(*closeObject)( bd, bi, tp, voiddisplay );
+	(*closeObject)( bd, bi, tp, voidadd );
 	}
 
     return;
