@@ -36,11 +36,11 @@ static void psPrintHorizontalBorder(	const BorderProperties *	bp,
     {
     PrintingState *	ps= (PrintingState *)vps;
 
-    if  ( ! bp->bpIsSet )
+    if  ( bp->bpStyle == DOCbsNONE )
 	{ return;	}
 
     fprintf( ps->psFile, "%d %d %d %d rectfill\n",
-					x0, y, x1- x0, bp->bpWidthTwips );
+					x0, y, x1- x0, bp->bpPenWideTwips );
 
     return;
     }
@@ -53,8 +53,11 @@ static void psPrintVerticalBorder(	const BorderProperties *	bp,
     {
     PrintingState *	ps= (PrintingState *)vps;
 
+    if  ( bp->bpStyle == DOCbsNONE )
+	{ return;	}
+
     fprintf( ps->psFile, "%d %d %d %d rectfill\n",
-					x, y0, bp->bpWidthTwips, y1- y0 );
+					x, y0, bp->bpPenWideTwips, y1- y0 );
 
     return;
     }
@@ -87,8 +90,7 @@ static int psPrintMetafile(	FILE *				f,
     {
     SimpleInputStream *		sisMem;
     SimpleInputStream *		sisMeta;
-    MemoryBuffer		mb;
-    const ObjectData *		od;
+    const MemoryBuffer *	mb;
 
     PostScriptFont *		fontList= (PostScriptFont *)0;
     int				fontCount= 0;
@@ -104,17 +106,17 @@ static int psPrintMetafile(	FILE *				f,
     switch( io->ioKind )
 	{
 	case DOCokPICTWMETAFILE:
-	    od= &(io->ioObjectData);
+	    mb= &(io->ioObjectData);
 	    playMetafile= appMetaPlayFilePs;
 	    break;
 
 	case DOCokMACPICT:
-	    od= &(io->ioObjectData);
+	    mb= &(io->ioObjectData);
 	    playMetafile= appMacPictPlayFilePs;
 	    break;
 
 	case DOCokOLEOBJECT:
-	    od= &(io->ioResultData);
+	    mb= &(io->ioResultData);
 	    playMetafile= appMetaPlayFilePs;
 	    break;
 
@@ -124,16 +126,13 @@ static int psPrintMetafile(	FILE *				f,
 	    LDEB(io->ioKind); return 0;
 	}
 
-    mb.mbCapacity= od->odSize;
-    mb.mbBytes= od->odBytes;
-
-    sisMem= sioInMemoryOpen( &mb );
+    sisMem= sioInMemoryOpen( mb );
     if  ( ! sisMem )
 	{ XDEB(sisMem); return -1;	}
 
     sisMeta= sioInHexOpen( sisMem );
     if  ( ! sisMeta )
-	{ XDEB(sisMem); return -1;	}
+	{ XDEB(sisMem); sioInClose( sisMem ); return -1;	}
 
     y0= baseline- ( ( scaleY/100.0 )* twipsHigh );
 
@@ -161,6 +160,60 @@ static int psPrintMetafile(	FILE *				f,
     return 0;
     }
 
+static int psPrintIncludeEpsObject(	PrintingState *		ps,
+					InsertedObject *	io,
+					int			x0,
+					int			baseLine )
+    {
+    SimpleInputStream *		sisMem;
+    SimpleInputStream *		sisHex;
+
+    char			line[512+2];
+
+    sisMem= sioInMemoryOpen( &(io->ioResultData) );
+    if  ( ! sisMem )
+	{ XDEB(sisMem); return -1;	}
+
+    sisHex= sioInHexOpen( sisMem );
+    if  ( ! sisHex )
+	{ XDEB(sisHex); sioInClose( sisMem ); return -1;	}
+
+    appPsBeginEpsObject( ps, x0, baseLine,
+				0, 0, io->ioTwipsWide, io->ioTwipsHigh,
+				io->ioObjectData.mbBytes );
+
+    while( sioInGetString( line, 512+1, sisHex ) )
+	{
+	int		emit= 1;
+	const char *	s= line;
+
+	while( isspace( *s ) )
+	    { s++;	}
+
+	if  ( ! *s || *s == '%' )
+	    { emit= 0;	}
+
+	if  ( emit )
+	    { fputs( line, ps->psFile ); }
+
+	while( ! strchr( line, '\n' ) )
+	    {
+	    if  ( ! sioInGetString( line, 512+1, sisHex ) )
+		{ break;	}
+
+	    if  ( emit )
+		{ fputs( line, ps->psFile ); }
+	    }
+	}
+
+    sioInClose( sisHex );
+    sioInClose( sisMem );
+
+    appPsEndEpsObject( ps );
+
+    return 0;
+    }
+
 /************************************************************************/
 /*									*/
 /*  Write links to be picked up by the distiller.			*/
@@ -427,12 +480,10 @@ static int psPrintParticules(	PrintingState *			ps,
 		    appPsSetLinkColor( ps );
 		    }
 
-		drawn= 1;
-		return drawn;
+		return drawn= 1;
 		}
 
-	    drawn= 1;
-	    return drawn;
+	    return drawn= 1;
 
 	case DOCkindFIELDEND:
 	    if  ( ps->psInLink )
@@ -443,16 +494,13 @@ static int psPrintParticules(	PrintingState *			ps,
 
 	    appPsSetTextColor( ps );
 
-	    drawn= 1;
-
-	    return drawn;
+	    return drawn= 1;
 
 	case DOCkindXE:
 	case DOCkindTC:
 	case DOCkindLINEBREAK:
 	case DOCkindPAGEBREAK:
-	    drawn= 1;
-	    return drawn;
+	    return drawn= 1;
 
 	case DOCkindOBJECT:
 	    io= bi->biParaObjects+ tp->tpObjectNumber;
@@ -511,6 +559,19 @@ static int psPrintParticules(	PrintingState *			ps,
 			}
 		    break;
 
+		case DOCokINCLUDEPICTURE:
+		    if  ( io->ioResultKind == DOCokEPSF )
+			{
+			if  ( psPrintIncludeEpsObject( ps,
+						io, pd->pdX0, baseLine ) )
+			    { LDEB(1); break;	}
+
+			ps->psCurrentPhysicalFont= -1;
+			ps->psLinkParticulesDone++;
+			return 1;
+			}
+		    else{ LLDEB(io->ioKind,io->ioResultKind); break;	}
+
 		default:
 		    LDEB(io->ioKind); return 0;
 		}
@@ -678,8 +739,7 @@ static int psPrintTextLine(	PrintingState *			ps,
 
 static int docPsPrintTextLine(	const BufferItem *		bi,
 				int				line,
-				const FormattingFrame *		ffTwips,
-				const FormattingFrame *		ign_ffPixels,
+				const FormattingFrame *		ff,
 				void *				vps,
 				DrawingContext *		dc )
     {
@@ -705,7 +765,7 @@ static int docPsPrintTextLine(	const BufferItem *		bi,
     boxLine= *tl;
     accepted= docPsLineBox( &boxLine, bi, part,
 				    dp->dpTabIntervalTwips, &dp->dpFontList,
-				    apfl, tp, pd, ffTwips );
+				    apfl, tp, pd, ff );
 
     if  ( accepted < 1 )
 	{ LDEB(accepted); return -1;	}
@@ -725,21 +785,20 @@ static int docPsPrintTextLine(	const BufferItem *		bi,
 
 static int docPsPrintParaTop(	const BufferItem *		bi,
 				const BorderProperties *	bp,
-				const FormattingFrame *		ffTwips,
-				const FormattingFrame *		ign_ffPixels,
+				const FormattingFrame *		ff,
 				void *				vps,
 				DrawingContext *		dc )
     {
     PrintingState *	ps= (PrintingState *)vps;
-    int			x0= ffTwips->ffX0TextLines;
+    int			x0= ff->ffX0TextLinesTwips;
 
-    if  ( ffTwips->ffX0FirstLine < x0 )
-	{ x0= ffTwips->ffX0FirstLine;	}
+    if  ( ff->ffX0FirstLineTwips < x0 )
+	{ x0= ff->ffX0FirstLineTwips;	}
 
     appPsSetTextColor( ps );
 
     psPrintHorizontalBorder( bi->biParaBorderAboveParagraph,
-					vps, x0, ffTwips->ffX1TextLines,
+					vps, x0, ff->ffX1TextLinesTwips,
 					bi->biTopPosition.lpPageYTwips+
 					bi->biParaSpaceBeforeTwips );
 
@@ -748,21 +807,20 @@ static int docPsPrintParaTop(	const BufferItem *		bi,
 
 static int docPsPrintParaBottom( const BufferItem *		bi,
 				const BorderProperties *	bp,
-				const FormattingFrame *		ffTwips,
-				const FormattingFrame *		ign_ffPixels,
+				const FormattingFrame *		ff,
 				void *				vps,
 				DrawingContext *		dc )
     {
     PrintingState *	ps= (PrintingState *)vps;
-    int			x0= ffTwips->ffX0TextLines;
+    int			x0= ff->ffX0TextLinesTwips;
 
-    if  ( ffTwips->ffX0FirstLine < x0 )
-	{ x0= ffTwips->ffX0FirstLine;	}
+    if  ( ff->ffX0FirstLineTwips < x0 )
+	{ x0= ff->ffX0FirstLineTwips;	}
 
     appPsSetTextColor( ps );
 
     psPrintHorizontalBorder( bi->biParaBorderBelowParagraph,
-					vps, x0, ffTwips->ffX1TextLines,
+					vps, x0, ff->ffX1TextLinesTwips,
 					bi->biBelowPosition.lpPageYTwips-
 					bi->biParaSpaceAfterTwips );
 
@@ -781,7 +839,7 @@ static int docPsPrintCellTop(	const BorderProperties *	bp,
     {
     PrintingState *	ps= (PrintingState *)vps;
 
-    if  ( ! asGrid && bp->bpIsSet )
+    if  ( ! asGrid && bp->bpStyle != DOCbsNONE )
 	{
 	appPsSetTextColor( ps );
 
@@ -804,7 +862,7 @@ static int docPsPrintCellBottom( const BorderProperties *	bp,
     {
     PrintingState *	ps= (PrintingState *)vps;
 
-    if  ( ! asGrid && bp->bpIsSet )
+    if  ( ! asGrid && bp->bpStyle != DOCbsNONE )
 	{
 	appPsSetTextColor( ps );
 
@@ -826,7 +884,7 @@ static int docPsPrintCellLeft(	const BorderProperties *	bp,
     {
     PrintingState *	ps= (PrintingState *)vps;
 
-    if  ( ! asGrid && bp->bpIsSet )
+    if  ( ! asGrid && bp->bpStyle != DOCbsNONE )
 	{
 	appPsSetTextColor( ps );
 
@@ -848,7 +906,7 @@ static int docPsPrintCellRight(	const BorderProperties *	bp,
     {
     PrintingState *	ps= (PrintingState *)vps;
 
-    if  ( ! asGrid && bp->bpIsSet )
+    if  ( ! asGrid && bp->bpStyle != DOCbsNONE )
 	{
 	appPsSetTextColor( ps );
 
@@ -943,8 +1001,8 @@ int docPsPrintDocument(	FILE *				f,
 			int				firstPage,
 			int				lastPage,
 			const DocumentGeometry *	dgPrinter,
-			LAYOUT_EXTERNAL			layoutHeader,
-			LAYOUT_EXTERNAL			layoutFooter )
+			LAYOUT_EXTERNAL			layoutExternal,
+			DOC_CLOSE_OBJECT		closeObject )
     {
     DocumentProperties *	dp= &(bd->bdProperties);
     DocumentGeometry *		dg= &(dp->dpGeometry);
@@ -968,8 +1026,8 @@ int docPsPrintDocument(	FILE *				f,
     dc.dcDrawCellRight= docPsPrintCellRight;
     dc.dcFinishPage= docPsFinishPage;
     dc.dcStartPage= docPsStartPage;
-    dc.dcLayoutHeader= layoutHeader;
-    dc.dcLayoutFooter= layoutFooter;
+    dc.dcLayoutExternal= layoutExternal;
+    dc.dcCloseObject= closeObject;
 
     dc.dcDrawingData= add;
     dc.dcDocument= bd;
@@ -1072,6 +1130,8 @@ int docPsPrintDocument(	FILE *				f,
     appPsSetRectfillEmulation( f );
     appPsSetSelectfontEmulation( f );
 
+    appPsDefineEpsProcs( f );
+
     appPsFontNames( f, fontList, fontCount, /*allFonts=*/ 1 );
 
     appMetaDefineProcsetPs( f );
