@@ -20,7 +20,7 @@
 /*  Translate shape type to string.					*/
 /*									*/
 /************************************************************************/
-#   if 0
+
 static char * docRtfShapeTypeString(	int	shapeType )
     {
     static char		scratch[12];
@@ -223,7 +223,6 @@ static char * docRtfShapeTypeString(	int	shapeType )
 	    return scratch;
 	}
     }
-#   endif
 
 /************************************************************************/
 /*									*/
@@ -556,7 +555,9 @@ static int docRtfShapeNumber(	SimpleInputStream *	sis,
     switch( rcw->rcwId )
 	{
 	case RTFSHP_shapeType:
-	    /*SDEB(docRtfShapeTypeString(val));*/
+	    if  ( rcw->rcwId != RTFSHP_shapeType )
+		{ SDEB(docRtfShapeTypeString(val));	}
+
 	    ds->ds_shapeType= val;
 	    break;
 
@@ -1122,6 +1123,8 @@ static int docRtfShapeProperty(	SimpleInputStream *	sis,
 
     int				i;
 
+    const int			listOnly= 1;
+
     const RtfControlWord *	rcwSn;
 
     control= docRtfFindControl( sis, rrc, &c, controlWord, &gotArg, &locArg );
@@ -1149,7 +1152,7 @@ static int docRtfShapeProperty(	SimpleInputStream *	sis,
 	controlWord[i++]= c;
 	}
 
-    rcwSn= docRtfFindWord( controlWord, docRtfShapeProperties, 1 );
+    rcwSn= docRtfFindWord( controlWord, docRtfShapeProperties, listOnly );
     if  ( ! rcwSn )
 	{ SDEB(controlWord);	}
 
