@@ -66,6 +66,7 @@ void docRtfInitReadingContext(	RtfReadingContext *	rrc	)
 				/****************************************/
 				/* Attributes of the current position.	*/
 				/****************************************/
+    docInitTextFrameProperties( &(rrc->rrcTextFrameProperties) );
     docInitParagraphProperties( &(rrc->rrcParagraphProperties) );
     docInitCellProperties( &(rrc->rrcCellProperties) );
     docInitRowProperties( &(rrc->rrcRowProperties) );
@@ -92,9 +93,7 @@ void docRtfInitReadingContext(	RtfReadingContext *	rrc	)
 				/*  Document properties.		*/
 				/****************************************/
     docInitFont( &(rrc->rrcCurrentFont) );
-    rrc->rrcStyleNumber= 0;
-    rrc->rrcStyleBasedOn= 0;
-    rrc->rrcStyleNext= 0;
+    docInitStyle( &(rrc->rrcStyle) );
 				/****************************************/
 				/*  For reading the font table.		*/
 				/****************************************/
@@ -130,6 +129,8 @@ void docRtfCleanReadingContext(	RtfReadingContext *	rrc	)
 
     docCleanDocumentList( &(rrc->rrcDocumentList) );
 
+    docCleanStyle( &(rrc->rrcStyle) );
+
     if  ( rrc->rrcParagraphNumbers )
 	{ free( rrc->rrcParagraphNumbers );	}
 
@@ -152,6 +153,7 @@ int docRtfCopyReadingContext(	RtfReadingContext *		to,
     ParagraphProperties		ppCopy;
     RowProperties		rpCopy;
     SectionProperties		spCopy;
+    DocumentStyle		dsCopy;
 
     docInitParagraphProperties( &ppCopy );
     if  ( docCopyParagraphProperties( &ppCopy,
@@ -166,6 +168,10 @@ int docRtfCopyReadingContext(	RtfReadingContext *		to,
     if  ( docCopySectionProperties( &spCopy, &(from->rrcSectionProperties) ) )
 	{ LDEB(1); return -1;	}
 
+    docInitStyle( &dsCopy );
+    if  ( docCopyStyle( &dsCopy, &(from->rrcStyle) ) )
+	{ LDEB(1); return -1;	}
+
     *to= *from;
 
     to->rrcParagraphNumbers= (ParagraphNumber *)0;
@@ -174,6 +180,7 @@ int docRtfCopyReadingContext(	RtfReadingContext *		to,
     to->rrcParagraphProperties= ppCopy;
     to->rrcRowProperties= rpCopy;
     to->rrcSectionProperties= spCopy;
+    to->rrcStyle= dsCopy;
 
     to->rrcTopBookmark= from->rrcTopBookmark;
 
@@ -273,22 +280,50 @@ static int docRtfReadControlWord(	SimpleInputStream *	sis,
     /*HACK*/
     if  ( ! strcmp( controlWord, "endash" )	||
           ! strcmp( controlWord, "emdash" )	)
-	{ *pC= ISO1_hyphen; return 2;	}
+	{
+	*pC= ISO1_hyphen;
+	if  ( c != ' ' )
+	    { sioInUngetLastRead( sis );	}
+	return 2;
+	}
 
     if  ( ! strcmp( controlWord, "rquote" )	)
-	{ *pC= ISO1_quoteright; return 2;	}
+	{
+	*pC= ISO1_quoteright;
+	if  ( c != ' ' )
+	    { sioInUngetLastRead( sis );	}
+	return 2;
+	}
 
     if  ( ! strcmp( controlWord, "lquote" )	)
-	{ *pC= ISO1_quoteleft; return 2;	}
+	{
+	*pC= ISO1_quoteleft;
+	if  ( c != ' ' )
+	    { sioInUngetLastRead( sis );	}
+	return 2;
+	}
 
     if  ( ! strcmp( controlWord, "ldblquote" )	)
-	{ *pC= ISO1_quotedbl; return 2;	}
+	{
+	*pC= ISO1_quotedbl;
+	if  ( c != ' ' )
+	    { sioInUngetLastRead( sis );	}
+	return 2;
+	}
 
     if  ( ! strcmp( controlWord, "rdblquote" )	)
-	{ *pC= ISO1_quotedbl; return 2;	}
+	{
+	*pC= ISO1_quotedbl;
+	return 2;
+	}
 
     if  ( ! strcmp( controlWord, "bullet" )	)
-	{ *pC= ISO1_periodcentered; return 2;	}
+	{
+	*pC= ISO1_periodcentered;
+	if  ( c != ' ' )
+	    { sioInUngetLastRead( sis );	}
+	return 2;
+	}
 
     if  ( c == '-' )
 	{ sign= -1; c= sioInGetCharacter( sis );	}
@@ -314,6 +349,7 @@ static int docRtfReadControlWord(	SimpleInputStream *	sis,
 /*  Find a Control word.						*/
 /*									*/
 /************************************************************************/
+
 const RtfControlWord * docRtfFindWord(	const char *		controlWord,
 					const RtfControlWord *	contolWords,
 					int			listOnly )
@@ -466,7 +502,14 @@ int docRtfFindControl(		SimpleInputStream *	sis,
 		    if  ( res > 0 )
 			{
 			if  ( c == '*' )
-			    { return RTFfiSTARGROUP;	}
+			    {
+			    res= docRtfReadControlWord( sis, rrc, &c,
+						controlWord, pGotArg, pArg );
+			    if  ( res )
+				{ LDEB(res); return -1;	}
+
+			    return RTFfiSTARGROUP;
+			    }
 
 			if  ( c=='\'' )
 			    {
@@ -801,10 +844,11 @@ static int docRtfReadText(	SimpleInputStream *	sis,
 /*  Skip an unknown or superfluous group.				*/
 /*									*/
 /************************************************************************/
+
 int docRtfSkipGroup(	SimpleInputStream *	sis,
 			const RtfControlWord *	rcw,
 			int			textLevel,
-			RtfReadingContext *	rrc		)
+			RtfReadingContext *	rrc )
     {
     int		complainUnknown= rrc->rrcComplainUnknown;
 
@@ -825,6 +869,7 @@ int docRtfSkipGroup(	SimpleInputStream *	sis,
 /*  Read an RTF control group. typically the document.			*/
 /*									*/
 /************************************************************************/
+
 int docRtfReadGroup(	SimpleInputStream *	sis,
 			int			textLevel,
 
@@ -866,14 +911,74 @@ int docRtfReadGroup(	SimpleInputStream *	sis,
     return rval;
     }
 
+/************************************************************************/
+/*									*/
+/*  Read an unknown group.						*/
+/*									*/
+/************************************************************************/
+
+int docRtfReadUnknownGroup(	SimpleInputStream *	sis,
+				RtfReadingContext *	rrc )
+    {
+    int		complainUnknown= rrc->rrcComplainUnknown;
+
+    rrc->rrcComplainUnknown= 0;
+
+    if  ( docRtfReadGroup( sis, rrc->rrcLevel,
+					(RtfControlWord *)0, 0, 0, rrc,
+					docRtfEmptyTable, docRtfEmptyTable,
+					docRtfIgnoreParticule ) )
+	{ LDEB(1); return -1;	}
+
+    rrc->rrcComplainUnknown= complainUnknown;
+
+    return 0;
+    }
+
+int docRtfReadWordGroup(	SimpleInputStream *	sis,
+				int			textLevel,
+				RtfReadingContext *	rrc,
+				int			gotArg,
+				int			arg,
+				const char *		controlWord,
+				const RtfControlWord *	contolWords,
+				const RtfControlWord *	groupWords,
+				RtfAddTextParticule	addParticule )
+    {
+    const RtfControlWord *	rcw;
+    const int			listOnly= 0;
+
+    rcw= docRtfFindWord( controlWord, contolWords, listOnly );
+    if  ( rcw )
+	{
+	if  ( rcw->rcwLevel != DOClevANY		&&
+	      docRtfAdjustLevel( rrc,
+			    rcw->rcwLevel, textLevel )	)
+	    { SDEB(controlWord); return -1; }
+
+	if  ( docRtfReadGroup( sis, textLevel,
+				rcw, gotArg, arg, rrc,
+				contolWords, groupWords,
+				addParticule ) )
+	    { SDEB(rcw->rcwWord); return -1;	}
+	}
+    else{
+	if  ( rrc->rrcComplainUnknown )
+	    { SDEB(controlWord);	}
+
+	if  ( docRtfReadUnknownGroup( sis, rrc ) )
+	    { LDEB(1); return -1;	}
+	}
+
+    return 0;
+    }
+
 int docRtfConsumeGroup(	SimpleInputStream *	sis,
 			int			textLevel,
-
 			RtfReadingContext *	rrc,
-
 			const RtfControlWord *	contolWords,
 			const RtfControlWord *	groupWords,
-			RtfAddTextParticule	addParticule	)
+			RtfAddTextParticule	addParticule )
     {
     int				res;
     const RtfControlWord *	rcw;
@@ -893,12 +998,11 @@ int docRtfConsumeGroup(	SimpleInputStream *	sis,
 
     for (;;)
 	{
-	int	complainUnknown;
-
 	switch( res )
 	    {
 	    case RTFfiCLOSE:
 		return 0;
+
 	    case RTFfiCHAR:
 		if  ( textLevel != DOClevANY				&&
 		      docRtfAdjustLevel( rrc, textLevel, textLevel )	)
@@ -981,37 +1085,10 @@ int docRtfConsumeGroup(	SimpleInputStream *	sis,
 		rcw= docRtfFindWord( controlWord, groupWords, 1 );
 		if  ( ! rcw )
 		    {
-		    rcw= docRtfFindWord( controlWord, contolWords, 0 );
-		    if  ( rcw )
-			{
-			if  ( rcw->rcwLevel != DOClevANY		&&
-			      docRtfAdjustLevel( rrc,
-					    rcw->rcwLevel, textLevel )	)
-			    { SDEB(controlWord); return -1; }
-
-			if  ( docRtfReadGroup( sis, textLevel,
-						rcw, gotArg, arg, rrc,
-						contolWords, groupWords,
-						addParticule ) )
-			    { SDEB(rcw->rcwWord); return -1;	}
-			}
-		    else{
-			complainUnknown= rrc->rrcComplainUnknown;
-
-			if  ( rrc->rrcComplainUnknown )
-			    { SDEB(controlWord);	}
-
-			rrc->rrcComplainUnknown= 0;
-
-			if  ( docRtfReadGroup( sis,
-				    rrc->rrcLevel,
-				    (RtfControlWord *)0, 0, 0, rrc,
-				    docRtfEmptyTable, docRtfEmptyTable,
-				    docRtfIgnoreParticule ) )
-			    { SDEB(rcw->rcwWord); return -1;	}
-
-			rrc->rrcComplainUnknown= complainUnknown;
-			}
+		    if  ( docRtfReadWordGroup( sis, textLevel, rrc,
+				    gotArg, arg, controlWord,
+				    contolWords, groupWords, addParticule ) )
+			{ SDEB(controlWord); return -1;	}
 		    }
 		else{
 		  groupFound:
@@ -1032,31 +1109,19 @@ int docRtfConsumeGroup(	SimpleInputStream *	sis,
 		    { LDEB(res); return -1;	}
 		break;
 	    case RTFfiSTARGROUP:
-		res= docRtfReadControlWord( sis, rrc, &c,
-					    controlWord, &gotArg, &arg );
-		if  ( res )
-		    { LDEB(res); return -1;	}
 		rcw= docRtfFindWord( controlWord, groupWords, 1 );
 		if  ( rcw )
 		    { goto groupFound; }
 
-		complainUnknown= rrc->rrcComplainUnknown;
-
-		rrc->rrcComplainUnknown= 0;
-
-		if  ( docRtfReadGroup( sis, rrc->rrcLevel,
-			    (RtfControlWord *)0, 0, 0, rrc,
-			    docRtfEmptyTable, docRtfEmptyTable,
-			    docRtfIgnoreParticule ) )
-		    { SDEB(rcw->rcwWord); return -1;	}
-
-		rrc->rrcComplainUnknown= complainUnknown;
+		if  ( docRtfReadUnknownGroup( sis, rrc ) )
+		    { LDEB(1); return -1;	}
 
 		res= docRtfFindControl( sis, rrc, &c,
 					    controlWord, &gotArg, &arg );
 		if  ( res < 0 )
 		    { LDEB(res); return -1;	}
 		break;
+
 	    case RTFfiTEXTGROUP:
 		if  ( docRtfReadGroup( sis, textLevel,
 					(RtfControlWord *)0, 0, 0, rrc,
@@ -1069,6 +1134,7 @@ int docRtfConsumeGroup(	SimpleInputStream *	sis,
 		if  ( res < 0 )
 		    { LDEB(res); return -1;	}
 		break;
+
 	    default:
 		LDEB(res); return -1;
 	    }
