@@ -136,6 +136,12 @@ typedef struct TedAppResources
     char *			tarSelectionColor;
     char *			tarCopiedSelectionColor;
 
+				/****************************************/
+				/*  Defaults.				*/
+				/****************************************/
+    char *			tarDefaultAnsicpgString;
+    int				tarDefaultAnsicpgInt;
+
 				/****************************************/
 				/*  Global data.			*/
 				/****************************************/
@@ -160,6 +166,25 @@ typedef enum LinkKind
     LINKkind_COUNT
     } LinkKind;
 
+/************************************************************************/
+/*									*/
+/*  Supported document formats.						*/
+/*  Match an array in tedMain.c.					*/
+/*									*/
+/************************************************************************/
+
+typedef enum TedDocumentKind
+    {
+    TEDdockindRTF= 0,
+    TEDdockindTEXT_OPEN,
+    TEDdockindTEXT_SAVE_FOLDED,
+    TEDdockindTEXT_SAVE_WIDE,
+    TEDdockindHTML_FILES,
+    TEDdockindALL_FILES,
+
+    TEDdockind_COUNT
+    } TedDocumentKind;
+
 /************************************************************************/
 /*									*/
 /*  Shared resources.							*/
@@ -187,17 +212,8 @@ extern int tedLayoutItem(	BufferItem *		bi,
 				AppDrawingData *	add,
 				DocumentRectangle *	drChanged );
 
-extern int tedLayoutHeaderItem(	BufferItem *			bi,
+extern int tedLayoutExternal(	HeaderFooter *			hf,
 				int				page,
-				int				inHeaderFooter,
-				const SectionProperties *	sp,
-				const BufferDocument *		bd,
-				AppDrawingData *		add,
-				DocumentRectangle *		drChanged );
-
-extern int tedLayoutFooterItem(	BufferItem *			bi,
-				int				page,
-				int				inHeaderFooter,
 				const SectionProperties *	sp,
 				const BufferDocument *		bd,
 				AppDrawingData *		add,
@@ -206,7 +222,6 @@ extern int tedLayoutFooterItem(	BufferItem *			bi,
 extern int tedRedoHeaderFooterLayout( int *			pDone,
 				HeaderFooter *			hf,
 				int				page,
-				int				inHeadFoot,
 				const BufferItem *		sectBi,
 				BufferDocument *		bd,
 				AppDrawingData *		add,
@@ -410,8 +425,8 @@ extern void tedDocFontSupersub(		APP_WIDGET	option,
 extern int tedLayoutDocumentTree(	TedDocument *		td,
 					AppDrawingData *	add );
 
-extern void tedScreenRectangles(	AppDrawingData *	add,
-					DocumentGeometry *	dg );
+extern void tedScreenRectangles(	AppDrawingData *		add,
+					const DocumentGeometry *	dg );
 
 extern void tedAdaptPageToolToDocument(	EditApplication *	ea,
 					EditDocument *		ed );
@@ -576,9 +591,10 @@ extern void tedPositionCoordinates(	BufferPosition *	bp,
 extern void tedSelectionCoordinates(	BufferSelection *	bs,
 					const AppDrawingData *	add );
 
-extern void tedSelectionRectangle(	DocumentRectangle *	drClip,
-					const AppDrawingData *	add,
-					const BufferSelection *	bs );
+extern void tedSelectionRectangle(	DocumentRectangle *		drClip,
+					const AppDrawingData *		add,
+					const DocumentProperties *	dp,
+					const BufferSelection *		bs );
 
 extern void tedDrawRectangle(	APP_WIDGET		w,
 				EditDocument *		ed,
@@ -623,8 +639,9 @@ extern int tedInsertTable(	EditDocument *		ed,
 				int			rows,
 				int			columns );
 
-extern void tedParaScreenGeometry(	BufferItem *		bi,
-					const AppDrawingData *	add );
+extern void tedParaScreenGeometry(	const BufferItem *		bi,
+					const DocumentProperties *	dp,
+					const AppDrawingData *		add );
 
 extern int tedInsertRowsInTable(	BufferItem *		sectBi,
 					const RowProperties *	rp,
@@ -833,7 +850,6 @@ extern int tedAdjustParagraphLayout( BufferItem *		bi,
 				int				line,
 				int				stroffShift,
 				int				upto,
-				int				inHeaderFooter,
 				AppDrawingData *		add,
 				const BufferDocument *		bd,
 				DocumentRectangle *		drChanged );
@@ -875,6 +891,8 @@ extern void tedAppDeleteHeaderFooter(	EditApplication *	ea,
 					int			which );
 
 extern void tedAppEditHeaderFooter(	EditApplication *	ea,
+					APP_WIDGET		relative,
+					APP_WIDGET		option,
 					int			which );
 
 extern int tedGetSelection(	SelectionScope *	ss,
