@@ -205,9 +205,9 @@ void tedFreeDocument(		void *			voidtd,
 	docFreeDocument( bd );
 	}
 
-    sioMemoryClean( &(td->tdCopiedSelection) );
-    sioMemoryClean( &(td->tdCopiedFont) );
-    sioMemoryClean( &(td->tdCopiedRuler) );
+    utilCleanMemoryBuffer( &(td->tdCopiedSelection) );
+    utilCleanMemoryBuffer( &(td->tdCopiedFont) );
+    utilCleanMemoryBuffer( &(td->tdCopiedRuler) );
 
     appCleanBitmapImage( &(td->tdCopiedImage) );
 
@@ -255,7 +255,7 @@ static int tedDetermineCodepage(	BufferDocument *	bd,
     int				i;
 
     int				encoding= -1;
-    int				multipleEncodings= 0;
+    int				encodingCount= 0;
 
     if  ( docPsPrintGetItemFonts( &(bd->bdItem),
 				&(dp->dpFontList), &(add->addPhysicalFontList),
@@ -270,14 +270,14 @@ static int tedDetermineCodepage(	BufferDocument *	bd,
 	    { encoding= afi->afiEncoding;	}
 	else{
 	    if  ( encoding != afi->afiEncoding )
-		{ multipleEncodings= 1;	}
+		{ encodingCount++; }
 	    }
 	}
 
     if  ( fontList )
 	{ free( fontList );	}
 
-    if  ( multipleEncodings )
+    if  ( encodingCount != 1 )
 	{ return -1;	}
 
     switch( encoding )
@@ -304,7 +304,7 @@ int tedNewDocument(	EditApplication *	ea,
 			EditDocument *		ed,
 			const char *		filename )
     {
-    TedDocument *		td;
+    TedDocument *		td= (TedDocument *)ed->edPrivateData;
     BufferDocument *		bd;
 
     unsigned int		updMask;
@@ -316,16 +316,36 @@ int tedNewDocument(	EditApplication *	ea,
     long			now;
     DocumentProperties *	dp;
 
-    td= (TedDocument *)ed->edPrivateData;
+    TedAppResources *		tar= (TedAppResources *)ea->eaResourceData;
+
+    if  ( tar->tarDefaultAnsicpgInt < 0		&&
+	  tar->tarDefaultAnsicpgString		)
+	{
+	char *		past;
+	int		val;
+
+	val= strtol( tar->tarDefaultAnsicpgString, &past, 10 );
+	if  ( past != tar->tarDefaultAnsicpgString )
+	    {
+	    while( *past == ' ' )
+		{ past++;	}
+	    }
+	if  ( past != tar->tarDefaultAnsicpgString	&&
+	      ! *past					)
+	    { tar->tarDefaultAnsicpgInt= val;		}
+	else{ SDEB(tar->tarDefaultAnsicpgString);	}
+	}
 
     strcpy( scratch, ea->eaDefaultFont );
     if  ( appFontGetAttributes( scratch, &familyName, &updMask, &taNew ) )
 	{ SDEB(ea->eaDefaultFont); return -1;	}
 
-    if  ( ! ( updMask & TAupdFONTSIZE ) || taNew.taFontSizeHalfPoints < 4 )
+    if  ( ! PROPmaskISSET( updMask, TApropFONTSIZE )	||
+	  taNew.taFontSizeHalfPoints < 6		)
 	{
 	LDEB(taNew.taFontSizeHalfPoints);
-	taNew.taFontSizeHalfPoints= 24; updMask |= TAupdFONTSIZE;
+	taNew.taFontSizeHalfPoints= 24;
+	PROPmaskADD( &updMask, TApropFONTSIZE );
 	}
 
     bd= docNewFile( familyName, 10* taNew.taFontSizeHalfPoints,
@@ -333,9 +353,10 @@ int tedNewDocument(	EditApplication *	ea,
     if  ( ! bd )
 	{ XDEB(bd); return -1;	}
 
-    ed->edFormat= 0; /* rtf */
+    ed->edFormat= TEDdockindRTF; /* rtf */
     td->tdDocument= bd;
     dp= &(bd->bdProperties);
+    dp->dpAnsiCodePage= tar->tarDefaultAnsicpgInt;
 
     if  ( filename )
 	{
@@ -347,9 +368,9 @@ int tedNewDocument(	EditApplication *	ea,
 	else{ dot= strrchr( filename, '.' );	}
 
 	if  ( dot && ! strcmp( dot, ".rtf" ) )
-	    { ed->edFormat= 0;	}
+	    { ed->edFormat= TEDdockindRTF;	}
 	if  ( dot && ! strcmp( dot, ".txt" ) )
-	    { ed->edFormat= 1;	}
+	    { ed->edFormat= TEDdockindTEXT_SAVE_FOLDED;	}
 	}
 
     if  ( ea->eaAuthor )
@@ -375,27 +396,29 @@ int tedNewDocument(	EditApplication *	ea,
 int tedLayoutDocument(		void *				privateData,
 				int				format,
 				AppDrawingData *		add,
-				const DocumentGeometry *	defDg )
+				const DocumentGeometry *	dgDef )
     {
     TedDocument *		td= (TedDocument *)privateData;
     BufferDocument *		bd= td->tdDocument;
-    DocumentGeometry *		dg= &(bd->bdProperties.dpGeometry);
+    const DocumentProperties *	dp= &(bd->bdProperties);
+    const DocumentGeometry *	dgDoc= &(dp->dpGeometry);
 
     int				changed= 0;
 
-    tedScreenRectangles( add, dg );
+    tedScreenRectangles( add, dgDoc );
 
-    if  ( docRecalculateTextFields( &changed, bd, &(bd->bdItem),
-						    FIELDdoDOC_INFO |
-						    FIELDdoDOC_COMPLETE ) )
+    if  ( docRecalculateTextLevelFields( &changed, bd, &(bd->bdItem),
+				    FIELDdoDOC_INFO|FIELDdoDOC_COMPLETE,
+				    (void *)add, tedCloseObject ) )
 	{ LDEB(1); return -1;	}
 
     if  ( tedLayoutDocumentTree( td, add ) )
 	{ LDEB(1); return -1;	}
 
     changed= 0;
-    if  ( docRecalculateTextFields( &changed, bd, &(bd->bdItem),
-						    FIELDdoDOC_FORMATTED ) )
+    if  ( docRecalculateTextLevelFields( &changed, bd, &(bd->bdItem),
+				    FIELDdoDOC_FORMATTED,
+				    (void *)add, tedCloseObject ) )
 	{ LDEB(1); return -1;	}
 
     if  ( changed && tedLayoutDocumentTree( td, add ) )
@@ -419,9 +442,9 @@ int tedLayoutDocument(		void *				privateData,
 	td->tdSelection.bsDirection= 0;
 
 	tedSelectionRectangle( &(td->tdSelectedRectangle),
-						 add, &(td->tdSelection) );
+						 add, dp, &(td->tdSelection) );
 	}
-    else{ docListItem( 0,&(bd->bdItem) );	}
+    else{ docListItem( 0, &(bd->bdItem) );	}
     }
 
     /*  8  */
@@ -453,8 +476,11 @@ int tedSaveDocument(	const void *		privateData,
     BufferDocument *		bd= td->tdDocument;
     DocumentProperties *	dp= &(bd->bdProperties);
     SimpleOutputStream *	sos;
+
     const int			saveBookmarks= 1;
 
+    const int			asMimeAggr= 0;
+
     long			now;
 
     sos= sioOutStdioOpen( filename );
@@ -463,7 +489,7 @@ int tedSaveDocument(	const void *		privateData,
 
     switch( format )
 	{
-	case 0:
+	case TEDdockindRTF:
 	    now= time( (long *)0 );
 	    dp->dpRevtim= *localtime( &now );
 
@@ -474,7 +500,11 @@ int tedSaveDocument(	const void *		privateData,
 		{ SDEB(filename); sioOutClose( sos ); return -1;	}
 	    break;
 
-	case 1:
+	case TEDdockindTEXT_OPEN:
+	    LDEB(format);
+	    /*FALLTHROUGH*/
+
+	case TEDdockindTEXT_SAVE_FOLDED:
 	    if  ( docPlainSaveDocument( sos, bd,
 					    (const SelectionScope *)0,
 					    (const BufferSelection *)0,
@@ -482,8 +512,17 @@ int tedSaveDocument(	const void *		privateData,
 		{ SDEB(filename); sioOutClose( sos ); return -1;	}
 	    break;
 
-	case 2:
-	    if  ( docHtmlSaveDocument( sos, bd, filename ) )
+	case TEDdockindTEXT_SAVE_WIDE:
+	    if  ( docPlainSaveDocument( sos, bd,
+					    (const SelectionScope *)0,
+					    (const BufferSelection *)0,
+					    0, 1 ) )
+		{ SDEB(filename); sioOutClose( sos ); return -1;	}
+	    break;
+
+	case TEDdockindHTML_FILES:
+	    if  ( docHtmlSaveDocument( sos, bd, asMimeAggr,
+						(const char *)0, filename ) )
 		{ SDEB(filename); sioOutClose( sos ); return -1;	}
 	    break;
 
@@ -566,9 +605,9 @@ void * tedMakePrivateData()
     td->tdCanReplaceSelection= 0;
 
     td->tdCopiedSelectionClosed= 0;
-    sioMemoryInit( &(td->tdCopiedSelection) );
-    sioMemoryInit( &(td->tdCopiedFont) );
-    sioMemoryInit( &(td->tdCopiedRuler) );
+    utilInitMemoryBuffer( &(td->tdCopiedSelection) );
+    utilInitMemoryBuffer( &(td->tdCopiedFont) );
+    utilInitMemoryBuffer( &(td->tdCopiedRuler) );
 
     appInitBitmapImage( &(td->tdCopiedImage) );
 
@@ -618,8 +657,8 @@ APP_EVENT_HANDLER( tedObserveFocus, w, voided, event )
 	    if  ( ea->eaFontsTool )
 		{ tedAdaptFontsToolToDocument( ea, ed ); }
 
-	    if  ( ea->eaFindTool )
-		{ appFindToolTextReadonly( ea->eaFindTool, ed->edIsReadonly ); }
+	    if  ( ea->eaFindTool && ed->edIsReadonly )
+		{ appFindToolDisableReplace( ea->eaFindTool ); }
 
 	    if  ( ea->eaSpellTool )
 		{ appEnableSpellTool( ea->eaSpellTool, ! ed->edIsReadonly ); }
@@ -707,13 +746,25 @@ int tedFollowLink(	APP_WIDGET		relative,
 			const char *		markName,
 			int			markSize )
     {
-    int			rval= 0;
-    EditApplication *	ea= edFrom->edApplication;
+    int				rval= 0;
+    EditApplication *		ea= edFrom->edApplication;
 
-    char *		slash= (char *)0;
-    EditDocument *	edTo;
+    char *			slash= (char *)0;
+    EditDocument *		edTo;
 
-    char *		scratch= (char *)0;
+    char *			scratch= (char *)0;
+
+#   if 0
+    TedDocument *		tdFrom= (TedDocument *)edFrom->edPrivateData;
+    BufferDocument *		bdFrom= tdFrom->tdDocument;
+    const DocumentProperties *	dpFrom= &(bdFrom->bdProperties);
+
+    const char *		base= (const char *)dpFrom->dpHlinkbase;
+    int				baseLen= 0;
+
+    if  ( base )
+	{ baseLen= strlen( base );	}
+#   endif
 
     if  ( fileSize > 0 )
 	{
@@ -813,21 +864,18 @@ int tedPrintDocument(	FILE *				f,
     now= time( (long *)0 );
     dp->dpPrintim= *localtime( &now );
 
-    if  ( docRecalculateTextFields( &changed, bd, &(bd->bdItem),
-						FIELDdoDOC_FORMATTED	|
-						FIELDdoDOC_COMPLETE	|
-						FIELDdoDOC_INFO		) )
+    if  ( docRecalculateTextLevelFields( &changed, bd, &(bd->bdItem),
+		    FIELDdoDOC_FORMATTED|FIELDdoDOC_COMPLETE|FIELDdoDOC_INFO,
+		    (void *)add, tedCloseObject ) )
 	{ LDEB(1); return -1;	}
 
     if  ( changed && tedLayoutDocumentTree( td, add ) )
 	{ LDEB(1); return -1;	}
 
     if  ( docPsPrintDocument( f, title, ea->eaApplicationName,
-				ea->eaReference,
-				add, td->tdDocument,
+				ea->eaReference, add, td->tdDocument,
 				nup, horizontal, firstPage, lastPage,
-				dgPaper,
-				tedLayoutHeaderItem, tedLayoutFooterItem ) )
+				dgPaper, tedLayoutExternal, tedCloseObject ) )
 	{ LDEB(1); return -1;	}
 
     return 0;
