@@ -11,8 +11,8 @@
 #   include	<stdio.h>
 
 #   include	"tedApp.h"
+#   include	"docPs.h"
 #   include	"docDraw.h"
-#   include	<X11/cursorfont.h>
 
 #   include	<appDebugon.h>
 
@@ -429,8 +429,7 @@ static int tedDrawParticules(	const BufferItem *	bi,
 
 static int tedDrawTextLine(	const BufferItem *		bi,
 				int				line,
-				const FormattingFrame *		ign_ffTwips,
-				const FormattingFrame *		ffPixels,
+				const FormattingFrame *		ff,
 				void *				vsdd,
 				DrawingContext *		dc )
     {
@@ -439,8 +438,8 @@ static int tedDrawTextLine(	const BufferItem *		bi,
     const TextLine *		tl= bi->biParaLines+ line;
     const TextParticule *	tp= bi->biParaParticules+ tl->tlFirstParticule;
 
-    AppDrawingData *	add= dc->dcDrawingData;
-    int			done;
+    AppDrawingData *		add= dc->dcDrawingData;
+    int				done;
 
     done= 0;
     while( done < tl->tlParticuleCount )
@@ -452,7 +451,7 @@ static int tedDrawTextLine(	const BufferItem *		bi,
 	    { tedDrawSetForeground( add, sdd, SDDpixelLINK );	}
 	else{ tedDrawSetForeground( add, sdd, SDDpixelFORE );	}
 
-	baseline= tl->tlTopPosition.lpYPixels+ tl->tlLineAscentPixels;
+	baseline= TL_BASE_PIXELS( add, tl );
 
 	drawn= tedDrawParticules( bi, tp, tl->tlParticuleCount- done,
 							baseline, sdd, dc );
@@ -470,10 +469,9 @@ static int tedLineRectangle(	const BufferItem *		bi,
 				int				line,
 				const BufferSelection *		bs,
 				const DocumentRectangle *	drClip,
-				const FormattingFrame *		ffPixels,
+				const FormattingFrame *		ff,
 				DocumentRectangle *		drRedraw )
     {
-    double		xfac= add->addMagnifiedPixelsPerTwip;
     BufferPosition	bpLineBegin;
     BufferPosition	bpLineEnd;
 
@@ -485,21 +483,20 @@ static int tedLineRectangle(	const BufferItem *		bi,
 
     docLinePositions( &bpLineBegin, &bpLineEnd, bi, line );
 
-    drLine.drY0= tl->tlTopPosition.lpYPixels;
-    drLine.drY1= tl->tlTopPosition.lpYPixels+
-				TWIPStoPIXELS( xfac, tl->tlLineSpacingTwips );
+    drLine.drY0= TL_TOP_PIXELS( add, tl );
+    drLine.drY1= TL_BELOW_PIXELS( add, tl );
 
     if  ( docComparePositions( &(bs->bsBegin), &bpLineBegin, mindLine ) < 0 )
 	{
-	drLine.drX0= ffPixels->ffX0TextLines;
+	drLine.drX0= ff->ffX0TextLinesPixels;
 
 	if  ( bi->biParaFirstIndentTwips < 0 )
-	    { drLine.drX0= ffPixels->ffX0FirstLine;	}
+	    { drLine.drX0= ff->ffX0FirstLinePixels;	}
 	}
     else{ drLine.drX0= bs->bsBegin.bpXPixels;	}
 
     if  ( docComparePositions( &(bs->bsEnd), &bpLineEnd, mindLine ) > 0 )
-	{ drLine.drX1= ffPixels->ffX1TextLines;	}
+	{ drLine.drX1= ff->ffX1TextLinesPixels;		}
     else{ drLine.drX1= bs->bsEnd.bpXPixels;		}
 
     if  ( ! drClip || docIntersectRectangle( drRedraw, drClip, &drLine ) )
@@ -509,8 +506,7 @@ static int tedLineRectangle(	const BufferItem *		bi,
 
 static int tedDrawTextReverse(	const BufferItem *		bi,
 				int				line,
-				const FormattingFrame *		ign_ffTwips,
-				const FormattingFrame *		ffPixels,
+				const FormattingFrame *		ff,
 				void *				vsdd,
 				DrawingContext *		dc )
     {
@@ -522,7 +518,7 @@ static int tedDrawTextReverse(	const BufferItem *		bi,
     int				done;
 
     if  ( tedLineRectangle( bi, add, line, bs,
-					dc->dcClipRect, ffPixels, &drRedraw ) )
+					    dc->dcClipRect, ff, &drRedraw ) )
 	{
 	drRedraw.drX0 -= sdd->sddOx;
 	drRedraw.drX1 -= sdd->sddOx;
@@ -531,7 +527,7 @@ static int tedDrawTextReverse(	const BufferItem *		bi,
 
 	appDrawSetClipRect( add, &drRedraw );
 
-	done= tedDrawTextLine( bi, line, ign_ffTwips, ffPixels, vsdd, dc );
+	done= tedDrawTextLine( bi, line, ff, vsdd, dc );
 	}
     else{
 	TextLine *	tl= bi->biParaLines+ line;
@@ -544,8 +540,7 @@ static int tedDrawTextReverse(	const BufferItem *		bi,
 
 static int tedHighlightBack(	const BufferItem *		bi,
 				int				line,
-				const FormattingFrame *		ign_ffTwips,
-				const FormattingFrame *		ffPixels,
+				const FormattingFrame *		ff,
 				void *				vsdd,
 				DrawingContext *		dc )
     {
@@ -558,7 +553,7 @@ static int tedHighlightBack(	const BufferItem *		bi,
     DocumentRectangle		drRedraw;
 
     if  ( tedLineRectangle( bi, add, line, bs,
-					dc->dcClipRect, ffPixels, &drRedraw ) )
+					    dc->dcClipRect, ff, &drRedraw ) )
 	{
 	tedDrawSetForeground( add, sdd, SDDpixelBACK );
 
@@ -584,8 +579,7 @@ static int tedHighlightBack(	const BufferItem *		bi,
 
 static int tedDrawParaBottom(	const BufferItem *		bi,
 				const BorderProperties *	bp,
-				const FormattingFrame *		ign_ffTwips,
-				const FormattingFrame *		ffPixels,
+				const FormattingFrame *		ff,
 				void *				vsdd,
 				DrawingContext *		dc )
     {
@@ -600,16 +594,15 @@ static int tedDrawParaBottom(	const BufferItem *		bi,
 
     spaceAfterPixels= TWIPStoPIXELS( xfac, bi->biParaSpaceAfterTwips );
 
-    x0= ffPixels->ffX0TextLines;
+    x0= ff->ffX0TextLinesPixels;
     if  ( bi->biParaFirstIndentTwips < 0 )
-	{ x0= ffPixels->ffX0FirstLine;	}
+	{ x0= ff->ffX0FirstLinePixels;	}
 
     tedDrawSetForeground( add, sdd, SDDpixelFORE );
 
     tedDrawHorizontalBorder( bp, asGrid, dc, sdd, 1,
-				x0, ffPixels->ffX1TextLines,
-				bi->biBelowPosition.lpYPixels-
-				spaceAfterPixels -1 );
+			    x0, ff->ffX1TextLinesPixels,
+			    BI_BELOW_PIXELS( add, bi )- spaceAfterPixels -1 );
 
     return 0;
     }
@@ -617,8 +610,7 @@ static int tedDrawParaBottom(	const BufferItem *		bi,
 
 static int tedDrawParaTop(	const BufferItem *		bi,
 				const BorderProperties *	bp,
-				const FormattingFrame *		ign_ffTwips,
-				const FormattingFrame *		ffPixels,
+				const FormattingFrame *		ff,
 				void *				vsdd,
 				DrawingContext *		dc )
     {
@@ -633,16 +625,15 @@ static int tedDrawParaTop(	const BufferItem *		bi,
 
     spaceBeforePixels= TWIPStoPIXELS( xfac, bi->biParaSpaceBeforeTwips );
 
-    x0= ffPixels->ffX0TextLines;
+    x0= ff->ffX0TextLinesPixels;
     if  ( bi->biParaFirstIndentTwips < 0 )
-	{ x0= ffPixels->ffX0FirstLine;	}
+	{ x0= ff->ffX0FirstLinePixels;	}
 
     tedDrawSetForeground( add, sdd, SDDpixelFORE );
 
     tedDrawHorizontalBorder( bp, asGrid, dc, sdd, 0,
-					    x0, ffPixels->ffX1TextLines,
-					    bi->biTopPosition.lpYPixels+
-					    spaceBeforePixels );
+				x0, ff->ffX1TextLinesPixels,
+				BI_TOP_PIXELS( add, bi )+ spaceBeforePixels );
     return 0;
     }
 
@@ -664,7 +655,8 @@ static int tedDrawCellTop(	const BorderProperties *	bp,
     else{ tedDrawSetForeground( add, sdd, SDDpixelFORE );	}
 
     tedDrawHorizontalBorder( bp, asGrid, dc, sdd, 1,
-				    x0Pixels, x1Pixels, lpTop->lpYPixels );
+				    x0Pixels, x1Pixels,
+				    LP_YPIXELS( add, lpTop ) );
 
     return 0;
     }
@@ -686,7 +678,8 @@ static int tedDrawCellLeft(	const BorderProperties *	bp,
     else{ tedDrawSetForeground( add, sdd, SDDpixelFORE );	}
 
     tedDrawVerticalBorder( bp, asGrid, dc, sdd, x0Pixels,
-				    lpTop->lpYPixels, lpBelow->lpYPixels );
+				    LP_YPIXELS( add, lpTop ),
+				    LP_YPIXELS( add, lpBelow ) );
 
     return 0;
     }
@@ -708,7 +701,8 @@ static int tedDrawCellRight(	const BorderProperties *	bp,
     else{ tedDrawSetForeground( add, sdd, SDDpixelFORE );	}
 
     tedDrawVerticalBorder( bp, asGrid, dc, sdd, x1Pixels,
-				    lpTop->lpYPixels, lpBelow->lpYPixels );
+				    LP_YPIXELS( add, lpTop ),
+				    LP_YPIXELS( add, lpBelow ) );
 
     return 0;
     }
@@ -731,7 +725,8 @@ static int tedDrawCellBottom(	const BorderProperties *	bp,
     else{ tedDrawSetForeground( add, sdd, SDDpixelFORE );	}
 
     tedDrawHorizontalBorder( bp, asGrid, dc, sdd, 0,
-				    x0Pixels, x1Pixels, lpBottom->lpYPixels );
+				    x0Pixels, x1Pixels,
+				    LP_YPIXELS( add, lpBottom ) );
 
     return 0;
     }
@@ -782,9 +777,10 @@ static int tedDrawIBar(	const BufferPosition *	bp,
 			int			oy,
 			AppDrawingData *	add )
     {
-    int		y0= bp->bpTopPosition.lpYPixels;
+    int		y0= BP_TOP_PIXELS( add, bp );
 
-    appDrawFillRectangle( add, bp->bpXPixels- ox, y0- oy, 1, bp->bpY1Pixels- y0 );
+    appDrawFillRectangle( add, bp->bpXPixels- ox, y0- oy,
+						1, bp->bpY1Pixels- y0 );
 
     return 0;
     }
@@ -1006,12 +1002,12 @@ void tedRedrawRectangle(	APP_WIDGET		w,
 	dc.dcDrawCellBottom= (DRAW_CELL_BOTTOM)0;
 	dc.dcDrawCellLeft= (DRAW_CELL_LEFT)0;
 	dc.dcDrawCellRight= (DRAW_CELL_RIGHT)0;
-	dc.dcParaSidesPixels= tedParagraphFramePixels;
+	dc.dcParaFramePixels= tedParagraphFramePixels;
 	dc.dcFinishPage= (FINISH_PAGE)0;
 	dc.dcStartPage= (START_PAGE)0;
 
-	dc.dcLayoutHeader= (LAYOUT_EXTERNAL)0;
-	dc.dcLayoutFooter= (LAYOUT_EXTERNAL)0;
+	dc.dcLayoutExternal= (LAYOUT_EXTERNAL)0;
+	dc.dcCloseObject= tedCloseObject;
 	dc.dcDrawHeadersFooters= 0;
 
 	dc.dcSelection= &bs;
@@ -1033,10 +1029,10 @@ void tedRedrawRectangle(	APP_WIDGET		w,
     dc.dcDrawCellBottom= tedDrawCellBottom;
     dc.dcDrawCellLeft= tedDrawCellLeft;
     dc.dcDrawCellRight= tedDrawCellRight;
-    dc.dcParaSidesPixels= tedParagraphFramePixels;
+    dc.dcParaFramePixels= tedParagraphFramePixels;
 
-    dc.dcLayoutHeader= tedLayoutHeaderItem;
-    dc.dcLayoutFooter= tedLayoutFooterItem;
+    dc.dcLayoutExternal= tedLayoutExternal;
+    dc.dcCloseObject= tedCloseObject;
     dc.dcDrawHeadersFooters= 1;
 
     dc.dcSelection= (BufferSelection *)0;
@@ -1065,8 +1061,8 @@ void tedRedrawRectangle(	APP_WIDGET		w,
 	{ y0= 0;	}
 
     y1= drClip->drY1+ add->addPageStepPixels- 1;
-    if  ( y1 > docBi->biBelowPosition.lpYPixels )
-	{ y1=  docBi->biBelowPosition.lpYPixels;	}
+    if  ( y1 > BI_BELOW_PIXELS( add, docBi ) )
+	{ y1=  BI_BELOW_PIXELS( add, docBi );	}
 
     firstPage= y0/ add->addPageStepPixels;
     lastPage=  y1/ add->addPageStepPixels;
@@ -1080,7 +1076,10 @@ void tedRedrawRectangle(	APP_WIDGET		w,
     for ( sect= 0; sect < docBi->biChildCount; sect++ )
 	{
 	BufferItem *	sectBi= docBi->biChildren[sect];
+	/*
+	See below
 	BufferItem *	nextSectBi= (BufferItem *)0;
+	*/
 	BufferItem *	prevSectBi= (BufferItem *)0;
 	int		first= sectBi->biTopPosition.lpPage;
 	int		last= sectBi->biBelowPosition.lpPage;
@@ -1092,8 +1091,11 @@ void tedRedrawRectangle(	APP_WIDGET		w,
 
 	if  ( sect > 0 )
 	    { prevSectBi= docBi->biChildren[sect- 1];	}
+	/*
+	Word draws the footer of the first section on the page.
 	if  ( sect < docBi->biChildCount- 1 )
 	    { nextSectBi= docBi->biChildren[sect+ 1];	}
+	*/
 
 	for ( page= first; page <= last; page++ )
 	    {
@@ -1101,9 +1103,16 @@ void tedRedrawRectangle(	APP_WIDGET		w,
 		  prevSectBi->biBelowPosition.lpPage < page	)
 		{ docDrawPageHeader( sectBi, (void *)&sdd, &dc, page ); }
 
+	    if  ( ! prevSectBi					||
+		  prevSectBi->biBelowPosition.lpPage < page	)
+		{ docDrawPageFooter( sectBi, (void *)&sdd, &dc, page ); }
+
+	    /*
+	    Word draws the footer of the first section on the page.
 	    if  ( ! nextSectBi					||
 		  nextSectBi->biTopPosition.lpPage > page	)
 		{ docDrawPageFooter( sectBi, (void *)&sdd, &dc, page ); }
+	    */
 
 	    if  ( ss.ssInHeaderFooter != DOCinBODY		&&
 		  page == ss.ssHeaderFooterPage			&&
@@ -1136,12 +1145,12 @@ void tedRedrawRectangle(	APP_WIDGET		w,
 	dc.dcDrawCellBottom= (DRAW_CELL_BOTTOM)0;
 	dc.dcDrawCellLeft= (DRAW_CELL_LEFT)0;
 	dc.dcDrawCellRight= (DRAW_CELL_RIGHT)0;
-	dc.dcParaSidesPixels= tedParagraphFramePixels;
+	dc.dcParaFramePixels= tedParagraphFramePixels;
 	dc.dcFinishPage= (FINISH_PAGE)0;
 	dc.dcStartPage= (START_PAGE)0;
 
-	dc.dcLayoutHeader= (LAYOUT_EXTERNAL)0;
-	dc.dcLayoutFooter= (LAYOUT_EXTERNAL)0;
+	dc.dcLayoutExternal= (LAYOUT_EXTERNAL)0;
+	dc.dcCloseObject= tedCloseObject;
 	dc.dcDrawHeadersFooters= 0;
 
 	dc.dcSelection= &bs;
@@ -1159,7 +1168,7 @@ void tedRedrawRectangle(	APP_WIDGET		w,
 	const BufferPosition *	bpBegin= &(bs.bsBegin);
 
 	if  ( drClip->drY0 <= bpBegin->bpY1Pixels		&&
-	      drClip->drY1 >= bpBegin->bpTopPosition.lpYPixels	&&
+	      drClip->drY1 >= BP_TOP_PIXELS( add, bpBegin )	&&
 	      ! td->tdShowIBarId				)
 	    { tedDrawIBar( bpBegin, ox, oy, add ); }
 	}
@@ -1180,7 +1189,7 @@ void tedRedrawRectangle(	APP_WIDGET		w,
 	    tedPositionCoordinates( &bp, add );
 
 	    if  ( drClip->drY0 <= bp.bpY1Pixels			&&
-		  drClip->drY1 >= bp.bpTopPosition.lpYPixels	)
+		  drClip->drY1 >= BP_TOP_PIXELS( add, &bp )	)
 		{
 		if  ( io->ioDragWide > 0 )
 		    {
@@ -1270,10 +1279,14 @@ void tedExposeSelection(	const EditDocument *	ed,
 				int			scrolledX,
 				int			scrolledY )
     {
+    TedDocument *		td= (TedDocument *)ed->edPrivateData;
+    BufferDocument *		bd= td->tdDocument;
+    const DocumentProperties *	dp= &(bd->bdProperties);
+
     const AppDrawingData *	add= &(ed->edDrawingData);
     DocumentRectangle		drClip;
 
-    tedSelectionRectangle( &drClip, add, bs );
+    tedSelectionRectangle( &drClip, add, dp, bs );
 
     tedExposeRectangle( ed, &drClip, scrolledX, scrolledY );
 
@@ -1294,13 +1307,14 @@ void tedUndrawIBar(	const EditDocument *	ed )
     /*  1  */
 #   else
     TedDocument *		td= (TedDocument *)ed->edPrivateData;
+    const AppDrawingData *	add= &(ed->edDrawingData);
     const BufferPosition *	bp= &(td->tdSelection.bsBegin);
 
     DocumentRectangle		drIBar;
 
     drIBar.drX0= bp->bpXPixels- IW;
     drIBar.drX1= bp->bpXPixels+ IW;
-    drIBar.drY0= bp->bpTopPosition.lpYPixels;
+    drIBar.drY0= BP_TOP_PIXELS( add, bp );
     drIBar.drY1= bp->bpY1Pixels;
 
     tedExposeRectangle( ed, &drIBar, /*scrolledX,Y*/ 0,0 );
