@@ -11,6 +11,7 @@
 #   include	<stdio.h>
 #   include	<ctype.h>
 
+#   include	"docPs.h"
 #   include	"tedApp.h"
 
 #   include	<appDebugon.h>
@@ -142,12 +143,15 @@ void tedDeleteItems(	BufferDocument *	bd,
 /************************************************************************/
 
 /*  B  */
-static void tedAdjustRedrawBegin( TedDocument *			td,
-				const DocumentRectangle *	drBack,
+static void tedAdjustRedrawBegin( EditDocument *		ed,
 				DocumentRectangle *		drChanged,
 				int *				pLine,
 				int				part )
     {
+    AppDrawingData *		add= &(ed->edDrawingData);
+    const DocumentRectangle *	drBack= &(add->addBackRect);
+    TedDocument *		td= (TedDocument *)ed->edPrivateData;
+
     TextLine *	tl= td->tdSelection.bsBegin.bpBi->biParaLines+ *pLine;
 
     if  ( *pLine == 0 )
@@ -157,8 +161,8 @@ static void tedAdjustRedrawBegin( TedDocument *			td,
 	    drChanged->drX0= drBack->drX0;
 	    drChanged->drX1= drBack->drX1;
 
-	    if  ( drChanged->drY0 > tl->tlTopPosition.lpYPixels )
-		{ drChanged->drY0=  tl->tlTopPosition.lpYPixels;	}
+	    if  ( drChanged->drY0 > TL_TOP_PIXELS( add, tl ) )
+		{ drChanged->drY0=  TL_TOP_PIXELS( add, tl ); }
 	    }
 
 	return;
@@ -172,8 +176,8 @@ static void tedAdjustRedrawBegin( TedDocument *			td,
 	drChanged->drX0= drBack->drX0;
 	drChanged->drX1= drBack->drX1;
 
-	if  ( drChanged->drY0 > tl->tlTopPosition.lpYPixels )
-	    { drChanged->drY0=  tl->tlTopPosition.lpYPixels;	}
+	if  ( drChanged->drY0 > TL_TOP_PIXELS( add, tl ) )
+	    { drChanged->drY0=  TL_TOP_PIXELS( add, tl ); }
 	}
 
     return;
@@ -228,14 +232,14 @@ int tedReplaceSelection(	DocumentRectangle *	drChanged,
 	{ LDEB(addedLength); return -1;	}
 
     /*  B  */
-    tedAdjustRedrawBegin( td, &(add->addBackRect), drChanged, &line, part );
+    tedAdjustRedrawBegin( ed, drChanged, &line, part );
 
     if  ( paragraphsMerged )
 	{ bi->biBelowPosition= lpBelow; }
 
     /*  C  */
     if  ( tedAdjustParagraphLayout( bi, line, stroffShift, upto+ addedLength,
-				ss->ssInHeaderFooter, add, bd, drChanged ) )
+						    add, bd, drChanged ) )
 	{ LDEB(line); }
 
     if  ( paragraphsMerged )
@@ -442,6 +446,9 @@ static void tedMergeParagraphsInSelectionLow(	EditDocument *	ed,
 	appSetShellConstraints( ed );
 	}
 
+    if  ( ed->edFormat == TEDdockindTEXT_SAVE_FOLDED )
+	{ ed->edFormat= TEDdockindTEXT_SAVE_WIDE;	}
+
     return;
     }
 
@@ -747,7 +754,7 @@ static int tedIncludeHead(	BufferItem *		biTo,
 				const BufferItem *	biFrom,
 				const int		part,
 				const int		stroff,
-				TedDocument *		tdTo,
+				EditDocument *		edTo,
 				BufferDocument *	bdFrom,
 				int *			fieldMap,
 				unsigned int *		pFieldUpd,
@@ -759,8 +766,6 @@ static int tedIncludeHead(	BufferItem *		biTo,
 				const AppDrawingData *	add,
 				const char *		refFileName )
     {
-    const DocumentRectangle *	drBack= &(add->addBackRect);
-
     TextParticule *		tpTo;
     TextParticule *		tpFrom;
 
@@ -771,7 +776,9 @@ static int tedIncludeHead(	BufferItem *		biTo,
     int				particulesCopied;
     int				particuleSplit= 0;
 
+    TedDocument *		tdTo= (TedDocument *)edTo->edPrivateData;
     BufferDocument *		bdTo= tdTo->tdDocument;
+    const DocumentProperties *	dpTo= &(bdTo->bdProperties);
 
     tpTo= biTo->biParaParticules+ tdTo->tdSelection.bsBegin.bpParticule;
     tpFrom= biFrom->biParaParticules;
@@ -779,17 +786,25 @@ static int tedIncludeHead(	BufferItem *		biTo,
     if  ( biTo->biParaStrlen == 0 )
 	{
 	int		toInTable= biTo->biParaInTable;
+	unsigned int	ppUpdMask= 0;
 
-	if  ( docCopyParagraphRuler( &biTo->biParaProperties,
-					    &biFrom->biParaProperties )	)
+	PROPmaskCLEAR( &ppUpdMask );
+	PROPmaskADD( &ppUpdMask, PPpropLEFT_INDENT );
+	PROPmaskADD( &ppUpdMask, PPpropFIRST_INDENT );
+	PROPmaskADD( &ppUpdMask, PPpropRIGHT_INDENT );
+	PROPmaskADD( &ppUpdMask, PPpropALIGNMENT );
+	PROPmaskADD( &ppUpdMask, PPpropTAB_STOPS );
+
+	if  ( docUpdParaProperties( &ppUpdMask, &(biTo->biParaProperties),
+				ppUpdMask, &(biFrom->biParaProperties) ) )
 	    { LDEB(1); }
 
 	biTo->biParaInTable= toInTable;
-	tedParaScreenGeometry( biTo, add );
+	tedParaScreenGeometry( biTo, dpTo, add );
 	}
 
     /*  B  */
-    tedAdjustRedrawBegin( tdTo, drBack, drChanged, pLine, partTo );
+    tedAdjustRedrawBegin( edTo, drChanged, pLine, partTo );
 
     /*  1  */
     particulesCopied= 0;
@@ -920,7 +935,7 @@ static int tedIncludeDocument(	EditDocument *		edTo,
     if  ( ! biFrom->biParaInTable )
 	{
 	if  ( tedIncludeHead( biTo, biFrom, part, stroff,
-		    tdTo, bdFrom, fieldMap, &fieldUpd,
+		    edTo, bdFrom, fieldMap, &fieldUpd,
 		    &partShift, &stroffShift, &line,
 		    &charactersCopied, &drChanged, add, edTo->edFilename ) )
 	    { LDEB(1); return -1;	}
@@ -994,7 +1009,7 @@ static int tedIncludeDocument(	EditDocument *		edTo,
 
 	/*  C  */
 	if  ( tedAdjustParagraphLayout( biTo, line, stroffShift, stroffEnd,
-		ssTo->ssInHeaderFooter, add, bdTo, &drChanged ) )
+						add, bdTo, &drChanged ) )
 	    { LDEB(1); }
 
 	if  ( tedOpenItemObjects( biTo, &edTo->edColors, add ) )
@@ -1005,8 +1020,9 @@ static int tedIncludeDocument(	EditDocument *		edTo,
 	{
 	int		changed= 0;
 
-	if  ( docRecalculateTextFields( &changed,
-					bdTo, &(bdTo->bdItem), fieldUpd ) )
+	if  ( docRecalculateTextLevelFields( &changed,
+					bdTo, &(bdTo->bdItem), fieldUpd,
+					(void *)add, tedCloseObject ) )
 	    { LDEB(1);	}
 
 	if  ( changed )
@@ -1131,6 +1147,7 @@ int tedSplitParaContents(	BufferItem **		pNewBi,
     TedDocument *		td= (TedDocument *)ed->edPrivateData;
     SelectionScope *		ss= &(td->tdSelectionScope);
     BufferDocument *		bd= td->tdDocument;
+    const DocumentProperties *	dp= &(bd->bdProperties);
 
     BufferItem *		newBi;
 
@@ -1145,20 +1162,20 @@ int tedSplitParaContents(	BufferItem **		pNewBi,
 
     newBi->biParaStartsOnNewPage= ! newBi->biParaInTable && onNewPage;
     
-    tedParaScreenGeometry( newBi, add );
+    tedParaScreenGeometry( newBi, dp, add );
 
     /*  3  */
     *drChanged= td->tdSelectedRectangle;
     drChanged->drX0= add->addBackRect.drX0;
     drChanged->drX1= add->addBackRect.drX1; 
 
-    if  ( bi->biParaBottomBorder.bpIsSet )
-	{ bi->biParaBottomBorder.bpIsSet= 0;	}
+    if  ( bi->biParaBottomBorder.bpStyle != DOCbsNONE )
+	{ bi->biParaBottomBorder.bpStyle=   DOCbsNONE;	}
     if  ( bi->biParaSpaceAfterTwips > 0 )
 	{ bi->biParaSpaceAfterTwips= 0;		}
 
-    if  ( newBi->biParaTopBorder.bpIsSet )
-	{ newBi->biParaTopBorder.bpIsSet= 0;	}
+    if  ( newBi->biParaTopBorder.bpStyle != DOCbsNONE )
+	{ newBi->biParaTopBorder.bpStyle=   DOCbsNONE;	}
     if  ( newBi->biParaSpaceBeforeTwips > 0 )
 	{ newBi->biParaSpaceBeforeTwips= 0;	}
 
@@ -1226,7 +1243,6 @@ void tedSetTab(		EditDocument *		ed )
     {
     AppDrawingData *		add= &(ed->edDrawingData);
     TedDocument *		td= (TedDocument *)ed->edPrivateData;
-    SelectionScope *		ss= &(td->tdSelectionScope);
     BufferDocument *		bd= td->tdDocument;
     TextParticule *		tp;
 
@@ -1262,7 +1278,6 @@ void tedSetTab(		EditDocument *		ed )
     /*  3,4,5  */
     if  ( tedAdjustParagraphLayout( bi, line, stroffShift,
 				td->tdSelection.bsBegin.bpStroff+ stroffShift,
-					    ss->ssInHeaderFooter,
 					    add, bd, &drChanged ) )
 	{ LDEB(line); }
 
@@ -1285,7 +1300,6 @@ int tedInsertNewObject(		APP_WIDGET		w,
     {
     AppDrawingData *		add= &(ed->edDrawingData);
     TedDocument *		td= (TedDocument *)ed->edPrivateData;
-    SelectionScope *		ss= &(td->tdSelectionScope);
     BufferDocument *		bd= td->tdDocument;
     TextParticule *		tp;
 
@@ -1323,7 +1337,7 @@ int tedInsertNewObject(		APP_WIDGET		w,
     /*  3,4,5  */
     if  ( tedAdjustParagraphLayout( bi, line, stroffShift,
 			    td->tdSelection.bsBegin.bpStroff+ stroffShift+ 1,
-			    ss->ssInHeaderFooter, add, bd, &drChanged ) )
+						    add, bd, &drChanged ) )
 	{ LDEB(line); }
 
     /*  6,7  */
@@ -1346,11 +1360,9 @@ int tedResizeObject(		EditDocument *		ed,
     {
     BufferItem *		bi= bp->bpBi;
     TedDocument *		td= (TedDocument *)ed->edPrivateData;
-    SelectionScope *		ss= &(td->tdSelectionScope);
     BufferDocument *		bd= td->tdDocument;
     DocumentProperties *	dp= &(bd->bdProperties);
     AppDrawingData *		add= &(ed->edDrawingData);
-    const DocumentRectangle *	drBack= &(add->addBackRect);
     TextParticule *		tp= bi->biParaParticules+ bp->bpParticule;
     int				line= bp->bpLine;
 
@@ -1366,15 +1378,15 @@ int tedResizeObject(		EditDocument *		ed,
     drChanged.drX0= add->addBackRect.drX0;
     drChanged.drX1= add->addBackRect.drX1;
 
-    tedAdjustRedrawBegin( td, drBack, &drChanged, &line, bp->bpParticule );
+    tedAdjustRedrawBegin( ed, &drChanged, &line, bp->bpParticule );
 
     if  ( tedReopenObject( bd, bi, tp, &ed->edColors,
-					    &(dp->dpFontList), add ) )
+						&(dp->dpFontList), add ) )
 	{ LDEB(1); return -1;	}
 
     if  ( tedAdjustParagraphLayout( bi, line, stroffShift,
 				td->tdSelection.bsEnd.bpStroff+ stroffShift,
-				ss->ssInHeaderFooter, add, bd, &drChanged ) )
+						    add, bd, &drChanged ) )
 	{ LDEB(line); }
 
     tedDelimitCurrentSelection( td, add );
@@ -1484,8 +1496,8 @@ int tedDeleteCurrentParagraph(	EditApplication *	ea )
 
     drChanged.drX0= add->addBackRect.drX0;
     drChanged.drX1= add->addBackRect.drX1;
-    drChanged.drY0= paraBi->biTopPosition.lpYPixels- 1;
-    drChanged.drY1= paraBi->biBelowPosition.lpYPixels- 1;
+    drChanged.drY0= BI_TOP_PIXELS( add, paraBi )- 1;
+    drChanged.drY1= BI_BELOW_PIXELS( add, paraBi )- 1;
 
     docInitPosition( &bpNew );
     if  ( docLastPosition( paraBi, &bpNew ) 	||
@@ -1576,8 +1588,8 @@ int tedDeleteCurrentSection(	EditApplication *	ea )
 
     drChanged.drX0= add->addBackRect.drX0;
     drChanged.drX1= add->addBackRect.drX1;
-    drChanged.drY0= sectBi->biTopPosition.lpYPixels- 1;
-    drChanged.drY1= sectBi->biBelowPosition.lpYPixels- 1;
+    drChanged.drY0= BI_TOP_PIXELS( add, sectBi )- 1;
+    drChanged.drY1= BI_BELOW_PIXELS( add, sectBi )- 1;
 
     docInitPosition( &bpNew );
     if  ( docLastPosition( sectBi, &bpNew ) 	||
@@ -1673,8 +1685,8 @@ int tedInsertParagraph(	EditApplication *	ea,
 
     drChanged.drX0= add->addBackRect.drX0;
     drChanged.drX1= add->addBackRect.drX1;
-    drChanged.drY0= paraBi->biTopPosition.lpYPixels- 1;
-    drChanged.drY1= paraBi->biBelowPosition.lpYPixels- 1;
+    drChanged.drY0= BI_TOP_PIXELS( add, paraBi )- 1;
+    drChanged.drY1= BI_BELOW_PIXELS( add, paraBi )- 1;
 
     parentBi= paraBi->biParent;
 
@@ -1744,8 +1756,8 @@ int tedInsertSection(	EditApplication *	ea,
 
     drChanged.drX0= add->addBackRect.drX0;
     drChanged.drX1= add->addBackRect.drX1;
-    drChanged.drY0= sectBi->biTopPosition.lpYPixels- 1;
-    drChanged.drY1= sectBi->biBelowPosition.lpYPixels- 1;
+    drChanged.drY0= BI_TOP_PIXELS( add, sectBi )- 1;
+    drChanged.drY1= BI_BELOW_PIXELS( add, sectBi )- 1;
 
     parentBi= sectBi->biParent;
 
