@@ -9,6 +9,8 @@
 #   include	<stdlib.h>
 #   include	<stdio.h>
 #   include	<stddef.h>
+#   include	<limits.h>
+
 #   include	<appDebugon.h>
 
 #   include	<appGeoString.h>
@@ -30,6 +32,7 @@ typedef struct TedFormatToolResources
     TablePageResources		tttrTablePageResources;
     RowPageResources		tttrRowPageResources;
     ColumnPageResources		tttrColumnPageResources;
+    CellPageResources		tttrCellPageResources;
     ParagraphPageResources	tttrParaPageResources;
     SectionPageResources	tttrSectPageResources;
     PageLayoutPageResources	tttrPageLayoutPageResources;
@@ -75,118 +78,6 @@ static AppConfigurableResource TED_TedFormatToolResourceTable[]=
 		    offsetof(TedFormatToolResources,
 		    tttrTablePageResources.tprLeftIndent),
 		    "Left Margin" ),
-
-	/****************************************************************/
-	/*  'Column' page.						*/
-	/****************************************************************/
-	APP_RESOURCE( "tableToolColumn",
-		    offsetof(TedFormatToolResources,
-		    tttrSubjectResources[TEDtsiCOLUMN].isrSubjectName),
-		    "Column" ),
-	APP_RESOURCE( "tableToolChangeColumn",
-		    offsetof(TedFormatToolResources,
-		    tttrSubjectResources[TEDtsiCOLUMN].isrApplyToSubject),
-		    "Apply to Column" ),
-	APP_RESOURCE( "tableToolRevert",
-		    offsetof(TedFormatToolResources,
-		    tttrSubjectResources[TEDtsiCOLUMN].isrRevert),
-		    "Revert" ),
-	APP_RESOURCE( "tableToolNextColumn",
-		    offsetof(TedFormatToolResources,
-		    tttrSubjectResources[TEDtsiCOLUMN].isrNextButtonText),
-		    "Next Column" ),
-	APP_RESOURCE( "tableToolPreviousColumn",
-		    offsetof(TedFormatToolResources,
-		    tttrSubjectResources[TEDtsiCOLUMN].isrPrevButtonText),
-		    "Previous Column" ),
-	APP_RESOURCE( "tableToolSelectColumn",
-		    offsetof(TedFormatToolResources,
-		    tttrSubjectResources[TEDtsiCOLUMN].isrSelectButtonText),
-		    "Select Column" ),
-	APP_RESOURCE( "tableToolDeleteColumn",
-		    offsetof(TedFormatToolResources,
-		    tttrSubjectResources[TEDtsiCOLUMN].isrDeleteButtonText),
-		    "Delete Column" ),
-	APP_RESOURCE( "tableToolInsertColumn",
-		    offsetof(TedFormatToolResources,
-		    tttrSubjectResources[TEDtsiCOLUMN].isrInsertButtonText),
-		    "Insert Column" ),
-	APP_RESOURCE( "tableToolAppendColumn",
-		    offsetof(TedFormatToolResources,
-		    tttrSubjectResources[TEDtsiCOLUMN].isrAppendButtonText),
-		    "Append Column" ),
-	/**/
-	APP_RESOURCE( "tableToolWidth",
-		    offsetof(TedFormatToolResources,
-		    tttrColumnPageResources.cprWidth),
-		    "Width" ),
-	/**/
-	APP_RESOURCE( "tableToolColumnLeftBorder",
-		    offsetof(TedFormatToolResources,
-		    tttrColumnPageResources.cprLeftBorder),
-		    "Left Border" ),
-	APP_RESOURCE( "tableToolColumnRightBorder",
-		    offsetof(TedFormatToolResources,
-		    tttrColumnPageResources.cprRightBorder),
-		    "Right Border" ),
-
-	/****************************************************************/
-	/*  'Cell' page.						*/
-	/****************************************************************/
-	APP_RESOURCE( "tableToolCell",
-		    offsetof(TedFormatToolResources,
-		    tttrSubjectResources[TEDtsiCELL].isrSubjectName),
-		    "Cell" ),
-	APP_RESOURCE( "tableToolChangeCell",
-		    offsetof(TedFormatToolResources,
-		    tttrSubjectResources[TEDtsiCELL].isrApplyToSubject),
-		    "Apply to Cell" ),
-	APP_RESOURCE( "tableToolRevert",
-		    offsetof(TedFormatToolResources,
-		    tttrSubjectResources[TEDtsiCELL].isrRevert),
-		    "Revert" ),
-	APP_RESOURCE( "tableToolCellNextColumn",
-		    offsetof(TedFormatToolResources,
-		    tttrSubjectResources[TEDtsiCELL].isrNextButtonText),
-		    "Next Column" ),
-	APP_RESOURCE( "tableToolCellPreviousColumn",
-		    offsetof(TedFormatToolResources,
-		    tttrSubjectResources[TEDtsiCELL].isrPrevButtonText),
-		    "Previous Column" ),
-	APP_RESOURCE( "tableToolSelectCell",
-		    offsetof(TedFormatToolResources,
-		    tttrSubjectResources[TEDtsiCELL].isrSelectButtonText),
-		    "Select Cell" ),
-	/**/
-	APP_RESOURCE( "tableToolCellRow",
-		    offsetof(TedFormatToolResources,
-		    tttrColumnPageResources.cprRow),
-		    "Row" ),
-	APP_RESOURCE( "tableToolCellColumn",
-		    offsetof(TedFormatToolResources,
-		    tttrColumnPageResources.cprColumn),
-		    "Column" ),
-	/**/
-	APP_RESOURCE( "tableToolCellTopBorder",
-		    offsetof(TedFormatToolResources,
-		    tttrColumnPageResources.cprTopBorder),
-		    "Top Border" ),
-	APP_RESOURCE( "tableToolCellBottomBorder",
-		    offsetof(TedFormatToolResources,
-		    tttrColumnPageResources.cprBottomBorder),
-		    "Bottom Border" ),
-	APP_RESOURCE( "tableToolCellColumnSpan",
-		    offsetof(TedFormatToolResources,
-		    tttrColumnPageResources.cprColumnSpan),
-		    "Columns Occupied" ),
-	APP_RESOURCE( "tableToolCellShadingPattern",
-		    offsetof(TedFormatToolResources,
-		    tttrColumnPageResources.cprShadingPattern),
-		    "Shading Pattern" ),
-	APP_RESOURCE( "tableToolCellVerticalAlignment",
-		    offsetof(TedFormatToolResources,
-		    tttrColumnPageResources.cprVerticalAlignment),
-		    "Vertical Alignment" ),
     };
 
 /************************************************************************/
@@ -201,10 +92,14 @@ int tedFormatValidateDimension(		int *		pNewValue,
 					int		oldValue )
     {
     int				changed;
-    int				requirePositive= 1;
 
-    if  ( ! appGetLengthFromTextWidget( w, &oldValue, &changed,
-					    UNITtyPOINTS, requirePositive ) )
+    const int			minValue= 1;
+    const int			adaptToMin= 0;
+    const int			maxValue= INT_MAX;
+    const int			adaptToMax= 0;
+
+    if  ( ! appGetLengthFromTextWidget( w, &oldValue, &changed, UNITtyPOINTS,
+				minValue, adaptToMin, maxValue, adaptToMax ) )
 	{
 	char	scratch[50];
 
@@ -231,8 +126,14 @@ int tedFormatToolGetGapWidth(		const RowProperties *	rp,
     const CellProperties *	cp;
     int				col;
 
+    const int			minValue= 1;
+    const int			adaptToMin= 0;
+    const int			maxValue= INT_MAX;
+    const int			adaptToMax= 0;
+
     value= rp->rpHalfGapWidthTwips;
-    if  ( appGetLengthFromTextWidget( w, &value, &changed, UNITtyPOINTS, 1 ) )
+    if  ( appGetLengthFromTextWidget( w, &value, &changed, UNITtyPOINTS,
+				minValue, adaptToMin, maxValue, adaptToMax ) )
 	{ return -1;	}
 
     cp= rp->rpCells;
@@ -260,6 +161,7 @@ int tedFormatToolGetGapWidth(		const RowProperties *	rp,
 	return 1;
 	}
 
+    *pValue= value;
     return 0;
     }
 
@@ -273,11 +175,14 @@ int tedFormatToolGetRowLeftIndent(	const RowProperties *	rp,
 
     char			scratch[50];
 
-    const int			requirePositive= 0;
+    const int			minValue= INT_MIN;
+    const int			adaptToMin= 0;
+    const int			maxValue= INT_MAX;
+    const int			adaptToMax= 0;
 
     value= rp->rpLeftIndentTwips;
-    if  ( appGetLengthFromTextWidget( w, &value, &changed,
-					    UNITtyPOINTS, requirePositive ) )
+    if  ( appGetLengthFromTextWidget( w, &value, &changed, UNITtyPOINTS,
+				minValue, adaptToMin, maxValue, adaptToMax ) )
 	{ return -1;	}
 
     if  ( value < -pageLeftMargin )
@@ -330,7 +235,7 @@ static void tedFormatFillPages(	const TedFormatToolResources *	tftr,
 			    &(tftr->tttrSubjectResources[TEDtsiCOLUMN]) );
 
     tedFormatFillCellPage( &(tft->tftCellTool),
-			    &(tftr->tttrColumnPageResources),
+			    &(tftr->tttrCellPageResources),
 			    &(ai->aiSubjects[TEDtsiCELL]),
 			    ai->aiSubjects[TEDtsiCELL].isPage,
 			    &(tftr->tttrSubjectResources[TEDtsiCELL]) );
@@ -395,7 +300,9 @@ static void tedDestroyFormatTool(	void *	voidtft )
 
     docCleanParagraphProperties( &(pt->ptPropertiesSet) );
     docCleanParagraphProperties( &(pt->ptPropertiesChosen) );
+#   if 0
     tedFormatCleanShadingTool( &(pt->ptShadingTool) );
+#   endif
     tedFormatCleanBorderTool( &(pt->ptTopBorderTool) );
     tedFormatCleanBorderTool( &(pt->ptBottomBorderTool) );
 
@@ -411,7 +318,9 @@ static void tedDestroyFormatTool(	void *	voidtft )
 
     docCleanCellProperties( &(clt->ctPropertiesSet) );
     docCleanCellProperties( &(clt->ctPropertiesChosen) );
+#   if 0
     tedFormatCleanShadingTool( &(clt->ctShadingTool) );
+#   endif
     tedFormatCleanBorderTool( &(clt->ctTopBorderTool) );
     tedFormatCleanBorderTool( &(clt->ctBottomBorderTool) );
     tedFormatCleanBorderTool( &(clt->ctLeftBorderTool) );
@@ -497,6 +406,14 @@ void tedShowFormatTool(	APP_WIDGET		option,
 				&(tftr.tttrRowPageResources),
 				&(tftr.tttrSubjectResources[TEDtsiROW]) );
 
+	tedFormatToolGetColumnResourceTable( ea,
+				&(tftr.tttrColumnPageResources),
+				&(tftr.tttrSubjectResources[TEDtsiCOLUMN]) );
+
+	tedFormatToolGetCellResourceTable( ea,
+				&(tftr.tttrCellPageResources),
+				&(tftr.tttrSubjectResources[TEDtsiCELL]) );
+
 	tedFormatToolGetParaResourceTable( ea,
 				&(tftr.tttrParaPageResources),
 				&(tftr.tttrSubjectResources[TEDtsiPARA]) );
@@ -552,6 +469,17 @@ void tedShowFormatTool(	APP_WIDGET		option,
 				    tftr.tttrSubjectResources, TEDtsiCOUNT,
 				    tedDestroyFormatTool, (void *)tft );
 
+    tft->tftInspector= ai;
+
+    tft->tftSectionTool.stInspector= ai;
+    tft->tftPageLayoutTool.pltInspector= ai;
+    tft->tftHeaderFooterTool.hftInspector= ai;
+    tft->tftParagraphTool.ptInspector= ai;
+    tft->tftColumnTool.ctInspector= ai;
+    tft->tftRowTool.rtInspector= ai;
+    tft->tftTableTool.ttInspector= ai;
+    tft->tftNotesTool.ntInspector= ai;
+
     tedFormatFillPages( &tftr, tft, ai );
 
     tedFormatFillRowChoosers( &(tft->tftRowTool),
@@ -569,7 +497,7 @@ void tedShowFormatTool(	APP_WIDGET		option,
     tedFormatFillColumnChoosers( &(tft->tftColumnTool),
 					&(tftr.tttrColumnPageResources) );
     tedFormatFillCellChoosers( &(tft->tftCellTool),
-					&(tftr.tttrColumnPageResources) );
+					&(tftr.tttrCellPageResources) );
 
     appFinishInspector( ai );
 
@@ -588,7 +516,7 @@ void tedShowFormatTool(	APP_WIDGET		option,
     tedFormatFinishColumnPage( &(tft->tftColumnTool), tft,
 					&(tftr.tttrColumnPageResources) );
     tedFormatFinishCellPage( &(tft->tftCellTool), tft,
-					&(tftr.tttrColumnPageResources) );
+					&(tftr.tttrCellPageResources) );
 
     tar->tarInspector= ai;
 
@@ -711,8 +639,9 @@ void tedFormatToolAdaptToSelection( AppInspector *		ai,
 
 	    appEnableInspectorSubject( ai, TEDtsiCELL, 1 );
 
-	    /*  CellTool is NOT for 2.9 */
+	    /*  CellTool is NOT for 2.9
 	    appEnableInspectorSubject( ai, TEDtsiCELL, 0 );
+	    */
 	    }
 	else{
 	    appEnableInspectorSubject( ai, TEDtsiCELL, 0 );
