@@ -11,6 +11,7 @@
 #   include	<stdio.h>
 #   include	<ctype.h>
 
+#   include	"docPs.h"
 #   include	"tedApp.h"
 #   include	"tedRuler.h"
 
@@ -61,6 +62,7 @@
 #   define KEY_c		XK_c
 #   define KEY_x		XK_x
 #   define KEY_v		XK_v
+#   define KEY_y		XK_y
 #   define KEY_Shift_L		XK_Shift_L
 #   define KEY_Shift_R		XK_Shift_R
 #   define KEY_Alt_L		XK_Alt_L
@@ -109,6 +111,7 @@
 #   define KEY_c		GDK_c
 #   define KEY_x		GDK_x
 #   define KEY_v		GDK_v
+#   define KEY_y		GDK_y
 #   define KEY_Shift_L		GDK_Shift_L
 #   define KEY_Shift_R		GDK_Shift_R
 #   define KEY_Alt_L		GDK_Alt_L
@@ -296,6 +299,7 @@ static void tedButton1Pressed(	APP_WIDGET			w,
     int				page;
     BufferItem *		sectBi= (BufferItem *)0;
     HeaderFooter *		hf= (HeaderFooter *)0;
+    HeaderFooter *		hfx= (HeaderFooter *)0;
     const DocumentGeometry *	dgSect;
     int				otherHeaderFooter= 0;
 
@@ -307,6 +311,7 @@ static void tedButton1Pressed(	APP_WIDGET			w,
     int				button;
     int				upDown;
     int				seq;
+    int				inHeaderFooter;
 
     if  ( appGetCoordinatesFromMouseButtonEvent( &mouseX, &mouseY,
 						    &button, &upDown, &seq,
@@ -335,21 +340,25 @@ static void tedButton1Pressed(	APP_WIDGET			w,
 	{ LLDEB(page,sectNr); return;	}
 
     dgSect= &(sectBi->biSectDocumentGeometry);
-    if  ( pageY < TWIPStoPIXELS( xfac, dgSect->dgTopMarginTwips ) )
+
+    hfx= (HeaderFooter *)0;
+    inHeaderFooter= docWhatPageHeader( &hfx, sectBi, page, dp );
+    if  ( hfx							&&
+	  hfx->hfItem						&&
+	  pageY < TWIPStoPIXELS( xfac, hfx->hfY1ReservedTwips ) )
 	{
-	dc.dcSelectionScope.ssInHeaderFooter=
-				docWhatPageHeader( &hf, sectBi, page, dp );
-	if  ( ! hf || ! hf->hfItem )
-	    { dc.dcSelectionScope.ssInHeaderFooter= DOCinBODY;	}
+	hf= hfx;
+	dc.dcSelectionScope.ssInHeaderFooter= inHeaderFooter;
 	}
 
-    if  ( pageY > TWIPStoPIXELS( xfac,
-		    dgSect->dgPageHighTwips- dgSect->dgBottomMarginTwips ) )
+    hfx= (HeaderFooter *)0;
+    inHeaderFooter= docWhatPageFooter( &hfx, sectBi, page, dp );
+    if  ( hfx							&&
+	  hfx->hfItem						&&
+	  pageY > TWIPStoPIXELS( xfac, hfx->hfY0ReservedTwips ) )
 	{
-	dc.dcSelectionScope.ssInHeaderFooter=
-				docWhatPageFooter( &hf, sectBi, page, dp );
-	if  ( ! hf || ! hf->hfItem )
-	    { dc.dcSelectionScope.ssInHeaderFooter= DOCinBODY;	}
+	hf= hfx;
+	dc.dcSelectionScope.ssInHeaderFooter= inHeaderFooter;
 	}
 
     if  ( dc.dcSelectionScope.ssInHeaderFooter		!=
@@ -394,8 +403,7 @@ static void tedButton1Pressed(	APP_WIDGET			w,
 		{ bi= sectBi;	}
 	    else{
 		if  ( tedRedoHeaderFooterLayout( &done, hf, page,
-				    dc.dcSelectionScope.ssInHeaderFooter,
-				    sectBi, bd, add, &drChanged ) )
+					    sectBi, bd, add, &drChanged ) )
 		    { LDEB(page); return;	}
 
 		bi= hf->hfItem;
@@ -535,6 +543,8 @@ static void tedInputExtendSelection(	EditDocument *		ed,
 					const BufferPosition *	bpSet )
     {
     TedDocument *		td= (TedDocument *)ed->edPrivateData;
+    BufferDocument *		bd= td->tdDocument;
+    const DocumentProperties *	dp= &(bd->bdProperties);
     const AppDrawingData *	add= &(ed->edDrawingData);
     BufferPosition		bpNew;
 
@@ -545,7 +555,7 @@ static void tedInputExtendSelection(	EditDocument *		ed,
     tedExtendSelectionToPosition( ed, ss, &(td->tdSelection.bsAnchor), &bpNew );
 
     tedSelectionRectangle( &(td->tdSelectedRectangle),
-						 add, &(td->tdSelection) );
+						 add, dp, &(td->tdSelection) );
 
     tedAdaptToolsToSelection( ed );
 
@@ -950,6 +960,12 @@ static void tedProcessKeyEvent(	EditDocument *		ed,
 	    return;
 #	endif
 
+#	if 1
+	case KEY_y:
+	    docListItem(0,bs->bsBegin.bpBi);
+	    break;
+#	endif
+
 	case KEY_Shift_L:
 	case KEY_Shift_R:
 	case KEY_Alt_L:
@@ -1083,7 +1099,7 @@ static void tedObjectHandleMove(	TedObjectDrag *		tod,
 	{
 	int		baseline;
 
-	baseline= tl->tlTopPosition.lpYPixels+ tl->tlLineAscentPixels;
+	baseline= TL_BASE_PIXELS( add, tl );
 
 	appExposeRectangle( add,
 			tp->tpX0- ox,
@@ -1267,7 +1283,8 @@ APP_EVENT_HANDLER( tedMouseButtonPressed, w, voided, downEvent )
     switch( downEvent->xbutton.button )
 	{
 	case Button1:
-	    if  ( downEvent->xbutton.subwindow )
+	    if  ( ! ed->edFileReadOnly		&&
+		  downEvent->xbutton.subwindow	)
 		{
 		tedObjectDrag( w, ed, downEvent );
 		return;
@@ -1292,9 +1309,12 @@ APP_EVENT_HANDLER( tedMouseButtonPressed, w, voided, downEvent )
 	int	res;
 
 	case 1:
-	    res= tedObjectDrag( w, ed, downEvent );
-	    if  ( res < 1 )
-		{ return;	}
+	    if  ( ! ed->edFileReadOnly )
+		{
+		res= tedObjectDrag( w, ed, downEvent );
+		if  ( res < 1 )
+		    { return;	}
+		}
 
 	    tedButton1Pressed( w, ed, downEvent );
 	    break;
