@@ -31,10 +31,10 @@ int tedBorderThick(		int *				pWide,
 				const BorderProperties *	bp,
 				const AppDrawingData *		add )
     {
-    int			wide= bp->bpWidthTwips+ bp->bpSpacingTwips;
-    int			thick= bp->bpWidthTwips;
+    int			wide= bp->bpPenWideTwips+ bp->bpSpacingTwips;
+    int			thick= bp->bpPenWideTwips;
 
-    if  ( ! bp->bpIsSet )
+    if  ( bp->bpStyle == DOCbsNONE )
 	{ thick= 0;	}
 
     if  ( thick > 0 )
@@ -509,7 +509,7 @@ static void tedSqueezeParticules(	TextParticule *		tp,
 
 static int tedLayoutLine(	const BufferItem *	bi,
 				AppDrawingData *	add,
-				const FormattingFrame *	ffPixels,
+				const FormattingFrame *	ff,
 				TextParticule *		tp,
 				ParticuleData *		pd,
 				int			particuleCount,
@@ -537,8 +537,7 @@ static int tedLayoutLine(	const BufferItem *	bi,
 	if  ( past < particuleCount )
 	    { includeTab= 1; past++;	}
 
-	accepted= tedLayoutParticules( bi, add, ffPixels, tp, pd,
-							    past- part, x0 );
+	accepted= tedLayoutParticules( bi, add, ff, tp, pd, past- part, x0 );
 
 	if  ( accepted != past- part )
 	    { LLDEB(accepted,past- part); return -1;	}
@@ -577,7 +576,7 @@ static int tedLayoutLine(	const BufferItem *	bi,
 		}
 	    }
 	else{
-	    x1= ffPixels->ffX1TextLines;
+	    x1= ff->ffX1TextLinesPixels;
 
 	    if  ( tpp->tpX0+ pdd->pdVisiblePixels > x1	)
 		{
@@ -606,7 +605,7 @@ static int tedLayoutScreenLine(	TextLine *			tl,
 				int				part,
 				int				accepted,
 				AppDrawingData *		add,
-				const FormattingFrame *		ffPixels,
+				const FormattingFrame *		ff,
 				TextParticule *			tp,
 				ParticuleData *			pd )
     {
@@ -616,13 +615,12 @@ static int tedLayoutScreenLine(	TextLine *			tl,
     int		x0Pixels;
     int		ascentPixels;
     int		descentPixels;
-    int		lineDistPixels;
 
     if  ( part == 0 )
-	{ x0Pixels= ffPixels->ffX0FirstLine;	}
-    else{ x0Pixels= ffPixels->ffX0TextLines;	}
+	{ x0Pixels= ff->ffX0FirstLinePixels;	}
+    else{ x0Pixels= ff->ffX0TextLinesPixels;	}
 
-    if  ( tedLayoutLine( bi, add, ffPixels, tp, pd, accepted, x0Pixels ) )
+    if  ( tedLayoutLine( bi, add, ff, tp, pd, accepted, x0Pixels ) )
 	{ LDEB(1); return -1;	}
 
     switch( bi->biParaAlignment )
@@ -631,18 +629,18 @@ static int tedLayoutScreenLine(	TextLine *			tl,
 	    xShift= 0; break;
 	case DOCiaRIGHT:
 	    x1= tp[accepted-1].tpX0+ pd[accepted-1].pdVisiblePixels;
-	    xShift= ffPixels->ffX1TextLines- x1;
+	    xShift= ff->ffX1TextLinesPixels- x1;
 	    break;
 	case DOCiaCENTERED:
 	    x1= tp[accepted-1].tpX0+ pd[accepted-1].pdVisiblePixels;
-	    xShift= ( ffPixels->ffX1TextLines- x1 )/ 2;
+	    xShift= ( ff->ffX1TextLinesPixels- x1 )/ 2;
 	    break;
 	case DOCiaJUSTIFIED:
 	    xShift= 0;
 	    if  ( part+ accepted < bi->biParaParticuleCount )
 		{
 		tedJustifyLine( bi, tp, pd, accepted,
-						ffPixels->ffX1TextLines );
+						ff->ffX1TextLinesPixels );
 		}
 	    break;
 	default:
@@ -660,11 +658,6 @@ static int tedLayoutScreenLine(	TextLine *			tl,
     tedLineHeight( bi, bi->biParaParticules+ part, add, accepted,
 					    &ascentPixels, &descentPixels );
 
-    lineDistPixels= TWIPStoPIXELS( add->addMagnifiedPixelsPerTwip,
-						tl->tlLineSpacingTwips );
-
-    tl->tlLineAscentPixels= ascentPixels;
-
     return tl->tlParticuleCount;
     }
 
@@ -678,22 +671,15 @@ static int tedLayoutScreenLine(	TextLine *			tl,
 /*									*/
 /************************************************************************/
 
-void tedParagraphFramePixels( 	FormattingFrame *	ffPixels,
+void tedParagraphFramePixels( 	FormattingFrame *	ff,
 				const AppDrawingData *	add,
 				const BufferItem *	bi )
     {
     double		xfac= add->addMagnifiedPixelsPerTwip;
-    FormattingFrame	ffTwips;
 
-    docParagraphFrameTwips( &ffTwips, -1, -1, bi );
-
-    ffPixels->ffX0Geometry= TWIPStoPIXELS( xfac, ffTwips.ffX0Geometry );
-    ffPixels->ffX0TextLines= TWIPStoPIXELS( xfac, ffTwips.ffX0TextLines );
-    ffPixels->ffX0FirstLine= TWIPStoPIXELS( xfac, ffTwips.ffX0FirstLine );
-    ffPixels->ffX1Geometry= TWIPStoPIXELS( xfac, ffTwips.ffX1Geometry );
-    ffPixels->ffX1TextLines= TWIPStoPIXELS( xfac, ffTwips.ffX1TextLines );
-
-    ffPixels->ffY1= -1;
+    ff->ffX0TextLinesPixels= TWIPStoPIXELS( xfac, ff->ffX0TextLinesTwips );
+    ff->ffX0FirstLinePixels= TWIPStoPIXELS( xfac, ff->ffX0FirstLineTwips );
+    ff->ffX1TextLinesPixels= TWIPStoPIXELS( xfac, ff->ffX1TextLinesTwips );
 
     return;
     }
@@ -704,32 +690,38 @@ void tedParagraphFramePixels( 	FormattingFrame *	ffPixels,
 /*									*/
 /************************************************************************/
 
-void tedParaScreenGeometry(	BufferItem *		bi,
-				const AppDrawingData *	add )
+void tedParaScreenGeometry(	const BufferItem *		bi,
+				const DocumentProperties *	dp,
+				const AppDrawingData *		add )
     {
     double		xfac= add->addMagnifiedPixelsPerTwip;
 
     int			tab;
     TabStop *		ts;
 
-    FormattingFrame	ffPixels;
+    FormattingFrame	ff;
 
-    tedParagraphFramePixels( &ffPixels, add, bi );
+    const int		bottom= -1;
+    const int		stripHigh= -1;
+    const int		page= 0;
+    const int		column= 0;
+
+    docParagraphFrameTwips( &ff, bottom, stripHigh, bi, dp, page, column );
+
+    tedParagraphFramePixels( &ff, add, bi );
 
     ts= bi->biParaTabStops;
     for ( tab= 0; tab < bi->biParaTabCount; ts++, tab++ )
 	{
-	ts->tsPixels= ffPixels.ffX0Geometry+
-					TWIPStoPIXELS( xfac, ts->tsTwips );
+	ts->tsPixels= TWIPStoPIXELS( xfac, ff.ffX0GeometryTwips+ ts->tsTwips );
 	}
 
     return;
     }
 
-static int tedLayoutStartScreenPara( FormattingFrame *		ffPixels,
-				BufferItem *			bi,
-				AppDrawingData *		add,
-				const DocumentProperties *	dp )
+static int tedLayoutStartScreenPara(	BufferItem *			bi,
+					AppDrawingData *		add,
+					const DocumentProperties *	dp )
     {
     int			part;
     TextParticule *	tp;
@@ -746,6 +738,13 @@ static int tedLayoutStartScreenPara( FormattingFrame *		ffPixels,
 	    {
 	    InsertedObject *	io= bi->biParaObjects+ tp->tpObjectNumber;
 
+	    if  ( tp->tpObjectNumber < 0			||
+		  tp->tpObjectNumber >= bi->biParaObjectCount	)
+		{
+		LLDEB(tp->tpObjectNumber,bi->biParaObjectCount);
+		return -1;
+		}
+
 	    if  ( io->ioKind == DOCokMACPICT )
 		{
 		if  ( io->ioTwipsWide == 0 )
@@ -761,18 +760,15 @@ static int tedLayoutStartScreenPara( FormattingFrame *		ffPixels,
 	    }
 	}
 
-    tedParagraphFramePixels( ffPixels, add, bi );
-
-    tedParaScreenGeometry( bi, add );
+    tedParaScreenGeometry( bi, dp, add );
 
     return 0;
     }
 
-static int tedPlaceStartScreenPara( FormattingFrame *		ffPixels,
-				BufferItem *			bi,
-				AppDrawingData *		add,
-				const DocumentProperties *	dp )
-    { tedParagraphFramePixels( ffPixels, add, bi ); return 0; }
+static int tedPlaceStartScreenPara( BufferItem *			bi,
+				    AppDrawingData *		add,
+				    const DocumentProperties *	dp )
+    { return 0;	}
 
 static int tedLayoutStartScreenRow(	BufferItem *			rowBi,
 					AppDrawingData *		add,
@@ -806,14 +802,14 @@ static int tedLayoutStartScreenRow(	BufferItem *			rowBi,
 
 static void tedFillLayoutJob(	LayoutJob *	lj )
     {
-    lj->ljInHeaderFooter= DOCinBODY;
-
     lj->ljLayoutScreen.slStartRow= tedLayoutStartScreenRow;
     lj->ljLayoutScreen.slStartParagraph= tedLayoutStartScreenPara;
+    lj->ljLayoutScreen.slScreenFrame= tedParagraphFramePixels;
     lj->ljLayoutScreen.slLayoutLine= tedLayoutScreenLine;
 
     lj->ljPlaceScreen.slStartRow= (START_ROW)0;
     lj->ljPlaceScreen.slStartParagraph= tedPlaceStartScreenPara;
+    lj->ljPlaceScreen.slScreenFrame= tedParagraphFramePixels;
     lj->ljPlaceScreen.slLayoutLine= (LAYOUT_SCREEN_LINE)0;
 
     return;
@@ -830,8 +826,6 @@ int tedLayoutItem(	BufferItem *		bi,
 
     tedFillLayoutJob( &lj );
 
-    lj.ljInHeaderFooter= inHeaderFooter;
-
     if  ( bi->biNumberInParent == 0 )
 	{
 	if  ( bi->biParent )
@@ -874,25 +868,26 @@ int tedGetHeaderFooterBox(	DocumentRectangle *		dr,
 
     DocumentRectangle		drBox;
 
+    int				pageTopPixels= page* add->addPageStepPixels;
+
     drBox.drX0= TWIPStoPIXELS( xfac, dgSect->dgLeftMarginTwips );
     drBox.drX1= TWIPStoPIXELS( xfac, dgSect->dgPageWideTwips-
 						dgSect->dgRightMarginTwips );
 
     if  ( docWhatPageHeader( &hf, sectBi, page, dp ) == inHdFt )
 	{
-	drBox.drY0= page* add->addPageStepPixels+
-			TWIPStoPIXELS( xfac, dgSect->dgHeaderPositionTwips );
-	drBox.drY1= page* add->addPageStepPixels+
-			TWIPStoPIXELS( xfac, dgSect->dgTopMarginTwips );
+	drBox.drY0= pageTopPixels+
+				TWIPStoPIXELS( xfac, hf->hfY0ReservedTwips );
+	drBox.drY1= pageTopPixels+
+				TWIPStoPIXELS( xfac, hf->hfY1ReservedTwips );
 	}
     else{
 	if  ( docWhatPageFooter( &hf, sectBi, page, dp ) == inHdFt )
 	    {
-	    drBox.drY0= page* add->addPageStepPixels+ TWIPStoPIXELS( xfac,
-			    dgSect->dgPageHighTwips-
-					dgSect->dgFooterPositionTwips );
-	    drBox.drY1= page* add->addPageStepPixels+ TWIPStoPIXELS( xfac,
-			    dgSect->dgPageHighTwips );
+	    drBox.drY0= pageTopPixels+
+				TWIPStoPIXELS( xfac, hf->hfY0ReservedTwips );
+	    drBox.drY1= pageTopPixels+
+				TWIPStoPIXELS( xfac, hf->hfY1ReservedTwips );
 	    }
 	else{ LDEB(inHdFt); return -1;	}
 	}
@@ -906,9 +901,8 @@ int tedGetHeaderFooterBox(	DocumentRectangle *		dr,
 /*									*/
 /************************************************************************/
 
-int tedLayoutHeaderItem(	BufferItem *			bi,
+int tedLayoutExternal(		HeaderFooter *			hf,
 				int				page,
-				int				inHeaderFooter,
 				const SectionProperties *	sp,
 				const BufferDocument *		bd,
 				AppDrawingData *		add,
@@ -916,54 +910,16 @@ int tedLayoutHeaderItem(	BufferItem *			bi,
     {
     const int			andParents= 0;
     LayoutJob			lj;
-    const DocumentGeometry *	dgSect= &(sp->spDocumentGeometry);
-
-    double			xfac= add->addMagnifiedPixelsPerTwip;
 
     tedFillLayoutJob( &lj );
 
-    lj.ljInHeaderFooter= inHeaderFooter;
-
     lj.ljPosition.lpPage= page;
-    lj.ljPosition.lpPageYTwips= dgSect->dgHeaderPositionTwips;
-    lj.ljPosition.lpYPixels= page* add->addPageStepPixels+ 
-			    TWIPStoPIXELS( xfac, lj.ljPosition.lpPageYTwips );
+    lj.ljPosition.lpPageYTwips= hf->hfY0UsedTwips;
+    lj.ljPosition.lpAtTopOfColumn= 1; /* not really */
 
     lj.ljChangedRectanglePixels= drChanged;
 
-    if  ( docPsLayoutItemAndParents( bi, add, bd, &lj, andParents ) )
-	{ LDEB(1); return -1;	}
-
-    return 0;
-    }
-
-int tedLayoutFooterItem(	BufferItem *			bi,
-				int				page,
-				int				inHeaderFooter,
-				const SectionProperties *	sp,
-				const BufferDocument *		bd,
-				AppDrawingData *		add,
-				DocumentRectangle *		drChanged )
-    {
-    const int			andParents= 0;
-    LayoutJob			lj;
-    const DocumentGeometry *	dgSect= &(sp->spDocumentGeometry);
-
-    double			xfac= add->addMagnifiedPixelsPerTwip;
-
-    tedFillLayoutJob( &lj );
-
-    lj.ljInHeaderFooter= inHeaderFooter;
-
-    lj.ljPosition.lpPage= page;
-    lj.ljPosition.lpPageYTwips= dgSect->dgPageHighTwips-
-						dgSect->dgFooterPositionTwips;
-    lj.ljPosition.lpYPixels= page* add->addPageStepPixels+ 
-			TWIPStoPIXELS( xfac, lj.ljPosition.lpPageYTwips );
-
-    lj.ljChangedRectanglePixels= drChanged;
-
-    if  ( docPsLayoutItemAndParents( bi, add, bd, &lj, andParents ) )
+    if  ( docPsLayoutItemAndParents( hf->hfItem, add, bd, &lj, andParents ) )
 	{ LDEB(1); return -1;	}
 
     return 0;
@@ -972,7 +928,6 @@ int tedLayoutFooterItem(	BufferItem *			bi,
 int tedRedoHeaderFooterLayout(	int *				pDone,
 				HeaderFooter *			hf,
 				int				page,
-				int				inHeadFoot,
 				const BufferItem *		sectBi,
 				BufferDocument *		bd,
 				AppDrawingData *		add,
@@ -980,36 +935,32 @@ int tedRedoHeaderFooterLayout(	int *				pDone,
     {
     int		needed= hf->hfPageUsedFor != page;
 
-    switch( inHeadFoot )
+    if  ( ! hf->hfItem )
+	{ XDEB(hf->hfItem); return -1;	}
+    if  ( ! needed )
+	{ *pDone= needed; return 0;	}
+
+    switch( hf->hfItem->biInHeaderFooter )
 	{
 	case DOCinSECT_HEADER:
 	case DOCinFIRST_HEADER:
 	case DOCinLEFT_HEADER:
 	case DOCinRIGHT_HEADER:
-	    if  ( needed						&&
-		  docLayoutHeaderFooter( hf, page, inHeadFoot,
-				    bd, sectBi, add,
-				    tedLayoutHeaderItem, drChanged )	)
-		{ LDEB(page); return -1; }
-
-	    *pDone= needed;
-	    return 0;
 
 	case DOCinSECT_FOOTER:
 	case DOCinFIRST_FOOTER:
 	case DOCinLEFT_FOOTER:
 	case DOCinRIGHT_FOOTER:
-	    if  ( needed						&&
-		  docLayoutHeaderFooter( hf, page, inHeadFoot,
-				    bd, sectBi, add,
-				    tedLayoutFooterItem, drChanged )	)
+
+	    if  ( docLayoutHeaderFooter( hf, page, bd, sectBi, add,
+			    tedLayoutExternal, drChanged, tedCloseObject ) )
 		{ LDEB(page); return -1; }
 
 	    *pDone= needed;
 	    return 0;
 
 	default:
-	    LDEB(inHeadFoot); return -1;
+	    LDEB(hf->hfItem->biInHeaderFooter); return -1;
 	}
     }
 
@@ -1024,7 +975,6 @@ int tedAdjustParagraphLayout(	BufferItem *			bi,
 				int				line,
 				int				stroffShift,
 				int				upto,
-				int				inHeaderFooter,
 				AppDrawingData *		add,
 				const BufferDocument *		bd,
 				DocumentRectangle *		drChanged )
@@ -1038,9 +988,7 @@ int tedAdjustParagraphLayout(	BufferItem *			bi,
     /*  bogus  */
     lj.ljPosition.lpPage= 0;
     lj.ljPosition.lpPageYTwips= 0;
-    lj.ljPosition.lpYPixels= 0;
 
-    lj.ljInHeaderFooter= inHeaderFooter;
     lj.ljChangedRectanglePixels= drChanged;
 
     if  ( docPsAdjustParaLayout( bi, line, stroffShift, upto, add, bd, &lj ) )
@@ -1058,7 +1006,7 @@ int tedAdjustParagraphLayout(	BufferItem *			bi,
 static BufferItem * tedFindItem(	BufferItem *		bi,
 					const AppDrawingData *	add,
 					int			x,
-					int			y	)
+					int			y )
     {
     int				i;
     BufferItem *		child;
@@ -1081,7 +1029,7 @@ static BufferItem * tedFindItem(	BufferItem *		bi,
 		if  ( i+ 1 < bi->biChildCount )
 		    { next= bi->biChildren[i+ 1];	}
 
-		if  ( y < child->biBelowPosition.lpYPixels		||
+		if  ( y < BI_BELOW_PIXELS( add, child )			||
 		      ( next					&&
 		        next->biTopPosition.lpPage > page	)	)
 		    {
@@ -1143,7 +1091,6 @@ static BufferItem * tedFindItem(	BufferItem *		bi,
 		    }
 		}
 
-	    LLDEB(bi->biTopPosition.lpYPixels,bi->biBelowPosition.lpYPixels);
 	    LLDEB(x,y);
 	    return (BufferItem *)0;
 
@@ -1162,14 +1109,13 @@ static int tedFindLine(	const BufferItem *	bi,
 			int			x,
 			int			y	)
     {
-    double		xfac= add->addMagnifiedPixelsPerTwip;
     int			page= y/ add->addPageStepPixels;
 
     int			i;
     TextLine *		tl;
     int			y1;
 
-    if  ( y < bi->biTopPosition.lpYPixels )
+    if  ( y < BI_TOP_PIXELS( add, bi ) )
 	{ return 0;	}
 
     if  ( bi->biLevel != DOClevPARA )
@@ -1178,8 +1124,7 @@ static int tedFindLine(	const BufferItem *	bi,
     tl= bi->biParaLines;
     for ( i= 0; i < bi->biParaLineCount; tl++, i++ )
 	{
-	y1= tl->tlTopPosition.lpYPixels+
-			    TWIPStoPIXELS( xfac, tl->tlLineSpacingTwips );
+	y1= TL_BELOW_PIXELS( add, tl );
 
 	if  ( y < y1						||
 	      ( i+ 1 < bi->biParaLineCount		&&
@@ -1350,7 +1295,6 @@ int tedFindPosition(	BufferItem *		rootBi,
 			BufferPosition *	bp )
     {
     const BufferItem *	bi;
-    double		xfac= add->addMagnifiedPixelsPerTwip;
 
     int			part;
     int			stroff;
@@ -1362,19 +1306,19 @@ int tedFindPosition(	BufferItem *		rootBi,
     bi= tedFindItem( rootBi, add, x, y );
     if  ( ! bi )
 	{
-	if  ( y < rootBi->biTopPosition.lpYPixels )
+	if  ( y < BI_TOP_PIXELS( add, rootBi ) )
 	    {
 	    if  ( docFirstPosition( rootBi, bp ) )
-		{ LLDEB(rootBi->biTopPosition.lpYPixels,y); return -1; }
+		{ LDEB(y); return -1; }
 
 	    bi= bp->bpBi;
 	    return 0;
 	    }
 
-	if  ( y >= rootBi->biBelowPosition.lpYPixels )
+	if  ( y >= BI_BELOW_PIXELS( add, rootBi ) )
 	    {
 	    if  ( docLastPosition( rootBi, bp ) )
-		{ LLDEB(rootBi->biBelowPosition.lpYPixels,y); return -1; }
+		{ LDEB(y); return -1; }
 
 	    bi= bp->bpBi;
 	    /*
@@ -1386,8 +1330,6 @@ int tedFindPosition(	BufferItem *		rootBi,
 	    }
 
 	LDEB(y);
-	LDEB(rootBi->biTopPosition.lpYPixels);
-	LDEB(rootBi->biBelowPosition.lpYPixels);
 	return -1;
 	}
 
@@ -1395,7 +1337,7 @@ int tedFindPosition(	BufferItem *		rootBi,
 	{ LLDEB(bi->biLevel,DOClevPARA); return -1;	}
 
     if  ( bi->biParaLineCount > 0				&&
-	  y < bi->biParaLines[0].tlTopPosition.lpYPixels	)
+	  y < TL_TOP_PIXELS( add, &(bi->biParaLines[0]) )	)
 	{
 	line= 0;
 	part= 0;
@@ -1409,7 +1351,7 @@ int tedFindPosition(	BufferItem *		rootBi,
 	    if  ( bi->biParaLineCount < 1 )
 		{ LDEB(bi->biParaLineCount); return -1;	}
 
-	    if  ( y > bi->biBelowPosition.lpYPixels	&&
+	    if  ( y > BI_BELOW_PIXELS( add, bi )	&&
 		  bi->biParaLineCount > 0		&&
 		  bi->biParaParticuleCount > 0		)
 		{
@@ -1452,11 +1394,6 @@ int tedFindPosition(	BufferItem *		rootBi,
 
     tl= bi->biParaLines+ line;
 
-    /*
-    *pTl= tl;
-    *pTp= bi->biParaParticules+ part;
-    */
-
     bp->bpBi= (BufferItem *)bi;	/*  Discards const */
     bp->bpLine= line;
     bp->bpParticule= part;
@@ -1464,9 +1401,8 @@ int tedFindPosition(	BufferItem *		rootBi,
 
     bp->bpXPixels= barX;
     bp->bpTopPosition= tl->tlTopPosition;
-    bp->bpY1Pixels= tl->tlTopPosition.lpYPixels+
-				TWIPStoPIXELS( xfac, tl->tlLineSpacingTwips );
-    bp->bpBaselinePixels= tl->tlTopPosition.lpYPixels+ tl->tlLineAscentPixels;
+    bp->bpY1Pixels= TL_BELOW_PIXELS( add, tl );
+    bp->bpBaselinePixels= TL_BASE_PIXELS( add, tl );
 
     return 0;
     }
@@ -1482,16 +1418,24 @@ int tedFindPosition(	BufferItem *		rootBi,
 /*									*/
 /************************************************************************/
 
-#   define	IW	5
+#   define	IW		5
 
-void tedSelectionRectangle(	DocumentRectangle *	drClip,
-				const AppDrawingData *	add,
-				const BufferSelection *	bs )
+#   define	minX0(ff)	\
+		((ff).ffX0TextLinesPixels < (ff).ffX0FirstLinePixels ? \
+		 (ff).ffX0TextLinesPixels : (ff).ffX0FirstLinePixels )
+
+void tedSelectionRectangle(	DocumentRectangle *		drClip,
+				const AppDrawingData *		add,
+				const DocumentProperties *	dp,
+				const BufferSelection *		bs )
     {
-    FormattingFrame	ffPixels;
-    DocumentRectangle	dr;
+    DocumentRectangle		dr;
 
-    drClip->drY0= bs->bsBegin.bpTopPosition.lpYPixels;
+    FormattingFrame		ff;
+    const int			bottom= -1;
+    const int			stripHigh= -1;
+
+    drClip->drY0= BP_TOP_PIXELS( add, &(bs->bsBegin) );
     drClip->drY1= bs->bsEnd.bpY1Pixels;
 
     /*  1  */
@@ -1513,18 +1457,14 @@ void tedSelectionRectangle(	DocumentRectangle *	drClip,
 	    return;
 	    }
 
-	if  ( bs->bsBegin.bpBi->biParaInTable )
-	    {
-	    tedParagraphFramePixels( &ffPixels, add, bs->bsBegin.bpBi );
+	docParagraphFrameTwips( &ff, bottom, stripHigh,
+				    bs->bsBegin.bpBi, dp,
+				    bs->bsBegin.bpBi->biTopPosition.lpPage,
+				    bs->bsBegin.bpBi->biTopPosition.lpColumn );
+	tedParagraphFramePixels( &ff, add, bs->bsBegin.bpBi );
 
-	    drClip->drX0= ffPixels.ffX0TextLines;
-	    drClip->drX1= ffPixels.ffX1TextLines;
-
-	    return;
-	    }
-
-	drClip->drX0= add->addBackRect.drX0;
-	drClip->drX1= add->addBackRect.drX1;
+	drClip->drX0= minX0( ff );
+	drClip->drX1= ff.ffX1TextLinesPixels;
 
 	return;
 	}
@@ -1541,21 +1481,25 @@ void tedSelectionRectangle(	DocumentRectangle *	drClip,
     /*  3  */
     if  ( bs->bsBegin.bpBi->biParaInTable )
 	{
-	tedParagraphFramePixels( &ffPixels, add, bs->bsBegin.bpBi );
+	docParagraphFrameTwips( &ff, bottom, stripHigh,
+				    bs->bsBegin.bpBi, dp,
+				    bs->bsBegin.bpBi->biTopPosition.lpPage,
+				    bs->bsBegin.bpBi->biTopPosition.lpColumn );
+	tedParagraphFramePixels( &ff, add, bs->bsBegin.bpBi );
 
-	dr.drX0= ffPixels.ffX0TextLines;
-	dr.drX1= ffPixels.ffX1TextLines;
+	dr.drX0= minX0( ff );
+	dr.drX1= ff.ffX1TextLinesPixels;
 
 	if  ( docSelectionInsideCell( bs ) )
 	    {
-	    dr.drY0= bs->bsBegin.bpBi->biTopPosition.lpYPixels;
-	    dr.drY1= bs->bsBegin.bpBi->biBelowPosition.lpYPixels;
+	    dr.drY0= BI_TOP_PIXELS( add, bs->bsBegin.bpBi );
+	    dr.drY1= BI_BELOW_PIXELS( add, bs->bsBegin.bpBi );
 	    }
 	else{
 	    BufferItem *	rowBi= bs->bsBegin.bpBi->biParent->biParent;
 
-	    dr.drY0= rowBi->biTopPosition.lpYPixels;
-	    dr.drY1= rowBi->biBelowPosition.lpYPixels;
+	    dr.drY0= BI_TOP_PIXELS( add, rowBi );
+	    dr.drY1= BI_BELOW_PIXELS( add, rowBi );
 	    }
 
 	docUnionRectangle( drClip, drClip, &dr );
@@ -1564,21 +1508,25 @@ void tedSelectionRectangle(	DocumentRectangle *	drClip,
     /*  3  */
     if  ( bs->bsEnd.bpBi->biParaInTable )
 	{
-	tedParagraphFramePixels( &ffPixels, add, bs->bsEnd.bpBi );
+	docParagraphFrameTwips( &ff, bottom, stripHigh,
+				    bs->bsEnd.bpBi, dp,
+				    bs->bsEnd.bpBi->biTopPosition.lpPage,
+				    bs->bsEnd.bpBi->biTopPosition.lpColumn );
+	tedParagraphFramePixels( &ff, add, bs->bsEnd.bpBi );
 
-	dr.drX0= ffPixels.ffX0TextLines;
-	dr.drX1= ffPixels.ffX1TextLines;
+	dr.drX0= minX0( ff );
+	dr.drX1= ff.ffX1TextLinesPixels;
 
 	if  ( docSelectionInsideCell( bs ) )
 	    {
-	    dr.drY0= bs->bsEnd.bpBi->biTopPosition.lpYPixels;
-	    dr.drY1= bs->bsEnd.bpBi->biBelowPosition.lpYPixels;
+	    dr.drY0= BI_TOP_PIXELS( add, bs->bsEnd.bpBi );
+	    dr.drY1= BI_BELOW_PIXELS( add, bs->bsEnd.bpBi );
 	    }
 	else{
 	    BufferItem *	rowBi= bs->bsEnd.bpBi->biParent->biParent;
 
-	    dr.drY0= rowBi->biTopPosition.lpYPixels;
-	    dr.drY1= rowBi->biBelowPosition.lpYPixels;
+	    dr.drY0= BI_TOP_PIXELS( add, rowBi );
+	    dr.drY1= BI_BELOW_PIXELS( add, rowBi );
 	    }
 
 	docUnionRectangle( drClip, drClip, &dr );
@@ -1594,17 +1542,14 @@ void tedPositionCoordinates(	BufferPosition *	bp,
     TextLine *		tl;
     TextParticule *	tp;
 
-    double		xfac= add->addMagnifiedPixelsPerTwip;
-
     bi= bp->bpBi;
     tl= bi->biParaLines+ bp->bpLine;
     tp= bi->biParaParticules+ bp->bpParticule;
 
     bp->bpXPixels= tedCalculateX( bi, tp, add, bp->bpStroff );
     bp->bpTopPosition= tl->tlTopPosition;
-    bp->bpY1Pixels= tl->tlTopPosition.lpYPixels+
-				TWIPStoPIXELS( xfac, tl->tlLineSpacingTwips );
-    bp->bpBaselinePixels= tl->tlTopPosition.lpYPixels+ tl->tlLineAscentPixels;
+    bp->bpY1Pixels= TL_BELOW_PIXELS( add, tl );
+    bp->bpBaselinePixels= TL_BASE_PIXELS( add, tl );
 
 #   if 0
     Gives silly high I Bar.
