@@ -34,6 +34,10 @@ static void tedLinkArea(	DocumentRectangle *	drChanged,
 				int			startPart,
 				int			endPart )
     {
+    TedDocument *		td= (TedDocument *)ed->edPrivateData;
+    BufferDocument *		bd= td->tdDocument;
+    const DocumentProperties *	dp= &(bd->bdProperties);
+
     AppDrawingData *		add= &(ed->edDrawingData);
 
     BufferSelection		bsNew;
@@ -50,7 +54,7 @@ static void tedLinkArea(	DocumentRectangle *	drChanged,
 
     tedSelectionCoordinates( &bsNew, add );
 
-    tedSelectionRectangle( drChanged, add, &bsNew );
+    tedSelectionRectangle( drChanged, add, dp, &bsNew );
 
     return;
     }
@@ -87,7 +91,8 @@ static int tedFinishSetField(	EditDocument *		ed,
 
     if  ( whenMask )
 	{
-	if  ( docRecalculateTextFields( &changed, bd, bi, whenMask ) )
+	if  ( docRecalculateTextLevelFields( &changed, bd, bi, whenMask,
+						(void *)add, tedCloseObject ) )
 	    { XDEB(whenMask);	}
 	}
 
@@ -134,6 +139,8 @@ static void tedRemoveFieldParticules(	EditDocument *		ed,
     {
     TedDocument *		td= (TedDocument *)ed->edPrivateData;
     BufferDocument *		bd= td->tdDocument;
+    const DocumentProperties *	dp= &(bd->bdProperties);
+
     AppDrawingData *		add= &(ed->edDrawingData);
 
     DocumentRectangle		drChanged;
@@ -158,7 +165,7 @@ static void tedRemoveFieldParticules(	EditDocument *		ed,
 
     tedSelectionCoordinates( bs, add );
 
-    tedSelectionRectangle( &(td->tdSelectedRectangle), add, bs );
+    tedSelectionRectangle( &(td->tdSelectedRectangle), add, dp, bs );
 
     /*  4  */
     tedExposeRectangle( ed, &drChanged, /*scrolledX,Y*/ 0,0 );
@@ -413,19 +420,16 @@ int tedRemoveHyperlink(	EditDocument *		ed )
 				&fileName, &fileSize, &markName, &markSize ) )
 	{ LDEB(1); return -1;	}
     else{
-	int			part;
 	TextParticule *		tp;
 
 	tp= bi->biParaParticules+ startPart;
 	df= bd->bdFieldList.dflFields+ tp->tpObjectNumber;
 	docCleanField( bd, df ); docInitField( df );
 
-	tp= bi->biParaParticules+ startPart+ 1;
-	for ( part= startPart+ 1; part < endPart; tp++, part++ )
-	    { tp->tpTextAttribute.taShowAsLink= 0; }
-
 	tedRemoveFieldParticules( ed, bi, startPart, endPart );
 
+	docFieldRefreshFlags( bi, bd );
+
 	appDocumentChanged( ed, 1 );
 
 	return 0;
