@@ -26,6 +26,7 @@
 
 #   include	<sioStdio.h>
 #   include	<sioMemory.h>
+#   include	<sioBase64.h>
 
 #   include	<appDebugon.h>
 
@@ -94,16 +95,23 @@ static AppConfigurableResource TEDApplicationResourceTable[]=
 		    offsetof(TedAppResources,tarHelpMenuText),
 		    "Help" ),
 
+	/**/
 	APP_RESOURCE( "documentFileName",
 		    offsetof(TedAppResources,tarAppHelpFileName),
 		    INFODIR "/TedDocument.rtf" ),
 
+	/**/
 	APP_RESOURCE( "selectionColor",
 		    offsetof(TedAppResources,tarSelectionColor),
 		    "LightSteelBlue" ),
 	APP_RESOURCE( "copiedSelectionColor",
 		    offsetof(TedAppResources,tarCopiedSelectionColor),
 		    "gray70" ),
+
+	/**/
+	APP_RESOURCE( "defaultAnsicpg",
+		    offsetof(TedAppResources,tarDefaultAnsicpgString),
+		    (char *)0 ),
     };
 
 
@@ -197,6 +205,27 @@ static int tedOpenDocumentFile(	EditApplication *	ea,
 
     int				resp;
     AppFileMessageResources *	afmr= &(ea->eaFileMessageResources);
+    TedAppResources *		tar= (TedAppResources *)ea->eaResourceData;
+
+    int				longestPara;
+
+    if  ( tar->tarDefaultAnsicpgInt < 0		&&
+	  tar->tarDefaultAnsicpgString		)
+	{
+	char *		past;
+	int		val;
+
+	val= strtol( tar->tarDefaultAnsicpgString, &past, 10 );
+	if  ( past != tar->tarDefaultAnsicpgString )
+	    {
+	    while( *past == ' ' )
+		{ past++;	}
+	    }
+	if  ( past != tar->tarDefaultAnsicpgString	&&
+	      ! *past					)
+	    { tar->tarDefaultAnsicpgInt= val;		}
+	else{ SDEB(tar->tarDefaultAnsicpgString);	}
+	}
 
     if  ( slash )
 	{ dot= strrchr( slash+ 1, '.' );	}
@@ -207,7 +236,7 @@ static int tedOpenDocumentFile(	EditApplication *	ea,
     if  ( ! sis )
 	{
 	appQuestionRunSubjectErrorDialog( ea, relative, option,
-				    filename, afmr->afmrFileNoAccess );
+					filename, afmr->afmrFileNoAccess );
 
 	return -1;
 	}
@@ -215,12 +244,19 @@ static int tedOpenDocumentFile(	EditApplication *	ea,
     if  ( dot && ! strcmp( dot, ".rtf" ) )
 	{
 	/*  2  */
-	bd= docRtfReadFile( sis );
+	bd= docRtfReadFile( sis, tar->tarDefaultAnsicpgInt );
 
 	sioInClose( sis );
 	
 	if  ( bd )
-	    { *pFormat= 0; *pBd= bd; return 0; }
+	    {
+	    *pFormat= TEDdockindRTF; *pBd= bd;
+
+	    if  ( docPropertiesSetFilename( &(bd->bdProperties), filename ) )
+		{ LDEB(1);	}
+
+	    return 0;
+	    }
 	else{
 	    resp= appQuestionRunSubjectOkCancelDialog( ea,
 				    relative, option,
@@ -236,21 +272,40 @@ static int tedOpenDocumentFile(	EditApplication *	ea,
 
     if  ( dot && ! strcmp( dot, ".txt" ) )
 	{
-	bd= docPlainReadFile( sis, &(ea->eaDefaultDocumentGeometry) );
+	bd= docPlainReadFile( sis, &longestPara,
+					&(ea->eaDefaultDocumentGeometry) );
 
 	sioInClose( sis );
 
 	if  ( bd )
-	    { *pFormat= 1; *pBd= bd; return 0;	}
+	    {
+	    *pBd= bd;
+
+	    if  ( longestPara > 76 )
+		{ *pFormat= TEDdockindTEXT_SAVE_WIDE;		}
+	    else{ *pFormat= TEDdockindTEXT_SAVE_FOLDED;		}
+
+	    if  ( docPropertiesSetFilename( &(bd->bdProperties), filename ) )
+		{ LDEB(1);	}
+
+	    return 0;
+	    }
 	else{ SXDEB(filename,bd); return -1;	}
 	}
 
     /*  2  */
     if  ( ! triedRtf )
 	{
-	bd= docRtfReadFile( sis );
+	bd= docRtfReadFile( sis, tar->tarDefaultAnsicpgInt );
 	if  ( bd )
-	    { *pFormat= 0; *pBd= bd; return 0; }
+	    {
+	    *pFormat= TEDdockindRTF; *pBd= bd;
+	    
+	    if  ( docPropertiesSetFilename( &(bd->bdProperties), filename ) )
+		{ LDEB(1);	}
+
+	    return 0;
+	    }
 
 	resp= appQuestionRunSubjectOkCancelDialog( ea,
 					ea->eaToplevel.atTopWidget, option,
@@ -265,12 +320,24 @@ static int tedOpenDocumentFile(	EditApplication *	ea,
     if  ( ! sis )
 	{ SXDEB(filename,sis); return -1;	}
 
-    bd= docPlainReadFile( sis, &(ea->eaDefaultDocumentGeometry) );
+    bd= docPlainReadFile( sis, &longestPara,
+				    &(ea->eaDefaultDocumentGeometry) );
 
     sioInClose( sis );
 
     if  ( bd )
-	{ *pFormat= 1; *pBd= bd; return 0; }
+	{
+	*pBd= bd;
+
+	if  ( longestPara > 76 )
+	    { *pFormat= TEDdockindTEXT_SAVE_WIDE;		}
+	else{ *pFormat= TEDdockindTEXT_SAVE_FOLDED;		}
+
+	if  ( docPropertiesSetFilename( &(bd->bdProperties), filename ) )
+	    { LDEB(1);	}
+
+	return 0;
+	}
 
     SXDEB(filename,bd); return -1;
     }
@@ -1341,6 +1408,7 @@ void tedMakeDocumentReadonly(	EditDocument *	ed )
 /*  on and off when appropriate.					*/
 /*									*/
 /************************************************************************/
+
 static void tedMakePrivateDocumentMenus( EditApplication *	ea,
 					EditDocument *		ed,
 					APP_WIDGET		menubar )
@@ -1513,7 +1581,17 @@ static AppFileExtension	TedFileExtensions[]=
     { "txtFiles",	"*.txt",
 			"Text Files ( *.txt )",	
 			"txt"	 ,
-			( APPFILE_CAN_OPEN|APPFILE_CAN_SAVE ),
+			( APPFILE_CAN_OPEN ),
+			},
+    { "txtFilesFolded",	"*.txt",
+			"Folded Text Files ( *.txt )",	
+			"txt"	 ,
+			( APPFILE_CAN_SAVE|APPFILE_CAN_OPEN|APPFILE_HIDE_OPEN ),
+			},
+    { "txtFilesWide",	"*.txt",
+			"Wide Text Files ( *.txt )",	
+			"txt"	 ,
+			( APPFILE_CAN_SAVE|APPFILE_CAN_OPEN|APPFILE_HIDE_OPEN ),
 			},
     { "htmlFiles",	"*.html",
 			"HTML Files ( *.html )",
@@ -1528,24 +1606,101 @@ static AppFileExtension	TedFileExtensions[]=
 };
 
 static int tedWriteRtfMail(	SimpleOutputStream *		sos,
+				const char *			mimeBoundary,
 				void *				voidbd )
     {
-    const int	saveBookmarks= 1;
+    SimpleOutputStream *	sosBody;
+    const int			saveBookmarks= 1;
+    int				rval= 0;
 
-    return docRtfSaveDocument( sos,
-				(BufferDocument *)voidbd,
+    BufferDocument *		bd= (BufferDocument *)voidbd;
+    const DocumentProperties *	dp= &(bd->bdProperties);
+
+    const char *		file= (const char *)dp->dpFilename;
+
+    if  ( file )
+	{
+	const char *		relative;
+
+	relative= strrchr( file, '/' );
+	if  ( relative )
+	    { file= relative+ 1;	}
+	}
+
+    if  ( ! file || ! file[0] )
+	{ file= "file.rtf";	}
+
+    sosBody= sos;
+
+    if  ( mimeBoundary )
+	{
+	sioOutPutCharacter( '-', sos ); sioOutPutCharacter( '-', sos );
+	sioOutPutString( mimeBoundary, sos );
+	sioOutPutCharacter( '\r', sos ); sioOutPutCharacter( '\n', sos );
+	sioOutPutString( "Content-Type: text/plain", sos );
+	sioOutPutCharacter( '\r', sos ); sioOutPutCharacter( '\n', sos );
+	sioOutPutString( "Content-Transfer-Encoding: 7bit", sos );
+	sioOutPutCharacter( '\r', sos ); sioOutPutCharacter( '\n', sos );
+
+	sioOutPutCharacter( '\r', sos ); sioOutPutCharacter( '\n', sos );
+
+	sioOutPutCharacter( '-', sos ); sioOutPutCharacter( '-', sos );
+	sioOutPutString( mimeBoundary, sos );
+	sioOutPutCharacter( '\r', sos ); sioOutPutCharacter( '\n', sos );
+
+	sioOutPutString( "Content-Type: ", sos );
+	if  ( file && file[0] )
+	    {
+	    sioOutPutString( "application/rtf; name=\"", sos );
+	    sioOutPutString( file, sos );
+	    sioOutPutString( "\"", sos );
+	    }
+	else{
+	    sioOutPutString( "application/rtf", sos );
+	    }
+	sioOutPutCharacter( '\r', sos ); sioOutPutCharacter( '\n', sos );
+
+	sioOutPutString( "Content-Transfer-Encoding: ", sos );
+	sioOutPutString( "base64", sos );
+	sioOutPutCharacter( '\r', sos ); sioOutPutCharacter( '\n', sos );
+
+	sioOutPutCharacter( '\r', sos ); sioOutPutCharacter( '\n', sos );
+
+	sosBody= sioOutBase64Open( sos );
+	if  ( ! sosBody )
+	    { XDEB(sosBody); return -1;	}
+	}
+
+    rval= docRtfSaveDocument( sosBody, bd,
 				(const SelectionScope *)0,
 				(const BufferSelection *)0,
 				saveBookmarks );
+
+    if  ( mimeBoundary )
+	{
+	sioOutClose( sosBody );
+
+	sioOutPutCharacter( '-', sos ); sioOutPutCharacter( '-', sos );
+	sioOutPutString( mimeBoundary, sos );
+	sioOutPutCharacter( '-', sos ); sioOutPutCharacter( '-', sos );
+	sioOutPutCharacter( '\r', sos ); sioOutPutCharacter( '\n', sos );
+	}
+
+    return rval;
     }
 
 static int tedWriteHtmlMail(	SimpleOutputStream *		sos,
+				const char *			mimeBoundary,
 				void *				voidbd )
     {
-    return docHtmlSaveDocument( sos, (BufferDocument *)voidbd, (char *)0 );
+    int		asMimeAggr= mimeBoundary != (const char *)0;
+
+    return docHtmlSaveDocument( sos, (BufferDocument *)voidbd,
+					asMimeAggr, mimeBoundary, (char *)0 );
     }
 
 static int tedWritePlainMail(	SimpleOutputStream *		sos,
+				const char *			mimeBoundary,
 				void *				voidbd )
     {
     return docPlainSaveDocument( sos,
@@ -1558,11 +1713,22 @@ static int tedWritePlainMail(	SimpleOutputStream *		sos,
 static MailContent	TedMailContents[]=
     {
 	{ "mailPlain",	"text/plain",
-			"text", "plain",	tedWritePlainMail	},
+			"text/plain",
+			0,
+			tedWritePlainMail
+			},
+
 	{ "mailRtf",	"application/rtf",
-			"application", "rtf",	tedWriteRtfMail		},
+			"multipart/mixed",
+			1,
+			tedWriteRtfMail
+			},
+
 	{ "mailHtml",	"text/html",
-			"text", "html",		tedWriteHtmlMail	},
+			"multipart/related",
+			1,
+			tedWriteHtmlMail
+			},
     };
 
 static AppSelectionTargetType TedPrimaryTargets[]=
@@ -1685,7 +1851,7 @@ static EditApplication	TedApplication=
 		    /*  Name of the picture for the application window.	*/
 		    /****************************************************/
     "Ted",
-    "Ted, Version 2.9, January 31, 2001",
+    "Ted, Version 2.10, April 15, 2001",
     "http://www.nllgg.nl/Ted",
 
 
@@ -1860,6 +2026,8 @@ static EditApplication	TedApplication=
 int main(	int		argc,
 		char *		argv[]	)
     {
+    TEDResources.tarDefaultAnsicpgInt= -1;
+
     /*  1  */
     if  ( argc == 4				&&
 	  ! strcmp( argv[1], "++saveTo" )	)
