@@ -11,19 +11,21 @@
 #   include	<stdlib.h>
 #   include	<string.h>
 
-#   include	"tedApp.h"
-#   include	<appImage.h>
 #   include	<sioMemory.h>
 #   include	<sioEndian.h>
 #   include	<sioStdio.h>
 #   include	<sioHex.h>
 
-#   include	<appDebugon.h>
-
 #   include	<appDraw.h>
+#   include	<appImage.h>
 #   include	<appWinMeta.h>
 #   include	<appMacPict.h>
 
+#   include	"docPs.h"
+#   include	"tedApp.h"
+
+#   include	<appDebugon.h>
+
 int tedDrawObject(		const BufferItem *	bi,
 				const TextParticule *	tp,
 				int			baseline,
@@ -39,7 +41,8 @@ int tedDrawObject(		const BufferItem *	bi,
     if  ( ( io->ioKind == DOCokPICTWMETAFILE	||
 	    io->ioKind == DOCokMACPICT		||
 	    io->ioKind == DOCokPICTPNGBLIP	||
-	    io->ioKind == DOCokPICTJPEGBLIP	)	&&
+	    io->ioKind == DOCokPICTJPEGBLIP	||
+	    io->ioKind == DOCokINCLUDEPICTURE	)	&&
 	  io->ioPixmap					)
 	{
 	appDrawDrawPixmap( add, io->ioPixmap,
@@ -94,7 +97,7 @@ static int tedOpenMetafileObject(	InsertedObject *	io,
 					PLAY_META_X11		playMetaX11,
 					AppColors *		ac,
 					AppDrawingData *	add,
-					const ObjectData *	od )
+					const MemoryBuffer *	mb )
     {
     int			xExt= io->ioXExtent;
     int			yExt= io->ioYExtent;
@@ -102,12 +105,7 @@ static int tedOpenMetafileObject(	InsertedObject *	io,
     SimpleInputStream *	sisMem;
     SimpleInputStream *	sisMeta;
 
-    MemoryBuffer	mb;
-
-    mb.mbCapacity= od->odSize;
-    mb.mbBytes= od->odBytes;
-
-    sisMem= sioInMemoryOpen( &mb );
+    sisMem= sioInMemoryOpen( mb );
     if  ( ! sisMem )
 	{ XDEB(sisMem); return -1;	}
 
@@ -120,9 +118,9 @@ static int tedOpenMetafileObject(	InsertedObject *	io,
 	{ XDEB(io->ioPixmap); return -1;	}
 
     if  ( (*playMetaX11)( sisMeta, &io->ioPrivate, ac, add,
-		    io->ioPixmap, xExt, yExt,
-		    io->ioPixelsWide, io->ioPixelsHigh,
-		    io->ioTwipsWide, io->ioTwipsHigh ) )
+					io->ioPixmap, xExt, yExt,
+					io->ioPixelsWide, io->ioPixelsHigh,
+					io->ioTwipsWide, io->ioTwipsHigh ) )
 	{ LDEB(1);	}
 
     sioInClose( sisMeta );
@@ -135,26 +133,21 @@ static int tedOpenBitmapObject(	bmReadBitmap		readBitmap,
 				InsertedObject *	io,
 				double			pixelsPerTwip,
 				AppColors *		ac,
-				ObjectData *		od,
-				AppDrawingData *	add )
+				AppDrawingData *	add,
+				const MemoryBuffer *	mb )
     {
     AppBitmapImage *	abi= (AppBitmapImage *)malloc( sizeof(AppBitmapImage) );
 
     SimpleInputStream *	sisMem;
     SimpleInputStream *	sisBitmap;
 
-    MemoryBuffer	mb;
-
     APP_BITMAP_IMAGE	pixmap;
 
-    mb.mbCapacity= od->odSize;
-    mb.mbBytes= od->odBytes;
-
     if  ( ! abi )
 	{ XDEB(abi); return -1;	}
     appInitBitmapImage( abi );
 
-    sisMem= sioInMemoryOpen( &mb );
+    sisMem= sioInMemoryOpen( mb );
     if  ( ! sisMem )
 	{ XDEB(sisMem); return -1;	}
 
@@ -182,6 +175,34 @@ static int tedOpenBitmapObject(	bmReadBitmap		readBitmap,
     return 0;
     }
 
+static int tedOpenEpsfObject(		InsertedObject *	io,
+					AppColors *		ac,
+					AppDrawingData *	add,
+					const MemoryBuffer *	mb )
+    {
+#   if 0
+    SimpleInputStream *	sisMem;
+    SimpleInputStream *	sisMeta;
+
+    sisMem= sioInMemoryOpen( mb );
+    if  ( ! sisMem )
+	{ XDEB(sisMem); return -1;	}
+
+    sisMeta= sioInHexOpen( sisMem );
+    if  ( ! sisMeta )
+	{ XDEB(sisMem); return -1;	}
+
+    io->ioPixmap= appMakePixmap( add, io->ioPixelsWide, io->ioPixelsHigh );
+    if  ( ! io->ioPixmap )
+	{ XDEB(io->ioPixmap); return -1;	}
+
+    sioInClose( sisMeta );
+    sioInClose( sisMem );
+
+#   endif
+    return 0;
+    }
+
 static int tedOpenObject(	InsertedObject *	io,
 				AppColors *		ac,
 				AppDrawingData *	add )
@@ -194,51 +215,65 @@ static int tedOpenObject(	InsertedObject *	io,
 	case DOCokPICTWMETAFILE:
 	    if  ( ! io->ioPrivate					&&
 		  tedOpenMetafileObject( io, appMetaPlayWmfX11,
-				    ac, add, &io->ioObjectData )	)
+					ac, add, &io->ioObjectData )	)
 		{ LDEB(1); return 0;	}
-	    break;
+	    return 0;
 
 	case DOCokPICTEMFBLIP:
 	    if  ( ! io->ioPrivate					&&
 		  tedOpenMetafileObject( io, appMetaPlayEmfX11,
-				    ac, add, &io->ioObjectData )	)
+					ac, add, &io->ioObjectData )	)
 		{ LDEB(1); return 0;	}
-	    break;
+	    return 0;
 
 	case DOCokMACPICT:
 	    if  ( ! io->ioPrivate					&&
 		  tedOpenMetafileObject( io, appMacPictPlayFileX11,
-				    ac, add, &io->ioObjectData )	)
+					ac, add, &io->ioObjectData )	)
 		{ LDEB(1); return 0;	}
-	    break;
+	    return 0;
 
 	case DOCokPICTPNGBLIP:
 	    if  ( ! io->ioPrivate					&&
 		  tedOpenBitmapObject( bmPngReadPng, io,
-			add->addMagnifiedPixelsPerTwip, ac,
-			&io->ioObjectData, add )			)
+				    add->addMagnifiedPixelsPerTwip,
+					ac, add, &io->ioObjectData )	)
 		{ LDEB(1); return 0;	}
-	    break;
+	    return 0;
 
 	case DOCokPICTJPEGBLIP:
 	    if  ( ! io->ioPrivate					&&
 		  tedOpenBitmapObject( bmJpegReadJfif, io,
-			add->addMagnifiedPixelsPerTwip, ac,
-			&io->ioObjectData, add )			)
+				    add->addMagnifiedPixelsPerTwip,
+					ac, add, &io->ioObjectData )	)
 		{ LDEB(1); return 0;	}
-	    break;
+	    return 0;
 
 	case DOCokOLEOBJECT:
 	    if  ( io->ioResultKind == DOCokPICTWMETAFILE )
 		{
 		if  ( ! io->ioPrivate					&&
 		      tedOpenMetafileObject( io, appMetaPlayWmfX11,
-				    ac, add, &io->ioResultData )	)
+					ac, add, &io->ioResultData )	)
 		    { LDEB(1); return 0;	}
 
 		return 0;
 		}
 	    return 0;
+
+	case DOCokINCLUDEPICTURE:
+
+	    if  ( io->ioResultKind == DOCokEPSF )
+		{
+		if  ( ! io->ioPixmap					&&
+		      tedOpenEpsfObject( io,
+					ac, add, &io->ioResultData )	)
+		    { LDEB(1); return 0;	}
+
+		return 0;
+		}
+	    else{ LLDEB(io->ioKind,io->ioResultKind); return 0;	}
+
 	default:
 	    LDEB(io->ioKind); return 0;
 	}
@@ -385,6 +420,17 @@ void tedCloseObject(		BufferDocument *	bd,
 		io->ioPrivate= (void *)0;
 		}
 	    break;
+
+	case DOCokINCLUDEPICTURE:
+	    if  ( io->ioPixmap )
+		{
+		AppDrawingData *	add= (AppDrawingData *)voidadd;
+
+		appDrawFreePixmap( add, io->ioPixmap );
+		io->ioPixmap= (APP_WINDOW)0;
+		}
+	    break;
+
 	default:
 	    LDEB(io->ioKind); break;
 	}
@@ -419,6 +465,8 @@ static void tedScaleObjectToParagraph(	EditDocument *		ed,
 					InsertedObject *	io )
     {
     TedDocument *		td= (TedDocument *)ed->edPrivateData;
+    BufferDocument *		bd= td->tdDocument;
+    const DocumentProperties *	dp= &(bd->bdProperties);
     FormattingFrame		ff;
     double			scaleX;
     double			scaleY;
@@ -426,11 +474,17 @@ static void tedScaleObjectToParagraph(	EditDocument *		ed,
     SelectionScope		ss;
     BufferSelection		bs;
 
+    const int			bottom= -1;
+    const int			stripHigh= -1;
+
     if  ( tedGetSelection( &ss, &bs, td ) )
 	{ LDEB(1); return;	}
 
-    docParagraphFrameTwips( &ff, -1, -1, bs.bsBegin.bpBi );
-    scaleX= (double)( ff.ffX1TextLines- ff.ffX0TextLines )/
+    docParagraphFrameTwips( &ff, bottom, stripHigh, bs.bsBegin.bpBi, dp,
+				    bs.bsBegin.bpBi->biTopPosition.lpPage,
+				    bs.bsBegin.bpBi->biTopPosition.lpColumn );
+
+    scaleX= (double)( ff.ffX1TextLinesTwips- ff.ffX0TextLinesTwips )/
 						    (double)io->ioTwipsWide;
 
     scaleY= (double)ff.ffPageHigh/ (double)io->ioTwipsHigh;
@@ -523,7 +577,7 @@ int tedObjectInsertBitmap(	EditDocument *		ed,
     if  ( ! io )
 	{ XDEB(io); return -1;	}
 
-    sioMemoryInit( &mb );
+    utilInitMemoryBuffer( &mb );
 
     sosMem= sioOutMemoryOpen( &mb );
     if  ( ! sosMem )
@@ -553,8 +607,7 @@ int tedObjectInsertBitmap(	EditDocument *		ed,
     sioOutClose( sosHex );
     sioOutClose( sosMem );
 
-    io->ioObjectData.odBytes= mb.mbBytes;
-    io->ioObjectData.odSize= mb.mbCapacity;
+    io->ioObjectData= mb;
 
     switch( abi->abiBitmap.bdUnit )
 	{
@@ -653,7 +706,7 @@ static int tedObjectInsertMetafile(	EditDocument *		ed,
     if  ( ! io )
 	{ XDEB(io); return -1;	}
 
-    sioMemoryInit( &mb );
+    utilInitMemoryBuffer( &mb );
 
     sosMem= sioOutMemoryOpen( &mb );
     if  ( ! sosMem )
@@ -677,29 +730,46 @@ static int tedObjectInsertMetafile(	EditDocument *		ed,
 
     key= sioEndianGetLeUint32( sisIn );
 
-    if  ( key != 0x9ac6cdd7 )
-	{ XDEB(key); return -1;	}
+    if  ( key == 0x9ac6cdd7 )
+	{
+	handle= sioEndianGetLeUint16( sisIn );
+	left= sioEndianGetLeInt16( sisIn );
+	top= sioEndianGetLeInt16( sisIn );
+	right= sioEndianGetLeInt16( sisIn );
+	bottom= sioEndianGetLeInt16( sisIn );
+	inch= sioEndianGetLeUint16( sisIn );
 
-    handle= sioEndianGetLeUint16( sisIn );
-    left= sioEndianGetLeInt16( sisIn );
-    top= sioEndianGetLeInt16( sisIn );
-    right= sioEndianGetLeInt16( sisIn );
-    bottom= sioEndianGetLeInt16( sisIn );
-    inch= sioEndianGetLeUint16( sisIn );
+	reserved= sioEndianGetLeUint32( sisIn );
+	checksum= sioEndianGetLeUint16( sisIn );
 
-    reserved= sioEndianGetLeUint32( sisIn );
-    checksum= sioEndianGetLeUint16( sisIn );
+	if  ( right > left )
+	    { io->ioTwipsWide= ( 20* 72* ( right- left ) )/ inch;	}
+	else{ io->ioTwipsWide= ( 20* 72* ( left- right ) )/ inch;	}
 
-    if  ( right > left )
-	{ io->ioTwipsWide= ( 20* 72* ( right- left ) )/ inch;	}
-    else{ io->ioTwipsWide= ( 20* 72* ( left- right ) )/ inch;	}
+	if  ( bottom > top )
+	    { io->ioTwipsHigh= ( 20* 72* ( bottom- top ) )/ inch;	}
+	else{ io->ioTwipsHigh= ( 20* 72* ( top- bottom ) )/ inch;	}
 
-    if  ( bottom > top )
-	{ io->ioTwipsHigh= ( 20* 72* ( bottom- top ) )/ inch;	}
-    else{ io->ioTwipsHigh= ( 20* 72* ( top- bottom ) )/ inch;	}
-
-    io->ioXExtent= right- left;
-    io->ioYExtent= top- bottom;
+	io->ioXExtent= right- left;
+	io->ioYExtent= top- bottom;
+	}
+    else{
+	if  ( 0 && key == 0x90001 )
+	    {
+	    io->ioTwipsWide= 20* 72* 4;		/*  10 cm	*/
+	    io->ioTwipsHigh= 20* 72* 4;		/*  10 cm	*/
+	    io->ioXExtent= 72* 4;
+	    io->ioYExtent= 72* 4;
+	    sioEndianPutLeUint32( key, sosMeta );
+	    }
+	else{
+	    XDEB(key);
+	    sioOutClose( sosMeta );
+	    sioOutClose( sosMem );
+	    sioInClose( sisIn );
+	    return -1;
+	    }
+	}
     }
 
     for (;;)
@@ -719,8 +789,7 @@ static int tedObjectInsertMetafile(	EditDocument *		ed,
     sioOutClose( sosMem );
     sioInClose( sisIn );
 
-    io->ioObjectData.odBytes= mb.mbBytes;
-    io->ioObjectData.odSize= mb.mbCapacity;
+    io->ioObjectData= mb;
     io->ioKind= DOCokPICTWMETAFILE;
 
     tedScaleObjectToParagraph( ed, io );
