@@ -8,6 +8,9 @@
 
 #   include	<stdlib.h>
 #   include	<stdio.h>
+#   include	<stddef.h>
+#   include	<limits.h>
+
 #   include	<appDebugon.h>
 
 #   include	<appGeoString.h>
@@ -75,15 +78,17 @@ static void tedFormatToolRefreshParagraphPage(	ParagraphTool *	pt )
     appSetOptionmenu( &(pt->ptAlignOptionmenu), pp->ppAlignment );
 
     appGuiEnableWidget( pt->ptOnNewPageToggle, ! pp->ppInTable );
-    /*
-    appGuiEnableWidget( pt->ptOnOnePageToggle, ! pp->ppInTable );
-    */
 
     appGuiSetToggleState( pt->ptOnNewPageToggle, pp->ppStartsOnNewPage );
     appGuiSetToggleState( pt->ptOnOnePageToggle, pp->ppKeepOnPage );
 
+    appGuiSetToggleState( pt->ptWidctrlToggle, pp->ppWidowControl );
+    appGuiSetToggleState( pt->ptKeepWithNextToggle, pp->ppKeepWithNext );
+
+#   if 0
     appSetOptionmenu( &(pt->ptShadingTool.stStyleOptionmenu),
 						    pp->ppShadingPattern );
+#   endif
 
     return;
     }
@@ -320,7 +325,11 @@ static int tedFormatToolGetSpaceAround(		int *		pValue,
     {
     int			state;
     int			changed= 0;
-    const int		requirePositive= 1;
+
+    const int		minValue= 1;
+    const int		adaptToMin= 0;
+    const int		maxValue= INT_MAX;
+    const int		adaptToMax= 0;
 
     state= appGuiGetToggleState( toggle ) != 0;
     if  ( ! state )
@@ -328,7 +337,8 @@ static int tedFormatToolGetSpaceAround(		int *		pValue,
 
     state= value;
     if  ( appGetLengthFromTextWidget( text,
-			    &state, &changed, UNITtyPOINTS, requirePositive ) )
+			    &state, &changed, UNITtyPOINTS,
+			    minValue, adaptToMin, maxValue, adaptToMax ) )
 	{ return -1;	}
 
     if  ( changed )
@@ -343,7 +353,7 @@ static void tedFormatChangePara(	APP_WIDGET	w,
     {
     ParagraphTool *		pt= (ParagraphTool *)voidpt;
     EditApplication *		ea= pt->ptApplication;
-    ParagraphProperties *	ppNew= &(pt->ptPropertiesChosen);
+    ParagraphProperties *	pp= &(pt->ptPropertiesChosen);
 
     int				li;
     int				lipfi;
@@ -351,43 +361,62 @@ static void tedFormatChangePara(	APP_WIDGET	w,
     int				value;
     int				changed;
 
-    lipfi= ppNew->ppLeftIndentTwips+ ppNew->ppFirstIndentTwips;
+    unsigned int		ppUpdMask;
+
+    const int			maxValue= INT_MAX;
+    const int			adaptToMax= 0;
+
+    PROPmaskCLEAR( &ppUpdMask );
+    PROPmaskFILL( &ppUpdMask, PPprop_COUNT );
+
+    lipfi= pp->ppLeftIndentTwips+ pp->ppFirstIndentTwips;
     if  ( appGetLengthFromTextWidget( pt->ptFirstIndentText,
-					&lipfi, &changed, UNITtyPOINTS, 0 ) )
+					&lipfi, &changed, UNITtyPOINTS,
+					INT_MIN, 0, maxValue, adaptToMax ) )
 	{ return;	}
 
-    li= ppNew->ppLeftIndentTwips;
+    li= pp->ppLeftIndentTwips;
     if  ( appGetLengthFromTextWidget( pt->ptLeftIndentText,
-					&li, &changed, UNITtyPOINTS, 1 ) )
+					&li, &changed, UNITtyPOINTS,
+					1, 1, maxValue, adaptToMax ) )
 	{ return;	}
 
-    ppNew->ppLeftIndentTwips= li;
-    ppNew->ppFirstIndentTwips= lipfi- li;
+    pp->ppLeftIndentTwips= li;
+    pp->ppFirstIndentTwips= lipfi- li;
 
-    value= ppNew->ppRightIndentTwips;
+    value= pp->ppRightIndentTwips;
     if  ( appGetLengthFromTextWidget( pt->ptRightIndentText,
-					&value, &changed, UNITtyPOINTS, 1 ) )
+					&value, &changed, UNITtyPOINTS,
+					1, 1, maxValue, adaptToMax ) )
 	{ return;	}
-    ppNew->ppRightIndentTwips= value;
+    pp->ppRightIndentTwips= value;
 
-    value= ppNew->ppLineSpacingTwips;
+    value= pp->ppLineSpacingTwips;
     if  ( tedFormatToolGetHeight( &value, &(pt->ptLineDistChooser) ) )
 	{ return;	}
-    ppNew->ppLineSpacingTwips= value;
+    pp->ppLineSpacingTwips= value;
 
-    value= ppNew->ppSpaceBeforeTwips;
+    value= pp->ppSpaceBeforeTwips;
     if  ( tedFormatToolGetSpaceAround( &value, &changed,
 			pt->ptSpaceAboveText, pt->ptSpaceAboveToggle, value ) )
 	{ return;	}
-    ppNew->ppSpaceBeforeTwips= value;
+    pp->ppSpaceBeforeTwips= value;
 
-    value= ppNew->ppSpaceAfterTwips;
+    value= pp->ppSpaceAfterTwips;
     if  ( tedFormatToolGetSpaceAround( &value, &changed,
 			pt->ptSpaceBelowText, pt->ptSpaceBelowToggle, value ) )
 	{ return;	}
-    ppNew->ppSpaceAfterTwips= value;
+    pp->ppSpaceAfterTwips= value;
 
-    if  ( tedAppChangeParagraphProperties( ea, PPupdALL, ppNew ) )
+    if  ( tedBorderToolGetWidth( &(pt->ptTopBorderTool),
+						&(pp->ppTopBorder) ) )
+	{ return;	}
+
+    if  ( tedBorderToolGetWidth( &(pt->ptBottomBorderTool),
+						&(pp->ppBottomBorder) ) )
+	{ return;	}
+
+    if  ( tedAppChangeParagraphProperties( ea, ppUpdMask, pp ) )
 	{ LDEB(1);	}
 
     return;
@@ -495,6 +524,7 @@ static void tedParaAlignmentChosen(	APP_WIDGET	w,
     return;
     }
 
+#   if 0
 static void tedParaShadingChosen(	APP_WIDGET	w,
 					void *		voidpt,
 					void *		voidtbcs )
@@ -516,6 +546,7 @@ static void tedParaShadingChosen(	APP_WIDGET	w,
 
     return;
     }
+#   endif
 
 static void tedParaBorderStyleChosen(	APP_WIDGET	w,
 					void *		voidpt,
@@ -601,7 +632,21 @@ static void tedParagraphToggleChanged(	APP_WIDGET	w,
 
     if  ( w == pt->ptOnOnePageToggle )
 	{
-	pp->ppKeepOnPage= ( set != 0 ) && ! pp->ppInTable;
+	pp->ppKeepOnPage= ( set != 0 );
+
+	return;
+	}
+
+    if  ( w == pt->ptWidctrlToggle )
+	{
+	pp->ppWidowControl= ( set != 0 );
+
+	return;
+	}
+
+    if  ( w == pt->ptKeepWithNextToggle )
+	{
+	pp->ppKeepWithNext= ( set != 0 );
 
 	return;
 	}
@@ -685,9 +730,12 @@ void tedFormatFillParagraphPage( ParagraphTool *		pt,
 
     tedFormatMakeBorderTool( &(pt->ptBottomBorderTool), &row, pageWidget,
 		ppr->pprBottomBorder, tedParagraphToggleChanged, (void *)pt );
+
+#   if 0
     /**/
     tedFormatMakeShadingTool( &(pt->ptShadingTool), &row,
 				    pageWidget, ppr->pprShadingPattern );
+#   endif
 
 
     /**/
@@ -697,7 +745,20 @@ void tedFormatFillParagraphPage( ParagraphTool *		pt,
 		    tedParagraphToggleChanged,
 		    tedParagraphToggleChanged, (void *)pt );
 
+    /*
     appGuiEnableWidget( pt->ptOnOnePageToggle, 0 );
+    */
+
+    /**/
+    appInspectorMakeToggleRow( &row, pageWidget,
+		    &(pt->ptWidctrlToggle), &(pt->ptKeepWithNextToggle),
+		    ppr->pprWidctrl, ppr->pprKeepWithNext,
+		    tedParagraphToggleChanged,
+		    tedParagraphToggleChanged, (void *)pt );
+
+    /*
+    appGuiEnableWidget( pt->ptKeepWithNextToggle, 0 );
+    */
 
     /**/
     appInspectorMakeButtonRow( &row, pageWidget,
@@ -743,8 +804,10 @@ void tedFormatFillParagraphChoosers(	ParagraphTool *			pt,
 			&(pt->ptAlignOptionmenu),
 			tedParaAlignmentChosen, (void *)pt );
 
+#   if 0
     tedFormatFillShadingTool( &(pt->ptShadingTool),
 					tedParaShadingChosen, (void *)pt );
+#   endif
 
     tedFormatFillBorderTool( &(pt->ptTopBorderTool),
 					tedParaBorderStyleChosen, (void *)pt );
@@ -763,7 +826,9 @@ void tedFormatFinishParagraphPage(	ParagraphTool *			pt,
 
     appOptionmenuRefreshWidth( &(pt->ptAlignOptionmenu) );
 
+#   if 0
     tedFormatFinishShadingTool( &(pt->ptShadingTool), tft );
+#   endif
 
     tedFormatFinishBorderTool( &(pt->ptTopBorderTool), tft );
     tedFormatFinishBorderTool( &(pt->ptBottomBorderTool), tft );
@@ -871,6 +936,13 @@ static AppConfigurableResource TED_TedParagraphToolResourceTable[]=
     APP_RESOURCE( "formatToolParaOnOnePage",
 	offsetof(ParagraphPageResources,pprOnOnePage),
 	"Keep on One Page" ),
+    /**/
+    APP_RESOURCE( "formatToolParaWidctrl",
+	offsetof(ParagraphPageResources,pprWidctrl),
+	"Widow/Orphan Control" ),
+    APP_RESOURCE( "formatToolParaKeepWithNext",
+	offsetof(ParagraphPageResources,pprKeepWithNext),
+	"Keep with Next" ),
     };
 
 void tedFormatToolGetParaResourceTable(	EditApplication *		ea,
