@@ -8,6 +8,9 @@
 
 #   include	<stdlib.h>
 #   include	<stdio.h>
+#   include	<stddef.h>
+#   include	<limits.h>
+
 #   include	<appDebugon.h>
 
 #   include	<appGeoString.h>
@@ -103,21 +106,30 @@ static void tedFormatChangeRowPushed(	APP_WIDGET	w,
     {
     RowTool *			rt= (RowTool *)voidrt;
     RowProperties *		rp= &(rt->rtPropertiesChosen);
+    CellProperties *		cp= rp->rpCells;
 
     TableRectangle		tr;
 
+    int				col;
+
     int				width;
     int				changed;
 
     int				res;
 
+    const int			minValue= 1;
+    const int			adaptToMin= 0;
+    const int			maxValue= INT_MAX;
+    const int			adaptToMax= 0;
+
     if  ( tedFormatToolGetHeight( &(rp->rpHeightTwips),
 						&(rt->rtHeightChooser) ) )
 	{ return;	}
 
     width= rp->rpHalfGapWidthTwips;
-    if  ( appGetLengthFromTextWidget( rt->rtCellMarginText, &width, &changed,
-							UNITtyPOINTS, 1 ) )
+    if  ( appGetLengthFromTextWidget( rt->rtCellMarginText,
+				&width, &changed, UNITtyPOINTS,
+				minValue, adaptToMin, maxValue, adaptToMax ) )
 	{ return;	}
     rp->rpHalfGapWidthTwips= width;
 
@@ -130,6 +142,23 @@ static void tedFormatChangeRowPushed(	APP_WIDGET	w,
     tr= rt->rtTableRectangle;
     docExpandTableRectangleToWholeRows( &tr );
 
+    if  ( tedBorderToolGetWidth( &(rt->rtTopBorderTool),
+					&(rp->rpCells[0].cpTopBorder) ) )
+	{ return;	}
+
+    if  ( tedBorderToolGetWidth( &(rt->rtBottomBorderTool),
+					&(rp->rpCells[0].cpBottomBorder) ) )
+	{ return;	}
+
+    cp= rp->rpCells+ 1;
+    for ( col= 1; col < rp->rpCellCount; cp++, col++ )
+	{
+	cp->cpTopBorder.bpPenWideTwips=
+			    rp->rpCells[0].cpTopBorder.bpPenWideTwips;
+	cp->cpBottomBorder.bpPenWideTwips=
+			    rp->rpCells[0].cpBottomBorder.bpPenWideTwips;
+	}
+
     tedAppSetTableProperties( rt->rtApplication, &tr, rp );
 
     return;
