@@ -1297,9 +1297,9 @@ void tedHorizontalRulerTrackMouse(	int *			pFirstIndent,
 		    {
 		    *pTabCount= ttr->ttrTabCount;
 		    *pTabStops= ttr->ttrTabStops;
-		    *pChange= PPupdTAB_STOPS;
+		    *pChange= PPpropTAB_STOPS;
 		    }
-		else{ *pChange= PPupdNONE; }
+		else{ *pChange= PPprop_NONE; }
 
 		return;
 		}
@@ -1323,9 +1323,9 @@ void tedHorizontalRulerTrackMouse(	int *			pFirstIndent,
 	    *pFirstIndent= ttr->ttrFirstIndent;
 	    *pTabCount= ttr->ttrTabCount;
 	    *pTabStops= ttr->ttrTabStops;
-	    *pChange= PPupdFIRST_INDENT;
+	    *pChange= PPpropFIRST_INDENT;
 	    }
-	else{ *pChange= PPupdNONE; }
+	else{ *pChange= PPprop_NONE; }
 	return;
 	}
 
@@ -1347,9 +1347,9 @@ void tedHorizontalRulerTrackMouse(	int *			pFirstIndent,
 	    *pLeftIndent= ttr->ttrLeftIndent;
 	    *pTabCount= ttr->ttrTabCount;
 	    *pTabStops= ttr->ttrTabStops;
-	    *pChange= PPupdLEFT_INDENT;
+	    *pChange= PPpropLEFT_INDENT;
 	    }
-	else{ *pChange= PPupdNONE; }
+	else{ *pChange= PPprop_NONE; }
 	return;
 	}
 
@@ -1371,9 +1371,9 @@ void tedHorizontalRulerTrackMouse(	int *			pFirstIndent,
 	    *pRightIndent= ttr->ttrRightIndent;
 	    *pTabCount= ttr->ttrTabCount;
 	    *pTabStops= ttr->ttrTabStops;
-	    *pChange= PPupdRIGHT_INDENT;
+	    *pChange= PPpropRIGHT_INDENT;
 	    }
-	else{ *pChange= PPupdNONE; }
+	else{ *pChange= PPprop_NONE; }
 	return;
 	}
 
@@ -1389,7 +1389,7 @@ void tedHorizontalRulerTrackMouse(	int *			pFirstIndent,
 	fresh= (TabStop *)realloc( ttr->ttrTabStops,
 				    ( ttr->ttrTabCount+ 1)* sizeof(TabStop) );
 	if  ( ! fresh )
-	    { LXDEB(ttr->ttrTabCount,fresh); *pChange= PPupdNONE; return; }
+	    { LXDEB(ttr->ttrTabCount,fresh); *pChange= PPprop_NONE; return; }
 
 	ttr->ttrTabStops= fresh;
 
@@ -1415,7 +1415,7 @@ void tedHorizontalRulerTrackMouse(	int *			pFirstIndent,
 
 	*pTabCount= ttr->ttrTabCount;
 	*pTabStops= ttr->ttrTabStops;
-	*pChange= PPupdTAB_STOPS; return;
+	*pChange= PPpropTAB_STOPS; return;
 	}
 
     /*  8  */
@@ -1451,9 +1451,9 @@ void tedHorizontalRulerTrackMouse(	int *			pFirstIndent,
 		if  ( changed )
 		    {
 		    *pCsCount= ttr->ttrColumnCount; *pCs= ttr->ttrColumns;
-		    *pChange= PPupdCOLUMNS;
+		    *pChange= PPpropCOLUMNS;
 		    }
-		else{ *pChange= PPupdNONE;	}
+		else{ *pChange= PPprop_NONE;	}
 		return;
 		}
 	    }
@@ -1479,7 +1479,7 @@ void tedHorizontalRulerTrackMouse(	int *			pFirstIndent,
 			dr.drX1- dr.drX0+ 1, dr.drY1- dr.drY0+ 1 );
 	}
 
-    *pChange= PPupdNONE; return;
+    *pChange= PPprop_NONE; return;
     }
 
 /************************************************************************/
