@@ -8,6 +8,7 @@
 
 #   include	<stdlib.h>
 #   include	<stdio.h>
+#   include	<stddef.h>
 #   include	<limits.h>
 #   include	<appDebugon.h>
 
@@ -117,6 +118,13 @@ static void tedFormatChangeSectPushed(	APP_WIDGET	w,
 
     int				changed;
 
+    unsigned int		update;
+
+    const int			minValue= 1;
+    const int			adaptToMin= 0;
+    const int			maxValue= INT_MAX;
+    const int			adaptToMax= 0;
+
     spNew->spRestartPageNumbers=
 		appGuiGetToggleState( st->stPageRestartToggle ) != 0;
 
@@ -137,14 +145,19 @@ static void tedFormatChangeSectPushed(	APP_WIDGET	w,
 	{ return;	}
 
     if  ( appGetLengthFromTextWidget( st->stColumnSpacingText,
-		&(spNew->spColumnSpacingTwips), &changed, UNITtyPOINTS, 1 ) )
+		    &(spNew->spColumnSpacingTwips), &changed, UNITtyPOINTS,
+		    minValue, adaptToMin, maxValue, adaptToMax ) )
 	{ return;	}
 
     if  ( appGetLengthFromTextWidget( st->stColumnWidthText,
-		&(spNew->spColumnWidthTwips), &changed, UNITtyPOINTS, 1 ) )
+		    &(spNew->spColumnWidthTwips), &changed, UNITtyPOINTS,
+		    minValue, adaptToMin, maxValue, adaptToMax ) )
 	{ return;	}
 
-    if  ( tedAppChangeSectionProperties( st->stApplication, SPupdALL, spNew ) )
+    PROPmaskCLEAR( &update );
+    PROPmaskFILL( &update, SPprop_COUNT );
+
+    if  ( tedAppChangeSectionProperties( st->stApplication, update, spNew ) )
 	{ LDEB(1);	}
 
     return;
