@@ -12,6 +12,7 @@
 #   include	<ctype.h>
 #   include	<limits.h>
 
+#   include	"docPs.h"
 #   include	"tedApp.h"
 
 #   include	<appDebugon.h>
@@ -67,17 +68,20 @@ int tedGetSelection(	SelectionScope *	ss,
 void tedDelimitCurrentSelection(	TedDocument *		td,
 					AppDrawingData *	add )
     {
+    BufferDocument *		bd= td->tdDocument;
+    const DocumentProperties *	dp= &(bd->bdProperties);
+
     docFindLineAndParticule( td->tdSelection.bsBegin.bpBi,
-		td->tdSelection.bsBegin.bpStroff,
-		&(td->tdSelection.bsBegin), 1 );
+					td->tdSelection.bsBegin.bpStroff,
+					&(td->tdSelection.bsBegin), 1 );
     docFindLineAndParticule( td->tdSelection.bsEnd.bpBi,
-		td->tdSelection.bsEnd.bpStroff,
-		&(td->tdSelection.bsEnd), 0 );
+					td->tdSelection.bsEnd.bpStroff,
+					&(td->tdSelection.bsEnd), 0 );
 
     tedSelectionCoordinates( &(td->tdSelection), add );
 
     tedSelectionRectangle( &(td->tdSelectedRectangle),
-						 add, &(td->tdSelection) );
+					     add, dp, &(td->tdSelection) );
     }
 
 /************************************************************************/
@@ -118,8 +122,10 @@ static void tedScrollToPosition(	EditDocument *		ed,
 					int *			pScrolledX,
 					int *			pScrolledY )
     {
+    AppDrawingData *		add= &(ed->edDrawingData);
+
     appScrollToRectangle( ed,
-			bp->bpXPixels, bp->bpTopPosition.lpYPixels,
+			bp->bpXPixels, BP_TOP_PIXELS( add, bp ),
 			bp->bpXPixels, bp->bpY1Pixels,
 			pScrolledX, pScrolledY );
 
@@ -674,7 +680,7 @@ int tedPrevLine(	const AppDrawingData *	add,
     if  ( docPrevLine( &tp, &tl, bp ) )
 	{ return -1;	}
 
-    baseline= tl->tlTopPosition.lpYPixels+ tl->tlLineAscentPixels;
+    baseline= TL_BASE_PIXELS( add, tl );
 
     part= tedFindParticule( tl, bp->bpBi->biParaParticules, x, baseline );
     if  ( part < 0 )
@@ -708,7 +714,7 @@ int tedNextLine(	const AppDrawingData *	add,
     if  ( docNextLine( &tp, &tl, bp ) )
 	{ return -1;	}
 
-    baseline= tl->tlTopPosition.lpYPixels+ tl->tlLineAscentPixels;
+    baseline= TL_BASE_PIXELS( add, tl );
 
     part= tedFindParticule( tl, bp->bpBi->biParaParticules, x, baseline );
     if  ( part < 0 )
@@ -777,11 +783,13 @@ void tedScrollToSelection(	EditDocument *		ed,
 				int *			pScrolledX,
 				int *			pScrolledY )
     {
+    const AppDrawingData *	add= &(ed->edDrawingData);
     TedDocument *		td= (TedDocument *)ed->edPrivateData;
     const BufferSelection *	bs= &(td->tdSelection);
 
     appScrollToRectangle( ed,
-		    bs->bsBegin.bpXPixels, bs->bsBegin.bpTopPosition.lpYPixels,
+		    bs->bsBegin.bpXPixels,
+		    BP_TOP_PIXELS( add, &(bs->bsBegin) ),
 		    bs->bsEnd.bpXPixels, bs->bsEnd.bpY1Pixels,
 		    pScrolledX, pScrolledY );
 
@@ -846,9 +854,11 @@ void tedSetSelection(	EditDocument *		ed,
 			int *			pScrolledX,
 			int *			pScrolledY )
     {
+    EditApplication *		ea= ed->edApplication;
+
     TedDocument *		td= (TedDocument *)ed->edPrivateData;
     BufferDocument *		bd= td->tdDocument;
-    DocumentProperties *	dp= &(bd->bdProperties);
+    const DocumentProperties *	dp= &(bd->bdProperties);
     AppDrawingData *		add= &(ed->edDrawingData);
     int				hadSelection;
 
@@ -862,11 +872,12 @@ void tedSetSelection(	EditDocument *		ed,
 
     DocumentRectangle		drExpose;
 
-    DocumentRectangle		drHeadFoot;
+    DocumentRectangle		drOldHeadFoot;
+    DocumentRectangle		drNewHeadFoot;
     int				movedFromHeaderFooter= 0;
     int				movedToHeaderFooter= 0;
 
-    HeaderFooter *		hf;
+    HeaderFooter *		hf= (HeaderFooter *)0;
     BufferItem *		sectBi;
 
     /*  0  */
@@ -891,7 +902,7 @@ void tedSetSelection(	EditDocument *		ed,
 
 	    if  ( movedFromHeaderFooter )
 		{
-		if  ( tedGetHeaderFooterBox( &drHeadFoot, sectBi, dp, add,
+		if  ( tedGetHeaderFooterBox( &drOldHeadFoot, sectBi, dp, add,
 						ssOld.ssHeaderFooterPage,
 						ssOld.ssInHeaderFooter ) )
 		    {
@@ -902,6 +913,37 @@ void tedSetSelection(	EditDocument *		ed,
 	    }
 	}
 
+    if  ( ssSet->ssInHeaderFooter != DOCinBODY )
+	{
+	if  ( docGetHeaderFooter( &hf, &sectBi,
+				ssSet, &bsNew, bd, ssSet->ssInHeaderFooter ) )
+	    { LDEB(ssSet->ssInHeaderFooter);	}
+	else{
+	    int		done= 0;
+
+	    if  ( ssOld.ssInHeaderFooter != ssSet->ssInHeaderFooter )
+		{ movedToHeaderFooter= 1;	}
+	    if  ( ssOld.ssHeaderFooterSection != ssSet->ssHeaderFooterSection )
+		{ movedToHeaderFooter= 1;	}
+	    if  ( ssOld.ssHeaderFooterPage != ssSet->ssHeaderFooterPage )
+		{ movedToHeaderFooter= 1;	}
+
+	    if  ( movedToHeaderFooter )
+		{
+		if  ( tedGetHeaderFooterBox( &drNewHeadFoot, sectBi, dp, add,
+						ssSet->ssHeaderFooterPage,
+						ssSet->ssInHeaderFooter ) )
+		    { LDEB(ssSet->ssInHeaderFooter); movedToHeaderFooter= 0; }
+		}
+
+	    if  ( tedRedoHeaderFooterLayout( &done, hf,
+						ssSet->ssHeaderFooterPage,
+						sectBi, bd, add,
+						&drNewHeadFoot ) )
+		{ LDEB(1);	}
+	    }
+	}
+
     /*  A  */
     bsNew= *bsSet;
     docBalanceFieldSelection( &beginMoved, &endMoved, td->tdDocument,
@@ -916,7 +958,7 @@ void tedSetSelection(	EditDocument *		ed,
 	drExpose= td->tdSelectedRectangle;
 
 	if  ( movedFromHeaderFooter )
-	    { docUnionRectangle( &drExpose, &drExpose, &drHeadFoot ); }
+	    { docUnionRectangle( &drExpose, &drExpose, &drOldHeadFoot ); }
 
 	tedExposeRectangle( ed, &drExpose, *pScrolledX, *pScrolledY );
 
@@ -956,42 +998,19 @@ void tedSetSelection(	EditDocument *		ed,
 	drExpose= td->tdSelectedRectangle;
 
 	if  ( movedFromHeaderFooter )
-	    { docUnionRectangle( &drExpose, &drExpose, &drHeadFoot ); }
+	    { docUnionRectangle( &drExpose, &drExpose, &drOldHeadFoot ); }
 
 	tedExposeRectangle( ed, &drExpose, *pScrolledX, *pScrolledY );
 	}
 
-    if  ( ssSet->ssInHeaderFooter != DOCinBODY )
-	{
-	if  ( docGetHeaderFooter( &hf, &sectBi,
-				ssSet, &bsNew, bd, ssSet->ssInHeaderFooter ) )
-	    { LDEB(ssSet->ssInHeaderFooter);	}
-	else{
-	    if  ( ssOld.ssInHeaderFooter != ssSet->ssInHeaderFooter )
-		{ movedToHeaderFooter= 1;	}
-	    if  ( ssOld.ssHeaderFooterSection != ssSet->ssHeaderFooterSection )
-		{ movedToHeaderFooter= 1;	}
-	    if  ( ssOld.ssHeaderFooterPage != ssSet->ssHeaderFooterPage )
-		{ movedToHeaderFooter= 1;	}
-
-	    if  ( movedToHeaderFooter )
-		{
-		if  ( tedGetHeaderFooterBox( &drHeadFoot, sectBi, dp, add,
-						ssSet->ssHeaderFooterPage,
-						ssSet->ssInHeaderFooter ) )
-		    { LDEB(ssSet->ssInHeaderFooter); movedToHeaderFooter= 0; }
-		}
-	    }
-	}
-
     /*  6  */
     tedSelectionRectangle( &(td->tdSelectedRectangle),
-						add, &(td->tdSelection) );
+						add, dp, &(td->tdSelection) );
 
     drExpose= td->tdSelectedRectangle;
 
     if  ( movedToHeaderFooter )
-	{ docUnionRectangle( &drExpose, &drExpose, &drHeadFoot ); }
+	{ docUnionRectangle( &drExpose, &drExpose, &drNewHeadFoot ); }
 
     tedExposeRectangle( ed, &drExpose, *pScrolledX, *pScrolledY );
 
@@ -999,6 +1018,9 @@ void tedSetSelection(	EditDocument *		ed,
 	{ tedSetCurrentTextAttribute( td, 1, &(td->tdSelection.bsBegin) ); }
     else{ tedSetCurrentTextAttribute( td, 0, &(td->tdSelection.bsBegin) ); }
 
+    if  ( ea->eaFindTool )
+	{ appFindToolDisableReplace( ea->eaFindTool );	}
+
     return;
     }
 
@@ -1341,6 +1363,8 @@ static int TED_TryPageOffsets[]=
     };
 
 void tedAppEditHeaderFooter(		EditApplication *	ea,
+					APP_WIDGET		relative,
+					APP_WIDGET		option,
 					int			which )
     {
     EditDocument *		ed= ea->eaCurrentDocument;
@@ -1366,7 +1390,6 @@ void tedAppEditHeaderFooter(		EditApplication *	ea,
 
     SelectionScope		ssHeadFoot;
 
-    int				done;
     DocumentRectangle		drChanged;
 
     if  ( ! ed )
@@ -1393,7 +1416,8 @@ void tedAppEditHeaderFooter(		EditApplication *	ea,
 	{
 	BufferItem *	paraBi;
 
-	hf->hfItem= docMakeExternalItem( bd, &(sectBi->biSectProperties) );
+	hf->hfItem= docMakeExternalItem(
+				bd, &(sectBi->biSectProperties), which );
 	if  ( ! hf->hfItem )
 	    { XDEB(hf->hfItem); return;	}
 
@@ -1458,15 +1482,13 @@ void tedAppEditHeaderFooter(		EditApplication *	ea,
     if  ( page < sectBi->biTopPosition.lpPage		||
 	  page > sectBi->biBelowPosition.lpPage		)
 	{
-	LDEB(page);
-	LLDEB(sectBi->biTopPosition.lpPage,sectBi->biBelowPosition.lpPage);
+	TedAppResources *	tar= (TedAppResources *)ea->eaResourceData;
+
+	appQuestionRunErrorDialog( ea, relative, option,
+					    tar->tarHeadFootNotReachable );
 	return;
 	}
 
-    if  ( tedRedoHeaderFooterLayout( &done, hf, page, which,
-					    sectBi, bd, add, &drChanged ) )
-	{ LDEB(page); return;	}
-
     if  ( docFirstPosition( hf->hfItem, &bp ) )
 	{ LDEB(1); return;	}
 
@@ -1479,9 +1501,6 @@ void tedAppEditHeaderFooter(		EditApplication *	ea,
 
     tedAdaptToolsToPosition( ed );
 
-    if  ( done )
-	{ tedExposeRectangle( ed, &drChanged, scrolledX, scrolledY ); }
-
     return;
     }
 
