@@ -331,6 +331,7 @@ int tedDeleteRowsFromTable(	EditDocument *		ed,
     {
     TedDocument *		td= (TedDocument *)ed->edPrivateData;
     BufferDocument *		bd= td->tdDocument;
+    DocumentProperties *	dp= &(bd->bdProperties);
     BufferItem *		bi= td->tdSelection.bsBegin.bpBi;
     AppDrawingData *		add= &(ed->edDrawingData);
 
@@ -351,6 +352,9 @@ int tedDeleteRowsFromTable(	EditDocument *		ed,
     DocumentRectangle		drChanged;
 
     SelectionScope *		ss= &(td->tdSelectionScope);
+    HeaderFooter *		hf= (HeaderFooter *)0;
+
+    BufferItem *		docSectBi;
 
     if  ( docDelimitTable( bi, &sectBi, &col, &row0, &row, &row1 ) )
 	{ LDEB(1); return -1;	}
@@ -359,10 +363,10 @@ int tedDeleteRowsFromTable(	EditDocument *		ed,
     drChanged.drX1= add->addBackRect.drX1;
 
     rowBi= sectBi->biChildren[delRow0];
-    drChanged.drY0= rowBi->biTopPosition.lpYPixels- 1;
+    drChanged.drY0= BI_TOP_PIXELS( add, rowBi )- 1;
 
     rowBi= sectBi->biChildren[delRow0];
-    drChanged.drY1= rowBi->biBelowPosition.lpYPixels- 1;
+    drChanged.drY1= BI_BELOW_PIXELS( add, rowBi )- 1;
 
     docInitPosition( &bpNew );
     if  ( docLastPosition( sectBi->biChildren[delRow0], &bpNew )	||
@@ -378,6 +382,8 @@ int tedDeleteRowsFromTable(	EditDocument *		ed,
 
     switch( ss->ssInHeaderFooter )
 	{
+	int		inHeaderFooter;
+
 	case DOCinBODY:
 	    tedLayoutItem( sectBi, bd, ss->ssInHeaderFooter, add, &drChanged );
 	    break;
@@ -386,24 +392,38 @@ int tedDeleteRowsFromTable(	EditDocument *		ed,
 	case DOCinFIRST_HEADER:
 	case DOCinLEFT_HEADER:
 	case DOCinRIGHT_HEADER:
-	    sp= &(bd->bdItem.biChildren[
-				ss->ssHeaderFooterSection]->biSectProperties);
-	    page= ss->ssHeaderFooterPage;
 
-	    tedLayoutHeaderItem( sectBi, page, ss->ssInHeaderFooter, sp, bd,
-							    add, &drChanged );
+	    page= ss->ssHeaderFooterPage;
+	    docSectBi= bd->bdItem.biChildren[ss->ssHeaderFooterSection];
+	    sp= &(docSectBi->biSectProperties);
+
+	    inHeaderFooter= docWhatPageHeader( &hf, docSectBi, page, dp );
+
+	    if  ( ! hf						||
+		  inHeaderFooter != ss->ssInHeaderFooter	||
+		  hf->hfItem != sectBi				)
+		{ LDEB(1); return -1;	}
+
+	    tedLayoutExternal( hf, page, sp, bd, add, &drChanged );
 	    break;
 
 	case DOCinSECT_FOOTER:
 	case DOCinFIRST_FOOTER:
 	case DOCinLEFT_FOOTER:
 	case DOCinRIGHT_FOOTER:
-	    sp= &(bd->bdItem.biChildren[
-				ss->ssHeaderFooterSection]->biSectProperties);
-	    page= ss->ssHeaderFooterPage;
 
-	    tedLayoutFooterItem( sectBi, page, ss->ssInHeaderFooter, sp, bd,
-							    add, &drChanged );
+	    page= ss->ssHeaderFooterPage;
+	    docSectBi= bd->bdItem.biChildren[ss->ssHeaderFooterSection];
+	    sp= &(docSectBi->biSectProperties);
+
+	    inHeaderFooter= docWhatPageFooter( &hf, docSectBi, page, dp );
+
+	    if  ( ! hf						||
+		  inHeaderFooter != ss->ssInHeaderFooter	||
+		  hf->hfItem != sectBi				)
+		{ LDEB(1); return -1;	}
+
+	    tedLayoutExternal( hf, page, sp, bd, add, &drChanged );
 	    break;
 
 	default:
@@ -481,10 +501,10 @@ static int tedSplitColumnInRows(	EditDocument *	ed,
     drChanged.drX1= add->addBackRect.drX1;
 
     rowBi= sectBi->biChildren[row0];
-    drChanged.drY0= rowBi->biTopPosition.lpYPixels- 1;
+    drChanged.drY0= BI_TOP_PIXELS( add, rowBi )- 1;
 
     rowBi= sectBi->biChildren[row1];
-    drChanged.drY1= rowBi->biBelowPosition.lpYPixels- 1;
+    drChanged.drY1= BI_BELOW_PIXELS( add, rowBi )- 1;
 
     while( row0 <= row1 )
 	{
@@ -608,10 +628,10 @@ int tedDeleteColumnsFromRows(	EditDocument *	ed,
     drChanged.drX1= add->addBackRect.drX1;
 
     rowBi= sectBi->biChildren[row0];
-    drChanged.drY0= rowBi->biTopPosition.lpYPixels;
+    drChanged.drY0= BI_TOP_PIXELS( add, rowBi );
 
     rowBi= sectBi->biChildren[row1];
-    drChanged.drY1= rowBi->biBelowPosition.lpYPixels- 1;
+    drChanged.drY1= BI_BELOW_PIXELS( add, rowBi )- 1;
 
     docInitPosition( &bpNew );
     rowBi= sectBi->biChildren[delRow1];
@@ -818,10 +838,10 @@ void tedChangeTableLayout(	EditDocument *		ed,
     drChanged.drX1= add->addBackRect.drX1;
 
     rowBi= sectBi->biChildren[row0];
-    drChanged.drY0= rowBi->biTopPosition.lpYPixels;
+    drChanged.drY0= BI_TOP_PIXELS( add, rowBi );
 
     rowBi= sectBi->biChildren[row1];
-    drChanged.drY1= rowBi->biBelowPosition.lpYPixels- 1;
+    drChanged.drY1= BI_BELOW_PIXELS( add, rowBi )- 1;
 
     while( row0 <= row1 )
 	{
