@@ -521,6 +521,7 @@ static int appMakeFileChooser( AppChooserInformation **	pAci,
     XtSetArg( al[ac], XmNmwmFunctions,		hints.functions ); ac++;
     XtSetArg( al[ac], XmNdeleteResponse,	XmDO_NOTHING ); ac++;
     XtSetArg( al[ac], XmNallowShellResize,	True ); ac++;
+    XtSetArg( al[ac], XmNuseAsyncGeometry,	True ); ac++;
 
     XtSetArg( al[ac], XmNchildPlacement,XmPLACE_ABOVE_SELECTION ); ac++;
 #   if	XmVersion >= 2000
@@ -534,8 +535,9 @@ static int appMakeFileChooser( AppChooserInformation **	pAci,
 	XtSetArg( al[ac], XmNpattern, filterString ); ac++;
 	}
 
-    aci->aciDialog.adDialog= XmCreateFileSelectionDialog( ea->eaToplevel.atTopWidget,
-							WIDGET_NAME, al, ac );
+    aci->aciDialog.adDialog= XmCreateFileSelectionDialog(
+						ea->eaToplevel.atTopWidget,
+						WIDGET_NAME, al, ac );
     aci->aciDialog.adTopWidget= XtParent( aci->aciDialog.adDialog );
 
     if  ( filterString )
@@ -662,7 +664,8 @@ void appRunOpenChooser( Widget			option,
 	for ( i= 0; i < childCount; afe++, i++ )
 	    {
 	    if  ( ! ( afe->afeUseFlags & APPFILE_CAN_OPEN )	||
-		   ( afe->afeUseFlags & APPFILE_IS_BASIC_TYPE )	)
+		  ( afe->afeUseFlags & APPFILE_IS_BASIC_TYPE )	||
+		  ( afe->afeUseFlags & APPFILE_HIDE_OPEN )	)
 		{ XtUnmanageChild( children[i] ); continue;	}
 
 	    XtManageChild( children[i] );
@@ -733,6 +736,10 @@ static void appSaveOkPushed(		Widget		fileChooser,
 	    appGuiBreakDialog( &(aci->aciDialog), ACIrespSAVE );
 	    return;
 
+	case ACIrespCANCEL:
+	    appGuiBreakDialog( &(aci->aciDialog), ACIrespCANCEL );
+	    return;
+
 	case ACIrespNONE:
 	    free( aci->aciFilename ); aci->aciFilename= (char *)0;
 	    return;
@@ -904,7 +911,7 @@ void appDocFileSaveAs(	APP_WIDGET	option,
 	{ XDEB(ea->eaSaveDocument); return;	}
 
     response= appRunSaveChooser( option, ed->edToplevel.atTopWidget, ea, ed,
-		( APPFILE_CAN_OPEN|APPFILE_CAN_SAVE ),
+					( APPFILE_CAN_SAVE|APPFILE_CAN_OPEN ),
 		&format, &filename );
 
     switch( response )
@@ -949,7 +956,7 @@ void appDocFileSaveTo(	APP_WIDGET	option,
 	{ XDEB(ea->eaSaveDocument); return;	}
 
     response= appRunSaveChooser( option, ed->edToplevel.atTopWidget, ea, ed,
-					APPFILE_CAN_SAVE, &format, &filename );
+				    APPFILE_CAN_SAVE, &format, &filename );
 
     switch( response )
 	{
