@@ -121,6 +121,8 @@ static void appFindToolSetFindResult(	AppFindTool *	aft,
     else{
 	char *	replacement;
 
+	appGuiEnableWidget( aft->aftReplaceFrame, 1 );
+
 	replacement= appGetStringFromTextWidget( aft->aftReplaceText );
 
 	sensitive= replacement[0] != '\0';
@@ -700,17 +702,7 @@ void appEnableFindTool(		void *	voidaft,
     return;
     }
 
-void appFindToolTextReadonly(		void *	voidaft,
-					int	readonly )
-    {
-    AppFindTool *	aft= (AppFindTool *)voidaft;
-
-    appGuiEnableWidget( aft->aftReplaceFrame, readonly == 0 );
-
-    return;
-    }
-
-void appFindToolFoundObsolete(		void *	voidaft )
+void appFindToolDisableReplace(		void *	voidaft )
     {
     AppFindTool *	aft= (AppFindTool *)voidaft;
 
@@ -718,3 +710,4 @@ void appFindToolFoundObsolete(		void *	voidaft )
 
     return;
     }
+
