@@ -12,6 +12,8 @@
 
 #   include	<appDebugon.h>
 
+#   include	<utilPropMask.h>
+
 #   include	"appDraw.h"
 #   include	<bitmap.h>
 
@@ -1101,12 +1103,15 @@ int appOpenDocumentFont(	AppDrawingData *		add,
     }
 
 /************************************************************************/
+/*									*/
 /*  Format the current font name.					*/
+/*									*/
 /************************************************************************/
+
 void appFontFormatCurrent(	char *			target,
 				AppFontFamily *		aff,
 				AppFontTypeface *	aft,
-				int			size )
+				int			sizeHalfPoints )
     {
     const char *	familyText= "*";
     const char *	faceName= "*";
@@ -1125,8 +1130,12 @@ void appFontFormatCurrent(	char *			target,
     if  ( aft )
 	{ faceName= aft->aftFaceName;	}
 
-    if  ( size > 0 )
-	{ sprintf( sizeStr, "%d", size ); }
+    if  ( sizeHalfPoints > 0 )
+	{
+	if  ( sizeHalfPoints % 2 )
+	    { sprintf( sizeStr, "%d.5", sizeHalfPoints/ 2 );	}
+	else{ sprintf( sizeStr, "%d", sizeHalfPoints/ 2 );	}
+	}
 
     sprintf( target, "%s,%s,%s", familyText, faceName, sizeStr );
 
@@ -1136,6 +1145,7 @@ void appFontFormatCurrent(	char *			target,
 /************************************************************************/
 /*  Format the current attributes.					*/
 /************************************************************************/
+
 void appFontAttributeString(	char *			target,
 				const char *		familyName,
 				unsigned int		updMask,
@@ -1156,24 +1166,28 @@ void appFontAttributeString(	char *			target,
     sizeStr[0]= '*';
     sizeStr[1]= '\0';
 
-    if  ( updMask & TAupdFONTSIZE )
-	{ sprintf( sizeStr, "%d", taNew.taFontSizeHalfPoints/ 2 ); }
+    if  ( PROPmaskISSET( updMask, TApropFONTSIZE ) )
+	{
+	if  ( taNew.taFontSizeHalfPoints % 2 )
+	    { sprintf( sizeStr, "%d.5", taNew.taFontSizeHalfPoints/ 2 ); }
+	else{ sprintf( sizeStr, "%d", taNew.taFontSizeHalfPoints/ 2 );	 }
+	}
 
-    if  ( updMask & TAupdFONTBOLD )
+    if  ( PROPmaskISSET( updMask, TApropFONTBOLD ) )
 	{
 	if  ( taNew.taFontIsBold )
 	    { boldString= "Bold";	}
 	else{ boldString= empty;	}
 	}
 
-    if  ( updMask & TAupdFONTSLANTED )
+    if  ( PROPmaskISSET( updMask, TApropFONTSLANTED ) )
 	{
 	if  ( taNew.taFontIsSlanted )
 	    { slantString= "Slanted";	}
 	else{ slantString= empty;	}
 	}
 
-    if  ( updMask & TAupdTEXTUNDERLINED )
+    if  ( PROPmaskISSET( updMask, TApropTEXTUNDERLINED ) )
 	{
 	if  ( taNew.taIsUnderlined )
 	    { underlineString= "Underlined";	}
@@ -1223,7 +1237,8 @@ int appFontGetAttributes(	char *			attributeString,
     if  ( *s == '*' )
 	{ s++;	}
     else{
-	familyName= s; updMask |= TAupdFONTFAMILY;
+	familyName= s;
+	PROPmaskADD( &updMask, TApropFONTFAMILY );
 
 	while( *s && *s != ',' )
 	    { s++;	}
@@ -1240,11 +1255,15 @@ int appFontGetAttributes(	char *			attributeString,
 	    { s++;	}
 	else{
 	    if  ( *s == ',' )
-		{ taNew.taFontIsBold= 0; updMask |= TAupdFONTBOLD;	}
+		{
+		taNew.taFontIsBold= 0;
+		PROPmaskADD( &updMask, TApropFONTBOLD );
+		}
 	    else{
 		if  ( ! strncmp( s, "Bold,", 5 ) )
 		    {
-		    taNew.taFontIsBold= 1; updMask |= TAupdFONTBOLD;
+		    taNew.taFontIsBold= 1;
+		    PROPmaskADD( &updMask, TApropFONTBOLD );
 		    s += 4;
 		    }
 		}
@@ -1262,11 +1281,15 @@ int appFontGetAttributes(	char *			attributeString,
 	    { s++;	}
 	else{
 	    if  ( *s == ',' )
-		{ taNew.taFontIsSlanted= 0;  updMask |= TAupdFONTSLANTED; }
+		{
+		taNew.taFontIsSlanted= 0; 
+		PROPmaskADD( &updMask, TApropFONTSLANTED );
+		}
 	    else{
 		if  ( ! strncmp( s, "Slanted,", 8 ) )
 		    {
-		    taNew.taFontIsSlanted= 1; updMask |= TAupdFONTSLANTED;
+		    taNew.taFontIsSlanted= 1;
+		    PROPmaskADD( &updMask, TApropFONTSLANTED );
 		    s += 7;
 		    }
 		}
@@ -1285,8 +1308,8 @@ int appFontGetAttributes(	char *			attributeString,
 	else{
 	    char *	past;
 
-	    taNew.taFontSizeHalfPoints= 2* strtod( s, &past )+ 0.4995 ;
-	    updMask |= TAupdFONTSIZE;
+	    taNew.taFontSizeHalfPoints= 2* strtod( s, &past )+ 0.249;
+	    PROPmaskADD( &updMask, TApropFONTSIZE );
 
 	    if  ( past == s )
 		{ SSDEB(attributeString,s); return -1;	}
@@ -1305,11 +1328,15 @@ int appFontGetAttributes(	char *			attributeString,
 	    { s++;	}
 	else{
 	    if  ( ! *s )
-		{ taNew.taIsUnderlined= 0; updMask |= TAupdTEXTUNDERLINED; }
+		{
+		taNew.taIsUnderlined= 0;
+		PROPmaskADD( &updMask, TApropTEXTUNDERLINED );
+		}
 	    else{
 		if  ( ! strncmp( s, "Underlined", 10 ) )
 		    {
-		    taNew.taIsUnderlined= 1; updMask |= TAupdTEXTUNDERLINED;
+		    taNew.taIsUnderlined= 1;
+		    PROPmaskADD( &updMask, TApropTEXTUNDERLINED );
 		    s += 10;
 		    }
 		}
