@@ -9,9 +9,13 @@
 #   include	<stdlib.h>
 #   include	<stdio.h>
 #   include	<stddef.h>
+#   include	<limits.h>
 
 #   include	<appDebugon.h>
 
+#   include	<appUnit.h>
+#   include	<utilPropMask.h>
+
 #   include	"appFrame.h"
 #   include	"appDraw.h"
 
@@ -38,6 +42,7 @@ typedef struct AppFontChooser
     APP_WIDGET		afcFamilyList;
     APP_WIDGET		afcFaceList;
     APP_WIDGET		afcSizeList;
+    APP_WIDGET		afcSizeText;
 
     APP_WIDGET		afcButtonRow;
     APP_WIDGET		afcRevertButton;
@@ -49,13 +54,13 @@ typedef struct AppFontChooser
     int			afcFaceChosen;
     int			afcBoldChosen;
     int			afcSlantedChosen;
-    int			afcSizeChosen;
+    int			afcSizeChosenHalfPoints;
 
     int			afcFamilySet;
     int			afcFaceSet;
     int			afcBoldSet;
     int			afcSlantedSet;
-    int			afcSizeSet;
+    int			afcSizeSetHalfPoints;
 
     void *		afcTarget;
 
@@ -71,23 +76,39 @@ typedef struct AppFontChooser
     } AppFontChooser;
 
 /************************************************************************/
+/*									*/
 /*  Value lists that can be used when no particular font is selected.	*/
+/*									*/
 /************************************************************************/
-static int EditFontToolDummySizes[]=
-			    { 8, 9, 10, 11, 12, 14, 16, 18, 24, 36, 48, 64 };
 
-#   define DEFSIZES \
-    EditFontToolDummySizes, sizeof(EditFontToolDummySizes)/sizeof(int)
+static int EditFontToolSizesHalfPoints[]=
+			    {
+			    2* 8,
+			    2* 9,
+			    2* 10,
+			    2* 11,
+			    2* 12,
+			    2* 14,
+			    2* 16,
+			    2* 18,
+			    2* 24,
+			    2* 36,
+			    2* 48,
+			    2* 64
+			    };
+
+static const int EditFontToolSizeCount=
+		sizeof(EditFontToolSizesHalfPoints)/sizeof(int);
 
 static AppFontTypeface EditFontToolDummyFace=
-	{ "Any", DEFSIZES, 0, 0, 0, 0 };
+	{ "Any", (int *)0, 0, 0, 0, 0, 0 };
 
 static AppFontTypeface EditFontToolDefaultFaces[]=
     {
-	{ "Medium", DEFSIZES, 0, 0, 1, 0 },
-	{ "Bold", DEFSIZES, 1, 0, 1, 0 },
-	{ "Italic", DEFSIZES, 0, 1, 1, 0 },
-	{ "Bold Italic", DEFSIZES, 1, 1, 1, 0 },
+	{ "Medium", (int *)0, 0, 0, 0, 1, 0 },
+	{ "Bold", (int *)0, 0, 1, 0, 1, 0 },
+	{ "Italic", (int *)0, 0, 0, 1, 1, 0 },
+	{ "Bold Italic", (int *)0, 0, 1, 1, 1, 0 },
     };
 
 static AppFontFamily EditFontToolDummyFamily=
@@ -98,6 +119,20 @@ static AppFontFamily EditFontToolDummyFamily=
 	0, 1,
     };
 
+/************************************************************************/
+/*									*/
+/*  Format the size of the font in the Text box below the list.		*/
+/*									*/
+/************************************************************************/
+
+static void appFontToolFormatSize(	char *		scratch,
+					int		sizeHalfPoints )
+    {
+    if  ( sizeHalfPoints % 2 )
+	{ sprintf( scratch, "%d.5", sizeHalfPoints/ 2 );	}
+    else{ sprintf( scratch, "%d", sizeHalfPoints/ 2 );		}
+    }
+
 /************************************************************************/
 /*									*/
 /*  Redraw the sample of the font chooser.				*/
@@ -151,31 +186,142 @@ static APP_EVENT_HANDLER( appFontRedraw, w, voidafc, exposeEvent )
     return;
     }
 
+/************************************************************************/
+/*									*/
+/*  Show the current font size.						*/
+/*									*/
+/************************************************************************/
+
+static void appFontShowSizeInList(
+				AppFontChooser *	afc,
+				int			fontSizeHalfPoints )
+    {
+    int		pos;
+    int		sizeChosen= -1;
+
+    for ( pos= 0; pos < EditFontToolSizeCount; pos++ )
+	{
+	if  ( fontSizeHalfPoints > 0				&&
+	      EditFontToolSizesHalfPoints[pos] ==
+					fontSizeHalfPoints	)
+	    { sizeChosen= pos; break;	}
+	}
+
+    if  ( sizeChosen >= 0 )
+	{ appGuiSelectPositionInListWidget( afc->afcSizeList, sizeChosen ); }
+    else{ appGuiRemoveSelectionFromListWidget( afc->afcSizeList );	    }
+
+    return;
+    }
+
+static void appFontShowSizeInText(
+				AppFontChooser *	afc,
+				int			fontSizeHalfPoints )
+    {
+    char	scratch[80];
+
+    if  ( fontSizeHalfPoints > 0 )
+	{
+	appFontToolFormatSize( scratch, fontSizeHalfPoints );
+	appStringToTextWidget( afc->afcSizeText, scratch );
+	}
+    else{
+	appStringToTextWidget( afc->afcSizeText, "" );
+	}
+
+    return;
+    }
+
 /************************************************************************/
 /*									*/
 /*  Get currently selected font.					*/
 /*									*/
 /************************************************************************/
 
-static void appFontGetCurrent(		AppFontFamily **	pEff,
-					AppFontTypeface **	pEft,
-					int *			pFontSize,
-					int			anyway,
-					AppFontChooser *	afc )
+static int appFontToolGetSize(	int *			pFontSizeHalfPoints,
+				int			anyway,
+				AppFontChooser *	afc )
+    {
+    int			sizeChosenHalfPoints= afc->afcSizeChosenHalfPoints;
+
+    char *		val;
+    char *		s;
+
+    const int		minValue= 1;
+    const int		adaptToMin= 0;
+    const int		maxValue= INT_MAX;
+    const int		adaptToMax= 0;
+
+    int			isEmpty;
+
+    s= val= appGetStringFromTextWidget( afc->afcSizeText );
+    while( isspace( *s ) )
+	{ s++;	}
+    isEmpty= ! *s;
+
+    appFreeStringFromTextWidget( val );
+
+    if  ( isEmpty )
+	{ sizeChosenHalfPoints= -1;	}
+    else{
+	int		changed= 0;
+	int		sizeTwips= sizeChosenHalfPoints;
+
+	if  ( sizeTwips > 0 )
+	    { sizeTwips *= 10;	}
+
+	if  ( appGetLengthFromTextWidget( afc->afcSizeText,
+				&sizeTwips, &changed, UNITtyPOINTS,
+				minValue, adaptToMin, maxValue, adaptToMax ) )
+	    {
+	    if  ( anyway )
+		{ sizeChosenHalfPoints= -1;	}
+	    else{ return -1;			}
+	    }
+	else{
+	    char	scratch[80];
+
+	    sizeChosenHalfPoints= ( sizeTwips+ 5 )/ 10;
+
+	    appFontToolFormatSize( scratch, sizeChosenHalfPoints );
+
+	    val= appGetStringFromTextWidget( afc->afcSizeText );
+
+	    if  ( strcmp( val, scratch ) )
+		{ appStringToTextWidget( afc->afcSizeText, scratch );	}
+
+	    appFreeStringFromTextWidget( val );
+	    }
+	}
+
+    appFontShowSizeInList( afc, sizeChosenHalfPoints );
+
+    *pFontSizeHalfPoints= sizeChosenHalfPoints;
+    return 0;
+    }
+
+static int appFontGetCurrent(	AppFontFamily **	pEff,
+				AppFontTypeface **	pEft,
+				int *			pFontSizeHalfPoints,
+				int			anyway,
+				AppFontChooser *	afc )
     {
     AppFontFamily *	aff= (AppFontFamily *)0;
     AppFontTypeface *	aft= (AppFontTypeface *)0;
 
     int			familyChosen= afc->afcFamilyChosen;
     int			faceChosen= afc->afcFaceChosen;
-    int			sizeChosen= afc->afcSizeChosen;
+    int			sizeChosenHalfPoints= afc->afcSizeChosenHalfPoints;
+
+    if  ( appFontToolGetSize( &sizeChosenHalfPoints, anyway, afc ) )
+	{ return -1;	}
 
     if  ( familyChosen < 0 && anyway )
 	{ familyChosen= 0; }
     if  ( faceChosen < 0 && anyway )
 	{ faceChosen= 0; }
-    if  ( sizeChosen < 0 && anyway )
-	{ sizeChosen= 12; }
+    if  ( sizeChosenHalfPoints < 0 && anyway )
+	{ sizeChosenHalfPoints= 24; }
 
     if  ( familyChosen >= 0 )
 	{
@@ -195,10 +341,10 @@ static void appFontGetCurrent(		AppFontFamily **	pEff,
 	{ *pEff= aff;	}
     if  ( pEft )
 	{ *pEft= aft;	}
-    if  ( pFontSize )
-	{ *pFontSize= sizeChosen;	}
+    if  ( pFontSizeHalfPoints )
+	{ *pFontSizeHalfPoints= sizeChosenHalfPoints;	}
 
-    return;
+    return 0;
     }
 
 /************************************************************************/
@@ -213,7 +359,7 @@ static void appFontShowCurrent(	AppFontChooser *	afc )
 
     AppFontFamily *	aff= (AppFontFamily *)0;
     AppFontTypeface *	aft= (AppFontTypeface *)0;
-    int			pointSize= -1;
+    int			fontSizeHalfPoints= -1;
 
     char		scratch[120+1];
 
@@ -223,14 +369,14 @@ static void appFontShowCurrent(	AppFontChooser *	afc )
 	afc->afcFont= (APP_FONT *)0;
 	}
 
-    appFontGetCurrent( &aff, &aft, &pointSize, 0, afc );
+    appFontGetCurrent( &aff, &aft, &fontSizeHalfPoints, 0, afc );
 
-    appFontFormatCurrent( afc->afcFamilyFormat, aff, aft, pointSize );
+    appFontFormatCurrent( afc->afcFamilyFormat, aff, aft, fontSizeHalfPoints );
 
-    appFontGetCurrent( &aff, &aft, &pointSize, 1, afc );
+    appFontGetCurrent( &aff, &aft, &fontSizeHalfPoints, 1, afc );
 
     if  ( appFontXFont( scratch, add,
-				aff, aft, 20* pointSize, DOCfontREGULAR ) )
+			aff, aft, 10* fontSizeHalfPoints, DOCfontREGULAR ) )
 	{ return;	}
 
     afc->afcFont= appDrawOpenFont( add, scratch );
@@ -243,9 +389,9 @@ static void appFontShowCurrent(	AppFontChooser *	afc )
     appExposeRectangle( add, 0, 0, 0, 0 );
 
     appGuiEnableWidget( afc->afcSetButton,
-				afc->afcFamilyChosen >= 0	||
-				afc->afcFaceChosen >= 0		||
-				afc->afcSizeChosen >= 0		);
+			    afc->afcFamilyChosen >= 0		||
+			    afc->afcFaceChosen >= 0		||
+			    afc->afcSizeChosenHalfPoints >= 0	);
 
     return;
     }
@@ -256,31 +402,27 @@ static void appFontShowCurrent(	AppFontChooser *	afc )
 /*									*/
 /************************************************************************/
 
-static void appFontFillSizes(		AppFontChooser *	afc,
-					AppFontTypeface *	aft,
-					int			fontSize )
+static void appFontFillSizes(	AppFontChooser *	afc,
+				AppFontTypeface *	aft,
+				int			fontSizeHalfPoints )
     {
     int		pos;
-    int		sizeChosen= -1;
 
     appGuiEmptyListWidget( afc->afcSizeList );
 
-    for ( pos= 0; pos < aft->aftSizeCount; pos++ )
+    for ( pos= 0; pos < EditFontToolSizeCount; pos++ )
 	{
-	char		scratch[80];
+	char	scratch[80];
 
-	sprintf( scratch, "%d", aft->aftSizes[pos] );
+	appFontToolFormatSize( scratch, EditFontToolSizesHalfPoints[pos] );
 
 	appGuiAddValueToListWidget( afc->afcSizeList, scratch );
-
-	if  ( fontSize > 0 && aft->aftSizes[pos] == fontSize )
-	    { sizeChosen= pos;	}
 	}
 
-    if  ( sizeChosen >= 0 )
-	{ appGuiSelectPositionInListWidget( afc->afcSizeList, sizeChosen ); }
+    appFontShowSizeInList( afc, fontSizeHalfPoints );
+    appFontShowSizeInText( afc, fontSizeHalfPoints );
 
-    afc->afcSizeChosen= fontSize;
+    afc->afcSizeChosenHalfPoints= fontSizeHalfPoints;
 
     return;
     }
@@ -292,12 +434,13 @@ static void appFontFillSizes(		AppFontChooser *	afc,
 /*									*/
 /************************************************************************/
 
-static void appFontReflectFamily(	AppFontChooser *	afc,
-					int			familyNumber,
-					AppFontFamily *		aff,
-					int			fontBold,
-					int			fontSlanted,
-					int			fontSize )
+static void appFontReflectFamily(
+				AppFontChooser *	afc,
+				int			familyNumber,
+				AppFontFamily *		aff,
+				int			fontBold,
+				int			fontSlanted,
+				int			fontSizeHalfPoints )
     {
     AppFontTypeface *		aft;
     int				pos;
@@ -338,27 +481,14 @@ static void appFontReflectFamily(	AppFontChooser *	afc,
 	    {
 	    afc->afcFaceChosen= faceChosen;
 
-	    appFontFillSizes( afc, aff->affFaces+ faceChosen, fontSize );
+	    appFontFillSizes( afc, aff->affFaces+ faceChosen,
+							fontSizeHalfPoints );
 	    }
 	else{
-	    int		sizeChosen= -1;
+	    appFontShowSizeInList( afc, fontSizeHalfPoints );
+	    appFontShowSizeInText( afc, fontSizeHalfPoints );
 
-	    aft= aff->affFaces+ faceChosen;
-	    for ( pos= 0; pos < aft->aftSizeCount; pos++ )
-		{
-		if  ( fontSize > 0 && aft->aftSizes[pos] == fontSize )
-		    { sizeChosen= pos;	}
-		}
-
-	    appGuiRemoveSelectionFromListWidget( afc->afcSizeList );
-
-	    if  ( sizeChosen >= 0 )
-		{
-		appGuiSelectPositionInListWidget(
-					afc->afcSizeList, sizeChosen );
-		}
-
-	    afc->afcSizeChosen= fontSize;
+	    afc->afcSizeChosenHalfPoints= fontSizeHalfPoints;
 	    }
 	}
     else{
@@ -366,7 +496,7 @@ static void appFontReflectFamily(	AppFontChooser *	afc,
 
 	appGuiRemoveSelectionFromListWidget( afc->afcFaceList );
 
-	appFontFillSizes( afc, &EditFontToolDummyFace, fontSize );
+	appFontFillSizes( afc, &EditFontToolDummyFace, fontSizeHalfPoints );
 	}
 
     return;
@@ -494,34 +624,52 @@ static APP_LIST_CALLBACK( appFontSizeChosen, w, voidafc, voidlcs )
     int				sizeChosen;
     int				position;
 
-    int *			sizeList= EditFontToolDummySizes;
-    int				sizeCount= sizeof(EditFontToolDummySizes)/
-								sizeof(int);
-    if  ( afc->afcFamilyChosen >= 0			&&
-	  afc->afcFamilyChosen <  afc->afcFamilyCount	)
-	{
-	AppFontFamily *	aff= afc->afcFamilies+ afc->afcFamilyChosen;
-
-	if  ( afc->afcFaceChosen >= 0			&&
-	      afc->afcFaceChosen <  aff->affFaceCount	)
-	    {
-	    AppFontTypeface *	aft= aff->affFaces+ afc->afcFaceChosen;
-
-	    sizeList= aft->aftSizes;
-	    sizeCount= aft->aftSizeCount;
-	    }
-	}
+    char			scratch[80];
 
     position= appGuiGetPositionFromListCallback( w, voidlcs );
 
     sizeChosen= position;
-    if  ( sizeChosen < 0 || sizeChosen >= sizeCount )
-	{ LLDEB(sizeChosen,sizeCount); afc->afcSizeChosen= -1; return; }
+    if  ( sizeChosen < 0 || sizeChosen >= EditFontToolSizeCount )
+	{
+	LLDEB(sizeChosen,EditFontToolSizeCount);
+	afc->afcSizeChosenHalfPoints= -1; return;
+	}
 
-    afc->afcSizeChosen= sizeList[sizeChosen];
+    afc->afcSizeChosenHalfPoints= EditFontToolSizesHalfPoints[sizeChosen];
 
     appFontShowCurrent( afc );
 
+    appFontToolFormatSize( scratch, afc->afcSizeChosenHalfPoints );
+    appStringToTextWidget( afc->afcSizeText, scratch );
+
+    return;
+    }
+
+/************************************************************************/
+/*									*/
+/*  The user typed 'Enter' in the text box with the font size.		*/
+/*									*/
+/************************************************************************/
+
+static void appFontToolSizeChanged(	APP_WIDGET	w,
+					void *		voidafc,
+					void *		voidcbs )
+    {
+    AppFontChooser *		afc= (AppFontChooser *)voidafc;
+
+    appFontShowCurrent( afc );
+
+    return;
+    }
+
+static void appFontToolSizeTyped(	APP_WIDGET	w,
+					void *		voidafc,
+					void *		voidcbs )
+    {
+    AppFontChooser *		afc= (AppFontChooser *)voidafc;
+
+    appGuiRemoveSelectionFromListWidget( afc->afcSizeList );
+
     return;
     }
 
@@ -623,7 +771,6 @@ static void appFontMakeChooseForm(	APP_WIDGET		parent,
 
     APP_WIDGET		familyForm;
     APP_WIDGET		faceForm;
-    APP_WIDGET		sizeForm;
 
     const int		heightResizable= 1;
 
@@ -643,9 +790,29 @@ static void appFontMakeChooseForm(	APP_WIDGET		parent,
 				&(afc->afcFaceList), 9, aftr->aftrFace,
 				appFontFaceChosen, (void *)afc );
 
-    appFontToolTitledList( chooseRow, 2, &sizeForm,
-				&(afc->afcSizeList), 12, aftr->aftrSize,
-				appFontSizeChosen, (void *)afc );
+    {
+    APP_WIDGET		column;
+    APP_WIDGET		label;
+
+    const int		textColumns= 5;
+    const int		textEnabled= 1;
+
+    appMakeColumnInRow( &column, chooseRow, 2 );
+
+    appMakeLabelInColumn( &label, column, aftr->aftrSize );
+
+    appGuiMakeListInColumn( &(afc->afcSizeList),
+				column, 10, appFontSizeChosen, (void *)afc );
+
+    appMakeTextInColumn( &(afc->afcSizeText), column,
+						textColumns, textEnabled );
+
+    appGuiSetGotValueCallbackForText( afc->afcSizeText,
+					appFontToolSizeChanged, (void *)afc );
+
+    appGuiSetTypingCallbackForText( afc->afcSizeText,
+					appFontToolSizeTyped, (void *)afc );
+    }
 
     return;
     }
@@ -684,14 +851,15 @@ static void appFontSetPushed(		APP_WIDGET	w,
 
     AppFontFamily *		aff;
     AppFontTypeface *		aft;
-    int				size;
+    int				sizeHalfPoints;
 
     char *			familyText= (char *)0;
 
     if  ( ! afc->afcSetFont )
 	{ XDEB(afc->afcSetFont); return;	}
 
-    appFontGetCurrent( &aff, &aft, &size, 0, afc );
+    if  ( appFontGetCurrent( &aff, &aft, &sizeHalfPoints, 0, afc ) )
+	{ return;	}
 
     if  ( aff )
 	{
@@ -703,11 +871,11 @@ static void appFontSetPushed(		APP_WIDGET	w,
     if  ( aft )
 	{
 	(*afc->afcSetFont)( afc->afcTarget, familyText, afc->afcFamilyChosen,
-				    aft->aftIsBold, aft->aftIsSlanted, size );
+			aft->aftIsBold, aft->aftIsSlanted, sizeHalfPoints );
 	}
     else{
 	(*afc->afcSetFont)( afc->afcTarget, familyText, afc->afcFamilyChosen,
-								-1, -1, size );
+						    -1, -1, sizeHalfPoints );
 	}
 
     return;
@@ -735,12 +903,12 @@ static void appFontRevertPushed(	APP_WIDGET	w,
 
 	appFontReflectFamily( afc, afc->afcFamilySet, aff,
 				afc->afcBoldSet, afc->afcSlantedSet,
-				afc->afcSizeSet );
+				afc->afcSizeSetHalfPoints );
 	}
 
     afc->afcFamilyChosen= afc->afcFamilySet;
     afc->afcFaceChosen= afc->afcFaceSet;
-    afc->afcSizeChosen= afc->afcSizeSet;
+    afc->afcSizeChosenHalfPoints= afc->afcSizeSetHalfPoints;
     afc->afcBoldChosen= afc->afcBoldSet;
     afc->afcSlantedChosen= afc->afcSlantedSet;
 
@@ -767,27 +935,31 @@ int appFontSetCurrentFont(	void *		voidafc,
 
     int			fontBold= -1;
     int			fontSlanted= -1;
-    int			fontSize= -1;
+    int			fontSizeHalfPoints= -1;
 
     /*  1  */
-    if  ( updMask & TAupdFONTBOLD )
+    if  ( PROPmaskISSET( updMask, TApropFONTBOLD ) )
 	{ fontBold= taNew.taFontIsBold;	}
-    if  ( updMask & TAupdFONTSLANTED )
+    if  ( PROPmaskISSET( updMask, TApropFONTSLANTED ) )
 	{ fontSlanted= taNew.taFontIsSlanted;	}
-    if  ( updMask & TAupdFONTSIZE )
-	{ fontSize= taNew.taFontSizeHalfPoints/ 2;	}
+    if  ( PROPmaskISSET( updMask, TApropFONTSIZE ) )
+	{ fontSizeHalfPoints= taNew.taFontSizeHalfPoints;	}
 
     afc->afcFamilySet= fontFamilyNumber;
     afc->afcBoldChosen= afc->afcBoldSet= fontBold;
     afc->afcSlantedChosen= afc->afcSlantedSet= fontSlanted;
-    afc->afcSizeSet= fontSize;
+    afc->afcSizeSetHalfPoints= fontSizeHalfPoints;
 
     if  ( fontFamilyNumber < 0 )
 	{
 	appGuiRemoveSelectionFromListWidget( afc->afcFamilyList );
 
-	appFontReflectFamily( afc, fontFamilyNumber,
-		&EditFontToolDummyFamily, fontBold, fontSlanted, fontSize );
+	appFontReflectFamily( afc,
+				fontFamilyNumber,
+				&EditFontToolDummyFamily,
+				fontBold,
+				fontSlanted,
+				fontSizeHalfPoints );
 
 	afc->afcFaceSet= afc->afcFaceChosen;
 	}
@@ -797,8 +969,12 @@ int appFontSetCurrentFont(	void *		voidafc,
 	appGuiSelectPositionInListWidget( afc->afcFamilyList,
 							fontFamilyNumber );
 
-	appFontReflectFamily( afc, fontFamilyNumber, aff,
-			    afc->afcBoldSet, afc->afcSlantedSet, fontSize );
+	appFontReflectFamily( afc,
+				fontFamilyNumber,
+				aff,
+				afc->afcBoldSet,
+				afc->afcSlantedSet,
+				fontSizeHalfPoints );
 
 	afc->afcFaceSet= afc->afcFaceChosen;
 	}
@@ -929,7 +1105,7 @@ void * appMakeFontsTool(	APP_WIDGET		fontsOption,
     afc->afcFaceSet= afc->afcFaceChosen= -1;
     afc->afcBoldSet= afc->afcBoldChosen= -1;
     afc->afcSlantedSet= afc->afcSlantedChosen= -1;
-    afc->afcSizeSet= afc->afcSizeChosen= -1;
+    afc->afcSizeSetHalfPoints= afc->afcSizeChosenHalfPoints= -1;
 
 #   ifdef USE_MOTIF
     XtRealizeWidget( afc->afcTopWidget );
