@@ -97,6 +97,7 @@ typedef struct AppMenuItem
 #   define	APPFILE_CAN_OPEN	0x01
 #   define	APPFILE_CAN_SAVE	0x02
 #   define	APPFILE_IS_BASIC_TYPE	0x04
+#   define	APPFILE_HIDE_OPEN	0x08
 
 typedef struct AppFileExtension
     {
@@ -1002,7 +1003,10 @@ extern int appGetLengthFromTextWidget(	APP_WIDGET	w,
 					int *		pValue,
 					int *		pChanged,
 					int		defaultUnit,
-					int		requirePositive );
+					int		minValue,
+					int		adaptToMin,
+					int		maxValue,
+					int		adaptToMax );
 
 extern int appGetIntegerFromTextWidget(	APP_WIDGET	w,
 					int *		pValue,
@@ -1181,9 +1185,7 @@ extern void appShowFindTool(		APP_WIDGET		reference,
 extern void appEnableFindTool(		void *	voidaft,
 					int	enabled );
 
-extern void appFindToolTextReadonly(	void *	voidaft,
-					int	readonly );
-extern void appFindToolFoundObsolete(	void *	voidaft );
+extern void appFindToolDisableReplace(	void *	voidaft );
 
 extern void appMakeColumnInRow(		APP_WIDGET *	pColumn,
 					APP_WIDGET	row,
