@@ -305,7 +305,11 @@ APP_WIDGET appMakeToggleInRow(	APP_WIDGET		row,
     XtSetArg( al[ac], XmNrightOffset,		0 ); ac++;
 
     XtSetArg( al[ac], XmNrecomputeSize,		True ); ac++;
+#   if 1
+    XtSetArg( al[ac], XmNmarginTop,		6 ); ac++;
+#   else
     XtSetArg( al[ac], XmNmarginHeight,		8 ); ac++;
+#   endif
 
     XtSetArg( al[ac], XmNalignment,		XmALIGNMENT_BEGINNING ); ac++;
 
@@ -379,7 +383,11 @@ void appMakeLabelInRow(		APP_WIDGET *		pLabel,
     XtSetArg( al[ac], XmNalignment,		XmALIGNMENT_BEGINNING ); ac++;
     XtSetArg( al[ac], XmNlabelString,		labelString ); ac++;
 
+#   if 1
+    XtSetArg( al[ac], XmNmarginTop,		6 ); ac++;
+#   else
     XtSetArg( al[ac], XmNmarginHeight,		8 ); ac++;
+#   endif
 
     label= XmCreateLabel( row, WIDGET_NAME, al, ac );
 
@@ -739,7 +747,8 @@ void appMakeVerticalTool(	APP_WIDGET *		pShell,
     ac=0;
     XtSetArg( al[ac], XmNdeleteResponse,	XmDO_NOTHING ); ac++;
     XtSetArg( al[ac], XmNinput,			True ); ac++;
-    XtSetArg( al[ac], XmNallowShellResize,	True );
+    XtSetArg( al[ac], XmNallowShellResize,	True ); ac++;
+    XtSetArg( al[ac], XmNuseAsyncGeometry,	True ); ac++;
     XtSetArg( al[ac], XmNmwmDecorations,	hints.decorations ); ac++;
     XtSetArg( al[ac], XmNmwmFunctions,		hints.functions ); ac++;
 
@@ -1356,6 +1365,7 @@ int appGuiInitApplication(	EditApplication *	ea,
     ac= 0;
     XtSetArg( al[ac], XmNinput,			True ); ac++;
     XtSetArg( al[ac], XmNallowShellResize,	True ); ac++;
+    XtSetArg( al[ac], XmNuseAsyncGeometry,	True ); ac++;
     XtSetArg( al[ac], XmNmwmDecorations,	hints.decorations ); ac++;
     XtSetArg( al[ac], XmNmwmFunctions,		hints.functions ); ac++;
 
