@@ -593,6 +593,8 @@ int appRunMailDialog(			EditApplication *	ea,
     MailContent *		mc;
     SimpleOutputStream *	sos;
 
+    const char *		mimeBoundary= "-----------MimeBoundary";
+
     /*  1  */
     if  ( ! amd )
 	{
@@ -711,12 +713,15 @@ int appRunMailDialog(			EditApplication *	ea,
 
     sos= sioOutSmtpOpen( amd->amdMailHost,
 			from, to, cc, bcc, mailSubject,
-			mc->mcType, mc->mcSubtype,
+			mc->mcTypeSlashSubtype,
+			mc->mcNeedsBoundary?mimeBoundary:(const char *)0,
 			(void *)amd, appMailComplain );
     if  ( sos )
 	{
-	if  ( (*mc->mcWriteMail)( sos, voiddoc ) )
+	if  ( (*mc->mcWriteMail)( sos,
+		mc->mcNeedsBoundary?mimeBoundary:(const char *)0, voiddoc ) )
 	    { LDEB(1); rval= -1;	}
+
 	if  ( sioOutClose( sos ) )
 	    { LDEB(1); rval= -1;	}
 	}
