@@ -476,49 +476,14 @@ static AppConfigurableResource	APP_ApplicationResourceTable[]=
 
 /************************************************************************/
 /*									*/
-/*  Generic main() procedure.						*/
-/*									*/
-/*  1)  Connect to the GUI system.					*/
-/*  2)  Retrieve resource values and derive some application wide	*/
-/*	settings from them.						*/
+/*  Initialize geometry related settings for the application.		*/
 /*									*/
 /************************************************************************/
 
-int appMain(	EditApplication *	ea,
-		int			argc,
-		char *			argv[]	)
+static void appInitializeGeometrySettings(	EditApplication *	ea )
     {
-    int				arg;
-
-    APP_BITMAP_IMAGE		pixmap= (APP_BITMAP_IMAGE)0;
-    APP_BITMAP_MASK		mask= (APP_BITMAP_MASK)0;
-
     int				paperFormat;
 
-    APP_WIDGET			pictureLabel= (APP_WIDGET)0;
-    APP_WIDGET			label;
-
-    setlocale( LC_ALL, "" );
-
-    /*  1  */
-    if  ( appGuiInitApplication( ea, &argc, &argv ) )
-	{ LDEB(1); return 1;	}
-
-    appInitDocumentGeometry( &(ea->eaDefaultDocumentGeometry) );
-
-    /*  b  */
-    appGuiGetResourceValues( ea, ea->eaResourceData,
-				ea->eaResourceTable, ea->eaResourceCount );
-
-    appGuiGetResourceValues( ea, &(ea->eaFileMessageResources),
-			ea->eaFileMessageResourceTable,
-			ea->eaFileMessageResourceCount );
-
-    appGuiGetResourceValues( ea, (void *)ea,
-				APP_ApplicationResourceTable,
-				sizeof(APP_ApplicationResourceTable)/
-				sizeof(AppConfigurableResource) );
-
     ea->eaUnitInt= appUnitTypeInt( ea->eaUnitString );
     if  ( ea->eaUnitInt < 0 )
 	{ SDEB(ea->eaUnitString); ea->eaUnitInt= UNITtyINCH; }
@@ -568,6 +533,32 @@ int appMain(	EditApplication *	ea,
 	    { SDEB(ea->eaBottomMarginString);	}
 	}
 
+    {
+    double			horPixPerMM;
+    double			verPixPerMM;
+    double			xfac;
+    double			yfac;
+
+    appGetFactors( ea, &horPixPerMM, &verPixPerMM, &xfac, &yfac );
+    }
+
+    return;
+    }
+
+/************************************************************************/
+/*									*/
+/*  Fill the application main widget.					*/
+/*									*/
+/************************************************************************/
+
+static int appFinishApplicationWindow(	EditApplication *	ea )
+    {
+    APP_BITMAP_IMAGE		pixmap= (APP_BITMAP_IMAGE)0;
+    APP_BITMAP_MASK		mask= (APP_BITMAP_MASK)0;
+
+    APP_WIDGET			pictureLabel= (APP_WIDGET)0;
+    APP_WIDGET			label;
+
     appGuiInsertColumnInWindow( &(ea->eaMainWindow),
 					    ea->eaToplevel.atTopWidget );
 
@@ -662,6 +653,48 @@ int appMain(	EditApplication *	ea,
 #	endif
 	}
 
+    return 0;
+    }
+
+/************************************************************************/
+/*									*/
+/*  Generic main() procedure.						*/
+/*									*/
+/*  1)  Connect to the GUI system.					*/
+/*  2)  Retrieve resource values and derive some application wide	*/
+/*	settings from them.						*/
+/*									*/
+/************************************************************************/
+
+int appMain(	EditApplication *	ea,
+		int			argc,
+		char *			argv[] )
+    {
+    int				arg;
+
+    setlocale( LC_ALL, "" );
+
+    /*  1  */
+    if  ( appGuiInitApplication( ea, &argc, &argv ) )
+	{ LDEB(1); return 1;	}
+
+    appInitDocumentGeometry( &(ea->eaDefaultDocumentGeometry) );
+
+    /*  b  */
+    appGuiGetResourceValues( ea, ea->eaResourceData,
+				ea->eaResourceTable, ea->eaResourceCount );
+
+    appGuiGetResourceValues( ea, &(ea->eaFileMessageResources),
+				ea->eaFileMessageResourceTable,
+				ea->eaFileMessageResourceCount );
+
+    appGuiGetResourceValues( ea, (void *)ea,
+				APP_ApplicationResourceTable,
+				sizeof(APP_ApplicationResourceTable)/
+				sizeof(AppConfigurableResource) );
+
+    appInitializeGeometrySettings( ea );
+
     if  ( ea->eaPrintDestinationCount == 0				&&
 	  utilPrinterGetPrinters( &(ea->eaPrintDestinationCount),
 				    &(ea->eaDefaultPrintDestination),
@@ -670,6 +703,9 @@ int appMain(	EditApplication *	ea,
 				    ea->eaCustomPrinterName )		)
 	{ LDEB(1); 	}
 
+    if  ( appFinishApplicationWindow( ea ) )
+	{ LDEB(1); return -1;	}
+
     appAllocateCopyPasteTargetAtoms( ea );
 
 #   ifdef USE_MOTIF
@@ -814,15 +850,6 @@ int appMain(	EditApplication *	ea,
 	{ appHideShellWidget( ea->eaToplevel.atTopWidget );	}
     else{ appShowShellWidget( ea->eaToplevel.atTopWidget );	}
 
-    {
-    double			horPixPerMM;
-    double			verPixPerMM;
-    double			xfac;
-    double			yfac;
-
-    appGetFactors( ea, &horPixPerMM, &verPixPerMM, &xfac, &yfac );
-    }
-
 #   ifdef USE_MOTIF
     XtAppMainLoop( ea->eaContext );
 #   endif
