@@ -77,6 +77,11 @@ int appMetaInitDeviceContext(	DeviceContext *		dc,
     dc->dcTwipsWide= twipsWide;
     dc->dcTwipsHigh= twipsHigh;
 
+    dc->dcClipX0= 0;
+    dc->dcClipY0= 0;
+    dc->dcClipX1= 0;
+    dc->dcClipY1= 0;
+
     dc->dcObjects= (MetaFileObject *)0;
     dc->dcObjectCount= 0;
 
@@ -434,6 +439,9 @@ int appMetaCreateFontIndirect(	DeviceContext *		dc,
     if  ( lf->lfHeight < 0 )
 	{ done= ( dc->dcTwipsHigh* -lf->lfHeight )/ dct->dctExtY; }
     else{ done= ( dc->dcTwipsHigh*  lf->lfHeight )/ dct->dctExtY; }
+
+    if  ( done < 0 )
+	{ done= -done;	}
     }
 
     lf->lfTextAttribute.taFontNumber= 0;
@@ -716,6 +724,70 @@ int appMetaRestoreDC(		DeviceContext *		dc,
     return 0;
     }
 
+/************************************************************************/
+/*									*/
+/*  Handle Clipping by ignoring it.					*/
+/*									*/
+/************************************************************************/
+
+int appMetaOffsetClipRgn(	DeviceContext *		dc,
+				int			recordSize,
+				SimpleInputStream *	sis )
+    {
+    int		x0;
+    int		y0;
+
+    y0= sioEndianGetLeInt16( sis );
+    x0= sioEndianGetLeInt16( sis );
+
+    WMFDEB(appDebug("OffsetClipRgn(yClpOrg+=%d,xClpOrg+=%d)\n", y0,x0 ));
+
+    dc->dcClipY0 += y0;
+    dc->dcClipX0 += x0;
+    dc->dcClipY1 += y0;
+    dc->dcClipX1 += x0;
+
+    return 0;
+    }
+
+int appMetaExcludeClipRect(	DeviceContext *		dc,
+				int			recordSize,
+				SimpleInputStream *	sis )
+    {
+    int			y0;
+    int			x0;
+    int			y1;
+    int			x1;
+
+    y1= sioEndianGetLeInt16( sis );
+    x1= sioEndianGetLeInt16( sis );
+    y0= sioEndianGetLeInt16( sis );
+    x0= sioEndianGetLeInt16( sis );
+
+    WMFDEB(appDebug("ExcludeClipRect(%d..%d,%d..%d)\n", x0,x1,y0,y1));
+
+    return 0;
+    }
+
+int appMetaIntersectClipRect(	DeviceContext *		dc,
+				int			recordSize,
+				SimpleInputStream *	sis )
+    {
+    int			y0;
+    int			x0;
+    int			y1;
+    int			x1;
+
+    y1= sioEndianGetLeInt16( sis );
+    x1= sioEndianGetLeInt16( sis );
+    y0= sioEndianGetLeInt16( sis );
+    x0= sioEndianGetLeInt16( sis );
+
+    WMFDEB(appDebug("IntersectClipRect(%d..%d,%d..%d)\n", x0,x1,y0,y1));
+
+    return 0;
+    }
+
 /************************************************************************/
 /*									*/
 /*  Remember text alighnment.						*/
@@ -792,7 +864,7 @@ int appMetaBitmapImage(		AppBitmapImage **	pAbi,
 	{ XDEB(abi); return -1;	}
     appInitBitmapImage( abi );
 
-    done= bmBmpReadDib( &abi->abiBitmap, &abi->abiBuffer, sis );
+    done= bmBmpReadDib( &(abi->abiBitmap), &(abi->abiBuffer), sis );
     if  ( done < 0 || done > expectBytes )
 	{
 	LLDEB(done,expectBytes);
