@@ -1175,6 +1175,46 @@ static int appMeta_PiePs(	SimpleInputStream *	sis,
     return 0;
     }
 
+static int appMeta_ChordPs(	SimpleInputStream *	sis,
+				FILE *			f,
+				int			recordSize,
+				DeviceContext *		dc )
+    {
+    int		y0;
+    int		x0;
+    int		y1;
+    int		x1;
+
+    int		ys;
+    int		xs;
+
+    double	as;
+    double	ae;
+
+    appMetaGetArcPs( sis, dc, "Chord", 
+			    &y0, &x0, &y1, &x1,
+			    &ys, &xs, &as, &ae );
+
+    fprintf( f, "gsave [1 0 0 %g 0 %d] concat\n",
+			    (double)(y0- y1)/(double)(x1- x0), ( y0+ y1 )/ 2 );
+
+    if  ( dc->dcDrawBorders )
+	{
+	appMetaSetColorPs( f, dc, &(dc->dcPen.lpColor) );
+	appMetaSetPenPs( f, dc, &(dc->dcPen) );
+
+	fprintf( f, "%d %d bp ", xs, ys );
+
+	fprintf( f, "%d 0 %d %f %f arc %d %d lineto stroke\n",
+					( x0+ x1 )/ 2, (x1- x0)/2,
+					as, ae, xs, ys );
+	}
+
+    fprintf( f, "grestore\n" );
+
+    return 0;
+    }
+
 static int appMeta_EllipsePs(	SimpleInputStream *	sis,
 				FILE *			f,
 				DeviceContext *		dc )
@@ -1439,18 +1479,26 @@ int appMetaPlayFilePs(	FILE *				f,
 		    { LDEB(1); return -1;	}
 		continue;
 
+	    case WINMETA_OffsetClipRgn:
+		if  ( appMetaOffsetClipRgn( &dc, recordSize, sis ) )
+		    { LDEB(1); return -1;	}
+		continue;
+
+	    case WINMETA_ExcludeClipRect:
+		if  ( appMetaExcludeClipRect( &dc, recordSize, sis ) )
+		    { LDEB(1); return -1;	}
+		continue;
+
+	    case WINMETA_IntersectClipRect:
+		if  ( appMetaIntersectClipRect( &dc, recordSize, sis ) )
+		    { LDEB(1); return -1;	}
+		continue;
+
 	    case WINMETA_SetMapperFlags:
 		rop= sioEndianGetLeInt32( sis );
 		WMFDEB(appDebug("WINMETA_SetMapperFlags(0x%lx)\n",rop));
 		continue;
-	    case WINMETA_IntersectClipRect:
-		h0= sioEndianGetLeInt16( sis );
-		w0= sioEndianGetLeInt16( sis );
-		y0= sioEndianGetLeInt16( sis );
-		x0= sioEndianGetLeInt16( sis );
-		WMFDEB(appDebug("IntersectClipRect([%d+%d,%d+%d])\n",
-							    x0,w0,y0,h0 ));
-		continue;
+
 	    case WINMETA_StretchBlt:
 		rop= sioEndianGetLeInt32( sis );
 		h0= sioEndianGetLeInt16( sis );
@@ -1665,6 +1713,11 @@ int appMetaPlayFilePs(	FILE *				f,
 		    { LDEB(1); return -1;	}
 		continue;
 
+	    case WINMETA_Chord:
+		if  ( appMeta_ChordPs( sis, f, recordSize, &dc ) )
+		    { LDEB(1); return -1;	}
+		continue;
+
 	    case WINMETA_Ellipse:
 		if  ( appMeta_EllipsePs( sis, f, &dc ) )
 		    { LDEB(1); return -1;	}
@@ -2199,72 +2252,50 @@ static const char *	APPMETAPS_Bp[]=
 "",
 };
 
-static void appMetaDefineProcedure(	FILE *			f,
-					const char **		lines,
-					int			count )
-    {
-    int		i;
-
-    for ( i= 0; i < count; lines++, i++ )
-	{
-	const char *	s= *lines;
-
-	while( isspace( *s ) )
-	    { s++;	}
-
-	if  ( ! *s || *s == '%' )
-	    { continue;	}
-
-	fprintf( f, "%s\n", *lines );
-	}
-
-    fprintf( f, "\n" );
-    }
-
 int appMetaDefineProcsetPs(	FILE *		f )
     {
     fprintf( f, "%%%%BeginProcSet AppWmfToPs 1.0 1\n\n" );
 
-    appMetaDefineProcedure( f, APPMETAPS_Bp,
+    appPsDefineProcedure( f, APPMETAPS_Bp,
 		    sizeof(APPMETAPS_Bp)/sizeof(const char *) );
 
-    appMetaDefineProcedure( f, APPMETAPS_Rl,
+    appPsDefineProcedure( f, APPMETAPS_Rl,
 		    sizeof(APPMETAPS_Rl)/sizeof(const char *) );
 
-    appMetaDefineProcedure( f, APPMETAPS_FillPrep,
+    appPsDefineProcedure( f, APPMETAPS_FillPrep,
 		    sizeof(APPMETAPS_FillPrep)/sizeof(const char *) );
 
-    appMetaDefineProcedure( f, APPMETAPS_FillPattern,
+    appPsDefineProcedure( f, APPMETAPS_FillPattern,
 		    sizeof(APPMETAPS_FillPattern)/sizeof(const char *) );
 
-    appMetaDefineProcedure( f, APPMETAPS_HatchHorizontal,
+    appPsDefineProcedure( f, APPMETAPS_HatchHorizontal,
 		    sizeof(APPMETAPS_HatchHorizontal)/sizeof(const char *) );
 
-    appMetaDefineProcedure( f, APPMETAPS_HatchVertical,
+    appPsDefineProcedure( f, APPMETAPS_HatchVertical,
 		    sizeof(APPMETAPS_HatchVertical)/sizeof(const char *) );
 
-    appMetaDefineProcedure( f, APPMETAPS_HatchFdiagonal,
+    appPsDefineProcedure( f, APPMETAPS_HatchFdiagonal,
 		    sizeof(APPMETAPS_HatchFdiagonal)/sizeof(const char *) );
 
-    appMetaDefineProcedure( f, APPMETAPS_HatchBdiagonal,
+    appPsDefineProcedure( f, APPMETAPS_HatchBdiagonal,
 		    sizeof(APPMETAPS_HatchBdiagonal)/sizeof(const char *) );
 
-    appMetaDefineProcedure( f, APPMETAPS_FillHorizontal,
+    appPsDefineProcedure( f, APPMETAPS_FillHorizontal,
 		    sizeof(APPMETAPS_FillHorizontal)/sizeof(const char *) );
 
-    appMetaDefineProcedure( f, APPMETAPS_FillVertical,
+    appPsDefineProcedure( f, APPMETAPS_FillVertical,
 		    sizeof(APPMETAPS_FillVertical)/sizeof(const char *) );
 
-    appMetaDefineProcedure( f, APPMETAPS_FillCross,
+    appPsDefineProcedure( f, APPMETAPS_FillCross,
 		    sizeof(APPMETAPS_FillCross)/sizeof(const char *) );
 
-    appMetaDefineProcedure( f, APPMETAPS_FillBdiagonal,
+    appPsDefineProcedure( f, APPMETAPS_FillBdiagonal,
 		    sizeof(APPMETAPS_FillBdiagonal)/sizeof(const char *) );
 
-    appMetaDefineProcedure( f, APPMETAPS_FillFdiagonal,
+    appPsDefineProcedure( f, APPMETAPS_FillFdiagonal,
 		    sizeof(APPMETAPS_FillFdiagonal)/sizeof(const char *) );
 
-    appMetaDefineProcedure( f, APPMETAPS_FillDiagcross,
+    appPsDefineProcedure( f, APPMETAPS_FillDiagcross,
 		    sizeof(APPMETAPS_FillDiagcross)/sizeof(const char *) );
 
     fprintf( f, "%%%%EndProcSet\n\n" );
