@@ -618,7 +618,19 @@ static int appMetaBitmapImageX11(	AppBitmapImage **	pAbi,
     APP_IMAGE *		xim= (APP_IMAGE *)0;
 
     if  ( srcX != 0 || srcY != 0 )
-	{ LLDEB(srcX,srcY); return 0;	}
+	{ LLDEB(srcX,srcY); 	}
+
+    if  ( expectBytes < 40 )
+	{
+	int	done= 0;
+
+	LDEB(expectBytes);
+
+	while( done < expectBytes )
+	    { sioInGetCharacter( sis ); done++;	}
+
+	return 0;
+	}
 
     if  ( appMetaBitmapImage( &abi, sis, expectBytes ) )
 	{ LDEB(expectBytes); return -1;	}
@@ -1011,6 +1023,33 @@ static int appMeta_ArcX11(	SimpleInputStream *	sis,
     return 0;
     }
 
+static int appMeta_ChordX11(	SimpleInputStream *	sis,
+				int			recordSize,
+				DeviceContextX11 *	dcx )
+    {
+    DeviceContext *	dc= &(dcx->dcxDeviceContext);
+    AppDrawingData *	add= &(dc->dcDrawingData);
+
+    int			y0;
+    int			x0;
+    int			y1;
+    int			x1;
+
+    int			a1;
+    int			a2;
+
+    appMetaGetArcX11( sis, dcx, "Chord", &y0, &x0, &y1, &x1, &a1, &a2 );
+
+    if  ( dc->dcDrawBorders )
+	{
+	appDrawSetForegroundColor( add, &(dcx->dcxPenColor) );
+
+	appDrawDrawArc( add, x0, y0, x1- x0, y1- y0, a1, a2 );
+	}
+
+    return 0;
+    }
+
 static int appMeta_PieX11(	SimpleInputStream *	sis,
 				int			recordSize,
 				DeviceContextX11 *	dcx )
@@ -1100,8 +1139,7 @@ static void appMeta_SetFillX11(		DeviceContextX11 *	dcx,
     AppDrawingData *	add= &(dc->dcDrawingData);
 
     if  ( before )
-	{
-	appDrawSetForegroundColor( add, &(dcx->dcxBrushColor) );	}
+	{ appDrawSetForegroundColor( add, &(dcx->dcxBrushColor) );	}
 
     if  ( dcx->dcxFillTiled && before )
 	{
@@ -1468,6 +1506,51 @@ static int appMeta_PatBltX11(	SimpleInputStream *	sis,
     return 0;
     }
 
+static int appMeta_SetPolyFillModeX11(	SimpleInputStream *	sis,
+					int			recordSize,
+					DeviceContextX11 *	dcx )
+    {
+    DeviceContext *	dc= &(dcx->dcxDeviceContext);
+    AppDrawingData *	add= &(dc->dcDrawingData);
+    int			mode;
+
+    if  ( recordSize != 4 )
+	{ LDEB(recordSize); return -1;	}
+
+    mode= sioEndianGetLeInt16( sis );
+
+    switch( mode )
+	{
+	case ALTERNATE:
+	    WMFDEB(appDebug("SetPolyFillMode( ALTERNATE= %d )\n", mode ));
+
+#	    ifdef USE_MOTIF
+	    XSetFillRule( add->addDisplay, add->addGc, EvenOddRule );
+#	    endif
+#	    ifdef USE_GTK
+	    SLDEB("GDK",mode);
+#	    endif
+	    break;
+
+	case WINDING:
+	    WMFDEB(appDebug("SetPolyFillMode( WINDING= %d )\n", mode ));
+
+#	    ifdef USE_MOTIF
+	    XSetFillRule( add->addDisplay, add->addGc, WindingRule );
+#	    endif
+#	    ifdef USE_GTK
+	    SLDEB("GDK",mode);
+#	    endif
+	    break;
+
+	default:
+	    WMFDEB(appDebug("SetPolyFillMode( %d ) ??\n", mode ));
+	    break;
+	}
+
+    return 0;
+    }
+
 static int appMeta_PolyPolygonX11(	SimpleInputStream *	sis,
 					int			recordSize,
 					DeviceContextX11 *	dcx )
@@ -1491,19 +1574,21 @@ static int appMeta_PolyPolygonX11(	SimpleInputStream *	sis,
 	if  ( appMetaGetPointsX11( dcx, counts[i], sis ) )
 	    { LDEB(counts[i]); return -1;	}
 
-	if  ( dc->dcDrawBorders )
-	    {
-	    appDrawSetForegroundColor( add, &(dcx->dcxPenColor) );
-
-	    appDrawDrawLines( add, dc->dcPoints, counts[i] );
-	    }
-
+#	if 0
 	if  ( dc->dcFillInsides )
 	    {
 	    appDrawSetForegroundColor( add, &(dcx->dcxBrushColor) );
 
 	    appDrawFillPolygon( add, dc->dcPoints, counts[i] );
 	    }
+#	endif
+
+	if  ( dc->dcDrawBorders )
+	    {
+	    appDrawSetForegroundColor( add, &(dcx->dcxPenColor) );
+
+	    appDrawDrawLines( add, dc->dcPoints, counts[i] );
+	    }
 	}
 
     return 0;
@@ -1589,24 +1674,63 @@ static int appWinMetaPlayRecords(	SimpleInputStream *	sis,
 		    { LDEB(1); return -1;	}
 		continue;
 
+	    case WINMETA_OffsetClipRgn:
+		if  ( appMetaOffsetClipRgn( dc, recordSize, sis ) )
+		    { LDEB(1); return -1;	}
+		continue;
+
+	    case WINMETA_ExcludeClipRect:
+		if  ( appMetaExcludeClipRect( dc, recordSize, sis ) )
+		    { LDEB(1); return -1;	}
+		continue;
+
+	    case WINMETA_IntersectClipRect:
+		if  ( appMetaIntersectClipRect( dc, recordSize, sis ) )
+		    { LDEB(1); return -1;	}
+		continue;
+
 	    case WINMETA_SetMapperFlags:
 		rop= sioEndianGetLeInt32( sis );
 		WMFDEB(appDebug("WINMETA_SetMapperFlags(0x%lx)\n",rop));
 		continue;
-	    case WINMETA_IntersectClipRect:
-		h0= sioEndianGetLeInt16( sis );
-		w0= sioEndianGetLeInt16( sis );
+
+	    case WINMETA_BitBlt:
+		rop= sioEndianGetLeInt32( sis );
+		/*  source	*/
 		y0= sioEndianGetLeInt16( sis );
 		x0= sioEndianGetLeInt16( sis );
-		WMFDEB(appDebug("IntersectClipRect([%d+%d,%d+%d])\n",
-							    x0,w0,y0,h0 ));
+		/*  destination	*/
+		h1= sioEndianGetLeInt16( sis );
+		w1= sioEndianGetLeInt16( sis );
+		y1= sioEndianGetLeInt16( sis );
+		x1= sioEndianGetLeInt16( sis );
+
+		h0= h1;
+		w0= w1;
+
+		WMFDEB(appDebug("BitBlt([%d+%d,%d+%d]->[%d+%d,%d+%d],..)\n",
+				x0,w0,y0,h0, x1,w1,y1,h1 ));
+
+		x1= DCX_X( x1, dcx );
+		y1= DCX_Y( y1, dcx );
+
+		w1= DCX_W( w1, dcx );
+		h1= DCX_H( h1, dcx );
+
+		if  ( appMetaBitmapImageX11( &abi, sis, 2*(recordSize-3-2-6*1),
+			    dcx, ac, x0, y0, w0, h0, x1, y1, w1, h1 ) )
+		    { LDEB(1); return -1; }
+
 		continue;
+
 	    case WINMETA_StretchBlt:
 		rop= sioEndianGetLeInt32( sis );
+		/*  source	*/
 		h0= sioEndianGetLeInt16( sis );
 		w0= sioEndianGetLeInt16( sis );
 		y0= sioEndianGetLeInt16( sis );
 		x0= sioEndianGetLeInt16( sis );
+		/*  destination	*/
 		h1= sioEndianGetLeInt16( sis );
 		w1= sioEndianGetLeInt16( sis );
 		y1= sioEndianGetLeInt16( sis );
@@ -1626,6 +1750,7 @@ static int appWinMetaPlayRecords(	SimpleInputStream *	sis,
 		    { LDEB(1); return -1; }
 
 		continue;
+
 	    case WINMETA_StretchDIBits:
 		rop= sioEndianGetLeInt32( sis );
 		(void) sioEndianGetLeInt16( sis );
@@ -1699,6 +1824,11 @@ static int appWinMetaPlayRecords(	SimpleInputStream *	sis,
 
 		continue;
 
+	    case WINMETA_SetPolyFillMode:
+		if  ( appMeta_SetPolyFillModeX11( sis, recordSize, dcx ) )
+		    { LDEB(recordSize); return -1;	}
+		continue;
+
 	    case WINMETA_SetBkColor:
 		appMetaGetColor( sis, &(dcx->dcxDeviceContext.dcBkColor) );
 
@@ -1780,7 +1910,7 @@ static int appWinMetaPlayRecords(	SimpleInputStream *	sis,
 			LDEB(recordSize); return -1;
 		    }
 		WMFDEB(appDebug("SetStretchBltMode(%ld)\n",
-					dcx->dcxDeviceContext.dcStretchBltMode));
+				    dcx->dcxDeviceContext.dcStretchBltMode));
 		continue;
 
 	    case WINMETA_PolyLine:
@@ -1833,10 +1963,6 @@ static int appWinMetaPlayRecords(	SimpleInputStream *	sis,
 		    { LDEB(recordSize); return -1;	}
 		continue;
 
-	    case WINMETA_SetPolyFillMode:
-		WMFDEB(appDebug("SetPolyFillMode(...)\n"));
-		goto skipArguments;
-
 	    case WINMETA_RoundRect:
 		if  ( appMeta_RoundRectX11( sis, recordSize, dcx ) )
 		    { LDEB(1); return -1;	}
@@ -1852,6 +1978,11 @@ static int appWinMetaPlayRecords(	SimpleInputStream *	sis,
 		    { LDEB(1); return -1;	}
 		continue;
 
+	    case WINMETA_Chord:
+		if  ( appMeta_ChordX11( sis, recordSize, dcx ) )
+		    { LDEB(1); return -1;	}
+		continue;
+
 	    case WINMETA_Ellipse:
 		if  ( appMeta_EllipseX11( sis, recordSize, dcx ) )
 		    { LDEB(1); return -1;	}
