@@ -69,7 +69,7 @@ void appPsBoundingBoxComment(  const PrintingState *	ps,
 /*									*/
 /*  Initialisation for 'nup' printing.					*/
 /*									*/
-/*  1)  Hack: When the page in higher than the sheet, shift by the page	*/
+/*  1)  Hack: When the page is higher than the sheet, shift by the page	*/
 /*	size. This gives better results in case of miscongiguration.	*/
 /*									*/
 /************************************************************************/
@@ -742,41 +742,181 @@ void appPsWriteEpsHeader(	FILE *			f,
     return;
     }
 
+/************************************************************************/
+/*									*/
+/*  Define a procedure from an array of lines of PostScript code.	*/
+/*  Empty lines and lines completely consisting of a comment are	*/
+/*  skipped.								*/
+/*									*/
+/************************************************************************/
+
+void appPsDefineProcedure(	FILE *			f,
+				const char **		lines,
+				int			count )
+    {
+    int		i;
+
+    for ( i= 0; i < count; lines++, i++ )
+	{
+	const char *	s= *lines;
+
+	while( isspace( *s ) )
+	    { s++;	}
+
+	if  ( ! *s || *s == '%' )
+	    { continue;	}
+
+	fprintf( f, "%s\n", *lines );
+	}
+
+    fprintf( f, "\n" );
+    }
+
 /************************************************************************/
 /*									*/
 /*  Write the header for an EPS file.					*/
 /*									*/
 /************************************************************************/
 
+static const char * appPsRectfillEmulation[]=
+    {
+    "%%rectfill emulation for one rectangle only",
+    "/rectfill where",
+    "    { pop }",
+    "    { /rectfill",
+    "        { 4 2 roll moveto 2 copy",
+    "            0 exch rlineto 0 rlineto ",
+    "            neg 0 exch rlineto pop closepath fill",
+    "        } bind def",
+    "    }",
+    "ifelse",
+    };
+
 void appPsSetRectfillEmulation(	FILE *	f )
     {
-    fprintf( f, "\n%%rectfill emulation for one rectangle only\n" );
-    fprintf( f, "/rectfill where\n" );
-    fprintf( f, "    { pop }\n" );
-    fprintf( f, "    { /rectfill \n" );
-    fprintf( f, "        { 4 2 roll moveto 2 copy\n" );
-    fprintf( f, "            0 exch rlineto 0 rlineto \n" );
-    fprintf( f, "            neg 0 exch rlineto pop closepath fill\n" );
-    fprintf( f, "        } bind def\n" );
-    fprintf( f, "    }\n" );
-    fprintf( f, "ifelse\n" );
+    appPsDefineProcedure( f, appPsRectfillEmulation,
+		    sizeof(appPsRectfillEmulation)/sizeof(const char *) );
 
     return;
     }
 
+static const char * appPsSelectfontEmulation[]=
+    {
+    "%%selectfont emulation",
+    "/selectfont where",
+    "    { pop }",
+    "    { /selectfont",
+    "        { exch findfont exch dup type /arraytype eq",
+    "          { makefont } { scalefont } ifelse",
+    "          setfont",
+    "        } bind def",
+    "    }",
+    "ifelse",
+    };
+
 void appPsSetSelectfontEmulation(	FILE *	f )
     {
-    fprintf( f, "\n%%selectfont emulation\n" );
-    fprintf( f, "/selectfont where\n" );
-    fprintf( f, "    { pop }\n" );
-    fprintf( f, "    { /selectfont\n" );
-    fprintf( f, "        { exch findfont exch dup type /arraytype eq\n" );
-    fprintf( f, "          { makefont } { scalefont } ifelse\n" );
-    fprintf( f, "          setfont\n" );
-    fprintf( f, "        } bind def\n" );
-    fprintf( f, "    }\n" );
-    fprintf( f, "ifelse\n" );
+    appPsDefineProcedure( f, appPsSelectfontEmulation,
+		    sizeof(appPsSelectfontEmulation)/sizeof(const char *) );
 
     return;
     }
 
+/************************************************************************/
+/*									*/
+/*  Define procedures to Begin/End the inclusion of an EPS graphic.	*/
+/*									*/
+/*  *)  Copied directly from the EPS spec.				*/
+/*  1)  I spent some time wondering why the 'begin' operator was not	*/
+/*	matched by an 'end' operator. The matching end is in the loop	*/
+/*	as 'op_count' is remembered before the copy of 'userdict' is	*/
+/*	pushed.								*/
+/*									*/
+/************************************************************************/
+
+static const char * appBeginEPSF[]=
+    {
+    "/BeginEPSF",
+    "    {",
+    "    /b4_Inc_state save def          % Save state for cleanup",
+    "    /dict_count countdictstack def  % Count objects on dict stack",
+    "    /op_count count 1 sub def       % Count objects on operand stack",
+	 /*  1  */
+    "    userdict begin                  % Push userdict on dict stack",
+    "    /showpage { } def               % Redefine showpage, { } = null proc",
+    "    0 setgray 0 setlinecap          % Prepare graphics state",
+    "    1 setlinewidth 0 setlinejoin",
+    "    10 setmiterlimit [ ] 0 setdash newpath",
+    "    /languagelevel where            % If level not equal to 1 then",
+    "    { pop languagelevel             % set strokeadjust and",
+    "        1 ne                        % overprint to their defaults.",
+    "            { false setstrokeadjust false setoverprint",
+    "            } if",
+    "        } if",
+    "    } bind def",
+    };
+
+static const char * appEndEPSF[]=
+    {
+    "/EndEPSF",
+    "    {",
+    "    count op_count sub {pop} repeat % Clean up stacks",
+	 /*  1  */
+    "    countdictstack dict_count sub {end} repeat",
+    "    b4_Inc_state restore",
+    "    } bind def",
+    };
+
+void appPsDefineEpsProcs(	FILE *		f )
+    {
+    appPsDefineProcedure( f, appBeginEPSF,
+				sizeof(appBeginEPSF)/sizeof(const char *) );
+
+    appPsDefineProcedure( f, appEndEPSF,
+				sizeof(appEndEPSF)/sizeof(const char *) );
+
+    return;
+    }
+
+void appPsBeginEpsObject(	PrintingState *		ps,
+				int			x0Twips,
+				int			y0Twips,
+				int			llxTwips,
+				int			llyTwips,
+				int			urxTwips,
+				int			uryTwips,
+				const unsigned char *	file )
+    {
+    if  ( ! file )
+	{ file= (const unsigned char *)"??";	}
+
+    fprintf( ps->psFile, "BeginEPSF\n" );
+
+    fprintf( ps->psFile, "[ %d %d %d %d %d %d ] concat\n",
+				    20, 0, 0, -20, x0Twips, y0Twips );
+
+    fprintf( ps->psFile, "newpath %d %d moveto ",
+					    llxTwips/20, llyTwips/20 );
+    fprintf( ps->psFile, "%d %d lineto ",
+					    urxTwips/20, llyTwips/20 );
+    fprintf( ps->psFile, "%d %d lineto ",
+					    urxTwips/20, uryTwips/20 );
+    fprintf( ps->psFile, "%d %d lineto ",
+					    llxTwips/20, uryTwips/20 );
+    fprintf( ps->psFile, "closepath clip\n" );
+
+    fprintf( ps->psFile, "%%%%BeginDocument: (" );
+    appPsPrintString( ps->psFile, file, strlen( (char *)file ) );
+    fprintf( ps->psFile, ")\n" );
+
+    return;
+    }
+
+
+void appPsEndEpsObject(	PrintingState *		ps )
+    {
+    fprintf( ps->psFile, "%%%%EndDocument\n" );
+    fprintf( ps->psFile, "EndEPSF\n" );
+
+    return;
+    }
