@@ -29,7 +29,9 @@
 #   include	<appDebugon.h>
 
 /************************************************************************/
+/*									*/
 /*  Forward declarations						*/
+/*									*/
 /************************************************************************/
 
 static int sioSmtpSayRecipients(	int			fd,
@@ -43,12 +45,6 @@ static int mmWriteHeader(	int			fd,
 				void *			through,
 				APP_COMPLAIN		complain );
 
-static int mmWriteContentType(	int			fd,
-				const char *		type,
-				const char *		subtype,
-				void *			through,
-				APP_COMPLAIN		complain );
-
 static int mmWrite(		int			fd,
 				const char *		buf,
 				unsigned int		len,
@@ -77,7 +73,7 @@ static int mmReadCRLF(	int			fd,
 typedef struct MailContext
     {
     int			mcFd;
-    int			mcLastWasCRLF;
+    int			mcAtBeginOfLine;
     void *		mcThrough;
     APP_COMPLAIN	mcComplain;
     } MailContext;
@@ -85,20 +81,48 @@ typedef struct MailContext
 static char MMscratch[1024];
 
 /************************************************************************/
-/*  Open a connection to an SMTP server and send the mail headers	*/
+/*									*/
+/*  Abort an Smtp connection.						*/
+/*									*/
 /************************************************************************/
 
-SimpleOutputStream * sioOutSmtpOpen(	const char *		relay,
-					const char *		from,
-					const char *		to,
-					const char *		cc,
-					const char *		bcc,
-					const char *		subject,
-					const char *		type,
-					const char *		subtype,
+static void sioOutSmtpAbortMessage(	MailContext *		mc,
 					void *			through,
 					APP_COMPLAIN		complain )
     {
+    if  ( mmWriteCRLF( mc->mcFd, "RSET", through, complain ) )
+	{ LDEB(1); return;	}
+
+    if  ( mmReadCRLF( mc->mcFd, MMscratch, through, complain ) )
+	{ LDEB(1); return;	}
+
+    if  ( mmWriteCRLF( mc->mcFd, "QUIT", through, complain ) )
+	{ LDEB(1); return;	}
+
+    if  ( mmReadCRLF( mc->mcFd, MMscratch, through, complain ) )
+	{ LDEB(1); return;	}
+
+    return;
+    }
+
+
+/************************************************************************/
+/*									*/
+/*  Open a connection to an SMTP server and send the mail headers	*/
+/*									*/
+/************************************************************************/
+
+SimpleOutputStream * sioOutSmtpOpen(	const char *	relay,
+					const char *	from,
+					const char *	to,
+					const char *	cc,
+					const char *	bcc,
+					const char *	subject,
+					const char *	typeSlashSubtype,
+					const char *	mimeBoundary,
+					void *		through,
+					APP_COMPLAIN	complain )
+    {
     MailContext *		mc= (MailContext *)0;
     const char *		past;
     const char *		start;
@@ -136,7 +160,7 @@ SimpleOutputStream * sioOutSmtpOpen(	const char *		relay,
 	}
 
     mc->mcFd= -1;
-    mc->mcLastWasCRLF= 1;
+    mc->mcAtBeginOfLine= 1;
     mc->mcThrough= through;
     mc->mcComplain= complain;
 
@@ -261,10 +285,38 @@ SimpleOutputStream * sioOutSmtpOpen(	const char *		relay,
 	  mmWriteHeader( mc->mcFd, "To", to, through, complain )	||
 	  mmWriteHeader( mc->mcFd, "Cc", cc, through, complain )	||
 	  mmWriteHeader( mc->mcFd, "Subject", subject,
-					through, complain )		||
-	  mmWriteContentType( mc->mcFd, type, subtype,
-					through, complain )		||
-	  mmWriteCRLF( mc->mcFd, "\n", through, complain )		)
+					    through, complain )		)
+	{ LDEB(1); goto mmAbort;	}
+	
+    if  ( mimeBoundary )
+	{
+	const char *	name= "Content-Type";
+	int		len= strlen( name );
+	const char *	name2= " boundary=\"";
+
+	if  ( mmWrite( mc->mcFd, name, len, through, complain )		||
+	      mmWrite( mc->mcFd, ": ", 2, through, complain )		||
+	      mmWrite( mc->mcFd, typeSlashSubtype,
+				strlen( typeSlashSubtype ),
+				through, complain )			||
+	      mmWriteCRLF( mc->mcFd, ";", through, complain )		)
+	    { LDEB(1); goto mmAbort;	}
+
+	len= strlen( name2 );
+	if  ( mmWrite( mc->mcFd, name2, len, through, complain )	||
+	      mmWrite( mc->mcFd, mimeBoundary,
+				strlen( mimeBoundary ),
+				through, complain )			||
+	      mmWriteCRLF( mc->mcFd, "\"",  through, complain )		)
+	    { LDEB(1); goto mmAbort;	}
+	}
+    else{
+	if  ( mmWriteHeader( mc->mcFd, "Content-Type", typeSlashSubtype,
+						    through, complain )	)
+	    { LDEB(1); goto mmAbort;	}
+	}
+
+    if  ( mmWriteCRLF( mc->mcFd, "\n", through, complain )		)
 	{ LDEB(1); goto mmAbort;	}
 
     sos= sioOutOpen( (void *)mc, sioOutSmtpWriteBytes,
@@ -275,20 +327,10 @@ SimpleOutputStream * sioOutSmtpOpen(	const char *		relay,
 
     return sos;
 
-mmAbort:
-    if  ( mmWriteCRLF( mc->mcFd, "RSET", through, complain ) )
-	{ LDEB(1); goto mmError;				}
+  mmAbort:
+    sioOutSmtpAbortMessage( mc, through, complain );
 
-    if  ( mmReadCRLF( mc->mcFd, MMscratch, through, complain ) )
-	{ LDEB(1); goto mmError;				}
-
-    if  ( mmWriteCRLF( mc->mcFd, "QUIT", through, complain ) )
-	{ LDEB(1); goto mmError;				}
-
-    if  ( mmReadCRLF( mc->mcFd, MMscratch, through, complain ) )
-	{ LDEB(1); goto mmError;				}
-
-mmError:
+  mmError:
     if  ( mc )
 	{
 	if  ( mc->mcFd != -1 )
@@ -300,15 +342,21 @@ mmError:
     }
 
 /************************************************************************/
+/*									*/
 /*  Refuse to seek.							*/
+/*									*/
 /************************************************************************/
+
 static int sioOutSmtpSeek(	void *	voidmc,
 				long	pos )
     { LDEB(pos); return -1;	}
 
 /************************************************************************/
+/*									*/
 /*  Send the next part of the mail body					*/
+/*									*/
 /************************************************************************/
+
 static int sioOutSmtpWriteBytes(	void *			voidmc,
 					const unsigned char *	buf,
 					int			len )
@@ -339,10 +387,10 @@ static int sioOutSmtpWriteBytes(	void *			voidmc,
 					mc->mcThrough, mc->mcComplain ) )
 		{ LDEB(1); return -1;				}
 	    todo--; past++;
-	    mc->mcLastWasCRLF= 1;
+	    mc->mcAtBeginOfLine= 1;
 	    }
 	else{
-	    mc->mcLastWasCRLF= 0;
+	    mc->mcAtBeginOfLine= 0;
 	    }
 
 	start= past;
@@ -352,14 +400,17 @@ static int sioOutSmtpWriteBytes(	void *			voidmc,
     }
 
 /************************************************************************/
+/*									*/
 /*  Finish sending the mail body and close the connection		*/
+/*									*/
 /************************************************************************/
+
 static int sioOutSmtpClose(	void *		voidmc )
     {
     int			rval= 0;
     MailContext *	mc= (MailContext *)voidmc;
 
-    if  ( ! mc->mcLastWasCRLF						&&
+    if  ( ! mc->mcAtBeginOfLine						&&
 	  mmWriteCRLF( mc->mcFd, "\n", mc->mcThrough, mc->mcComplain )	)
 	{ LDEB(1); rval= -1;	}
 
@@ -445,8 +496,11 @@ static int sioSmtpSayRecipients(	int			fd,
     }	
 
 /************************************************************************/
+/*									*/
 /*  SMTP connection read and write utilities				*/
+/*									*/
 /************************************************************************/
+
 static int mmReadCRLF(	int			fd,
 			char *			buf,
 			void *			through,
@@ -455,7 +509,7 @@ static int mmReadCRLF(	int			fd,
     static char smallbuf[2];
     int		nbytes= 0;
 
-    for ( ;; )
+    for (;;)
 	{
 	int	done= read( fd, smallbuf, 1 );
 
@@ -527,27 +581,6 @@ static int mmWriteHeader(	int			fd,
     return 0;
     }
 
-static int mmWriteContentType(	int			fd,
-				const char *		type,
-				const char *		subtype,
-				void *			through,
-				APP_COMPLAIN		complain )
-    {
-    unsigned int	len;
-
-    if  ( ! type || ! type[0] || ! subtype || ! subtype[0] )
-	{ return 0;						}
-
-    len= strlen( type );
-    if  ( mmWrite( fd, "Content-Type: ", 14, through, complain )	||
-	  mmWrite( fd, type, len, through, complain )			||
-	  mmWrite( fd, "/", 1, through, complain )			||
-	  mmWriteCRLF( fd, subtype, through, complain )			)
-	{ LDEB(1); return -1;	}
-
-    return 0;
-    }
-
 static int mmWrite(	int			fd,
 			const char *		buf,
 			unsigned int		len,
