@@ -10,12 +10,19 @@
 /*									*/
 /************************************************************************/
 
+#   define	LOG_CUTS	0
+#   define	LOG_PALETTE	0
+
 typedef struct HistogramEntry
     {
     int			heCount;
     RGB8Color		heColor;
     } HistogramEntry;
 
+#   define	heRed	heColor.rgb8Red
+#   define	heGreen	heColor.rgb8Green
+#   define	heBlue	heColor.rgb8Blue
+
 typedef struct	HashBucket
     {
     HistogramEntry	hbHistogramEntry;
@@ -30,17 +37,24 @@ typedef struct	HashBucket
 
 typedef struct ColorBox
     {
-    int		cbFirst;
+    int		cbHistogramOffset;
     int		cbColorCount;
     int		cbPixelCount;
     int		cbCutNumber;
+
+    int		cbRMin;
+    int		cbRMax;
+    int		cbGMin;
+    int		cbGMax;
+    int		cbBMin;
+    int		cbBMax;
     } ColorBox;
 
 typedef struct CutNode
     {
     unsigned int	cnLeft;
     unsigned int	cnRight;
-    unsigned char	cnValue;
+    unsigned char	cnLeftMaximum;
     unsigned char	cnComponent;
     } CutNode;
 
@@ -111,32 +125,292 @@ static int bmInsertHash(	HashBucket **	hashTable,
 
 /************************************************************************/
 /*									*/
-/*  Compare colors, used with qsort()					*/
+/*  Determine median.							*/
 /*									*/
 /************************************************************************/
 
-static int bmHistRedCompare( const void * v1, const void * v2 )
+static int bmHistRedMedian(	int *			pPixelCount,
+				int *			pColorCount,
+				int			pixelCount,
+				HistogramEntry *	histogram,
+				int			colorCount,
+				unsigned int		cMin,
+				unsigned int		cMax )
     {
-    const	HistogramEntry *	he1= (HistogramEntry *)v1;
-    const	HistogramEntry *	he2= (HistogramEntry *)v2;
+    int		cMed;
 
-    return he1->heColor.rgb8Red- he2->heColor.rgb8Red;
+    int		lt= 0;
+    int		eq= 0;
+    int		gt= 0;
+
+    cMax++;
+
+    cMed= ( cMin+ cMax )/ 2;
+    for (;;)
+	{
+	int			i;
+	const HistogramEntry *	he;
+
+	lt= 0; eq= 0; gt= 0;
+
+	he= histogram;
+	for ( i= 0; i < colorCount; he++, i++ )
+	    {
+	    if  ( he->heRed > cMed )
+		{ gt += he->heCount; continue; }
+	    if  ( he->heRed < cMed )
+		{ lt += he->heCount; continue; }
+
+	    eq += he->heCount; continue;
+	    }
+
+	if  ( lt+ eq < pixelCount/ 2 )
+	    {
+	    cMin= cMed;
+	    cMed= ( cMin+ cMax )/ 2;
+	    continue;
+	    }
+
+	if  ( eq+ gt < pixelCount/ 2 )
+	    {
+	    cMax= cMed;
+	    cMed= ( cMin+ cMax )/ 2;
+	    continue;
+	    }
+
+	break;
+	}
+
+    if  ( lt+ eq < eq+ gt )
+	{ pixelCount= lt+ eq;		}
+    else{ cMed--; pixelCount= lt;	}
+
+    lt= 0; gt= colorCount- 1;
+    while( gt > lt )
+	{
+	while( gt > lt && histogram[lt].heColor.rgb8Red <= cMed )
+	    { lt++;	}
+	while( gt > lt && histogram[gt].heColor.rgb8Red >  cMed )
+	    { gt--;	}
+
+	if  ( gt > lt )
+	    {
+	    HistogramEntry	swap;
+
+	    swap= histogram[lt];
+	    histogram[lt]= histogram[gt];
+	    histogram[gt]= swap;
+	    }
+	}
+
+    while( lt < colorCount && histogram[lt].heColor.rgb8Red <= cMed )
+	{ lt++;	}
+
+    *pPixelCount= pixelCount; *pColorCount= lt; return cMed;
     }
 
-static int bmHistGreenCompare( const void * v1, const void * v2 )
+static int bmHistGreenMedian(	int *			pPixelCount,
+				int *			pColorCount,
+				int			pixelCount,
+				HistogramEntry *	histogram,
+				int			colorCount,
+				unsigned int		cMin,
+				unsigned int		cMax )
     {
-    const	HistogramEntry *	he1= (HistogramEntry *)v1;
-    const	HistogramEntry *	he2= (HistogramEntry *)v2;
+    int		cMed;
 
-    return he1->heColor.rgb8Green- he2->heColor.rgb8Green;
+    int		lt= 0;
+    int		eq= 0;
+    int		gt= 0;
+
+    cMax++;
+
+    cMed= ( cMin+ cMax )/ 2;
+    for (;;)
+	{
+	int			i;
+	const HistogramEntry *	he;
+
+	lt= 0; eq= 0; gt= 0;
+
+	he= histogram;
+	for ( i= 0; i < colorCount; he++, i++ )
+	    {
+	    if  ( he->heGreen > cMed )
+		{ gt += he->heCount; continue; }
+	    if  ( he->heGreen < cMed )
+		{ lt += he->heCount; continue; }
+
+	    eq += he->heCount; continue;
+	    }
+
+	if  ( lt+ eq < pixelCount/ 2 )
+	    {
+	    cMin= cMed;
+	    cMed= ( cMin+ cMax )/ 2;
+	    continue;
+	    }
+
+	if  ( eq+ gt < pixelCount/ 2 )
+	    {
+	    cMax= cMed;
+	    cMed= ( cMin+ cMax )/ 2;
+	    continue;
+	    }
+
+	break;
+	}
+
+    if  ( lt+ eq < eq+ gt )
+	{ pixelCount= lt+ eq;		}
+    else{ cMed--; pixelCount= lt;	}
+
+    lt= 0; gt= colorCount- 1;
+    while( gt > lt )
+	{
+	while( gt > lt && histogram[lt].heColor.rgb8Green <= cMed )
+	    { lt++;	}
+	while( gt > lt && histogram[gt].heColor.rgb8Green >  cMed )
+	    { gt--;	}
+
+	if  ( gt > lt )
+	    {
+	    HistogramEntry	swap;
+
+	    swap= histogram[lt];
+	    histogram[lt]= histogram[gt];
+	    histogram[gt]= swap;
+	    }
+	}
+
+    while( lt < colorCount && histogram[lt].heColor.rgb8Green <= cMed )
+	{ lt++;	}
+
+    *pPixelCount= pixelCount; *pColorCount= lt; return cMed;
     }
 
-static int bmHistBlueCompare( const void * v1, const void * v2 )
+static int bmHistBlueMedian(	int *			pPixelCount,
+				int *			pColorCount,
+				int			pixelCount,
+				HistogramEntry *	histogram,
+				int			colorCount,
+				unsigned int		cMin,
+				unsigned int		cMax )
     {
-    const	HistogramEntry *	he1= (HistogramEntry *)v1;
-    const	HistogramEntry *	he2= (HistogramEntry *)v2;
+    int		cMed;
 
-    return he1->heColor.rgb8Blue- he2->heColor.rgb8Blue;
+    int		lt= 0;
+    int		eq= 0;
+    int		gt= 0;
+
+    cMax++;
+
+
+    cMed= ( cMin+ cMax )/ 2;
+    for (;;)
+	{
+	int			i;
+	const HistogramEntry *	he;
+
+	lt= 0; eq= 0; gt= 0;
+
+	he= histogram;
+	for ( i= 0; i < colorCount; he++, i++ )
+	    {
+	    if  ( he->heBlue > cMed )
+		{ gt += he->heCount; continue; }
+	    if  ( he->heBlue < cMed )
+		{ lt += he->heCount; continue; }
+
+	    eq += he->heCount; continue;
+	    }
+
+	if  ( lt+ eq < pixelCount/ 2 )
+	    {
+	    cMin= cMed;
+	    cMed= ( cMin+ cMax )/ 2;
+	    continue;
+	    }
+
+	if  ( eq+ gt < pixelCount/ 2 )
+	    {
+	    cMax= cMed;
+	    cMed= ( cMin+ cMax )/ 2;
+	    continue;
+	    }
+
+	break;
+	}
+
+    if  ( lt+ eq < eq+ gt )
+	{ pixelCount= lt+ eq;		}
+    else{ cMed--; pixelCount= lt;	}
+
+    lt= 0; gt= colorCount- 1;
+    while( gt > lt )
+	{
+	while( gt > lt && histogram[lt].heColor.rgb8Blue <= cMed )
+	    { lt++;	}
+	while( gt > lt && histogram[gt].heColor.rgb8Blue >  cMed )
+	    { gt--;	}
+
+	if  ( gt > lt )
+	    {
+	    HistogramEntry	swap;
+
+	    swap= histogram[lt];
+	    histogram[lt]= histogram[gt];
+	    histogram[gt]= swap;
+	    }
+	}
+
+    while( lt < colorCount && histogram[lt].heColor.rgb8Blue <= cMed )
+	{ lt++;	}
+
+    *pPixelCount= pixelCount; *pColorCount= lt; return cMed;
+    }
+
+/************************************************************************/
+/*									*/
+/*  Determine the range of the colors in a box.				*/
+/*									*/
+/************************************************************************/
+
+static void bmDelimitColorBox(		ColorBox *		cb,
+					const HistogramEntry *	histogram )
+    {
+    const HistogramEntry *	he;
+    int				i;
+
+    if  ( cb->cbColorCount == 0 )
+	{ LDEB(cb->cbColorCount); return;	}
+
+    he= histogram+ cb->cbHistogramOffset;
+    cb->cbRMin= cb->cbRMax= he->heRed;
+    cb->cbGMin= cb->cbGMax= he->heGreen;
+    cb->cbBMin= cb->cbBMax= he->heBlue;
+
+    he++;
+
+    for ( i= 1; i < cb->cbColorCount; he++, i++ )
+	{
+	if  ( cb->cbRMin > he->heRed )
+	    { cb->cbRMin = he->heRed;	}
+	if  ( cb->cbRMax < he->heRed )
+	    { cb->cbRMax = he->heRed;	}
+
+	if  ( cb->cbGMin > he->heGreen )
+	    { cb->cbGMin = he->heGreen; }
+	if  ( cb->cbGMax < he->heGreen )
+	    { cb->cbGMax = he->heGreen; }
+
+	if  ( cb->cbBMin > he->heBlue )
+	    { cb->cbBMin = he->heBlue; }
+	if  ( cb->cbBMax < he->heBlue )
+	    { cb->cbBMax = he->heBlue; }
+	}
+
+    return;
     }
 
 /************************************************************************/
@@ -146,87 +420,100 @@ static int bmHistBlueCompare( const void * v1, const void * v2 )
 /************************************************************************/
 
 static void bmSplitBox(	HistogramEntry *	histogram,
-			int			colorCount,
-			int			pixelCount,
-			int *			pColorsLeft,
-			int *			pPixelsLeft,
-			int *			pComponent,
-			int *			pValue		)
+			const ColorBox *	cbParent,
+			ColorBox *		cbLow,
+			ColorBox *		cbHigh,
+			int *			pLeftMaximum,
+			int *			pComponent )
     {
-    unsigned char	rMin, rMax;
-    unsigned char	gMin, gMax;
-    unsigned char	bMin, bMax;
+    int				rDif= cbParent->cbRMax- cbParent->cbRMin;
+    int				gDif= cbParent->cbGMax- cbParent->cbGMin;
+    int				bDif= cbParent->cbBMax- cbParent->cbBMin;
 
-    int			i;
-    int			count;
+    histogram += cbParent->cbHistogramOffset;
 
-    HistogramEntry *	he;
+    *cbLow= *cbParent;
+    *cbHigh= *cbParent;
 
-    rMin= rMax= histogram[0].heColor.rgb8Red;
-    gMin= gMax= histogram[0].heColor.rgb8Green;
-    bMin= bMax= histogram[0].heColor.rgb8Blue;
-
-    he= histogram+ 1;
-    for ( i= 1; i < colorCount; he++, i++ )
+    if  ( 77* rDif > 150* gDif && 77* rDif >  29* bDif )
 	{
-	if  ( rMin > he->heColor.rgb8Red )
-	    { rMin = he->heColor.rgb8Red;	}
-	if  ( rMax < he->heColor.rgb8Red )
-	    { rMax = he->heColor.rgb8Red;	}
+	cbLow->cbRMax= bmHistRedMedian( &(cbLow->cbPixelCount),
+				    &(cbLow->cbColorCount),
+				    cbParent->cbPixelCount, histogram,
+				    cbParent->cbColorCount,
+				    cbParent->cbRMin, cbParent->cbRMax );
 
-	if  ( gMin > he->heColor.rgb8Green )
-	    { gMin = he->heColor.rgb8Green;	}
-	if  ( gMax < he->heColor.rgb8Green )
-	    { gMax = he->heColor.rgb8Green;	}
-
-	if  ( bMin > he->heColor.rgb8Blue )
-	    { bMin = he->heColor.rgb8Blue;	}
-	if  ( bMax < he->heColor.rgb8Blue )
-	    { bMax = he->heColor.rgb8Blue;	}
-	}
-
-    if  ( 77* ( rMax- rMin ) > 150* ( gMax- gMin ) &&
-	  77* ( rMax- rMin ) >  29* ( bMax- bMin ) )
-	{
-	qsort( histogram, colorCount, sizeof(HistogramEntry),
-							bmHistRedCompare );
+	cbHigh->cbRMin= cbLow->cbRMax+ 1;
+	cbHigh->cbColorCount= cbParent->cbColorCount- cbLow->cbColorCount;
+	cbHigh->cbPixelCount= cbParent->cbPixelCount- cbLow->cbPixelCount;
+	cbHigh->cbHistogramOffset= cbParent->cbHistogramOffset+
+							cbLow->cbColorCount;
+	*pLeftMaximum= cbLow->cbRMax;
 	*pComponent= CN_RED;
+
+#	if  LOG_CUTS
+	appDebug( "CUT RED   at %3d..%3d..%3d "
+			    "%4d+ %4d= %4dC %6d+ %6d= %6dP\n",
+			    cbParent->cbRMin, cbLow->cbRMax, cbParent->cbRMax,
+			    cbLow->cbColorCount, cbHigh->cbColorCount,
+			    cbParent->cbColorCount,
+			    cbLow->cbPixelCount, cbHigh->cbPixelCount,
+			    cbParent->cbPixelCount );
+#	endif
 	}
     else{
-	if  ( 150* ( gMax- gMin ) > 29* ( bMax- bMin ) )
+	if  ( 150* gDif > 29* bDif )
 	    {
-	    qsort( histogram, colorCount, sizeof(HistogramEntry),
-							bmHistGreenCompare );
+	    cbLow->cbGMax= bmHistGreenMedian( &(cbLow->cbPixelCount),
+				    &(cbLow->cbColorCount),
+				    cbParent->cbPixelCount, histogram,
+				    cbParent->cbColorCount,
+				    cbParent->cbGMin, cbParent->cbGMax );
+	    cbHigh->cbGMin= cbLow->cbGMax+ 1;
+	    cbHigh->cbColorCount= cbParent->cbColorCount- cbLow->cbColorCount;
+	    cbHigh->cbPixelCount= cbParent->cbPixelCount- cbLow->cbPixelCount;
+	    cbHigh->cbHistogramOffset= cbParent->cbHistogramOffset+
+							cbLow->cbColorCount;
+	    *pLeftMaximum= cbLow->cbGMax;
 	    *pComponent= CN_GREEN;
+
+#	    if  LOG_CUTS
+	    appDebug( "CUT GREEN at %3d..%3d..%3d "
+			    "%4d+ %4d= %4dC %6d+ %6d= %6dP\n",
+			    cbParent->cbGMin, cbLow->cbGMax, cbParent->cbGMax,
+			    cbLow->cbColorCount, cbHigh->cbColorCount,
+			    cbParent->cbColorCount,
+			    cbLow->cbPixelCount, cbHigh->cbPixelCount,
+			    cbParent->cbPixelCount );
+#	    endif
 	    }
 	else{
-	    qsort( histogram, colorCount, sizeof(HistogramEntry),
-							bmHistBlueCompare );
+	    cbLow->cbBMax= bmHistBlueMedian( &(cbLow->cbPixelCount),
+				    &(cbLow->cbColorCount),
+				    cbParent->cbPixelCount, histogram,
+				    cbParent->cbColorCount,
+				    cbParent->cbBMin, cbParent->cbBMax );
+
+	    cbHigh->cbBMin= cbLow->cbBMax+ 1;
+	    cbHigh->cbColorCount= cbParent->cbColorCount- cbLow->cbColorCount;
+	    cbHigh->cbPixelCount= cbParent->cbPixelCount- cbLow->cbPixelCount;
+	    cbHigh->cbHistogramOffset= cbParent->cbHistogramOffset+
+							cbLow->cbColorCount;
+	    *pLeftMaximum= cbLow->cbBMax;
 	    *pComponent= CN_BLUE;
+
+#	    if  LOG_CUTS
+	    appDebug( "CUT BLUE  at %3d..%3d..%3d "
+			    "%4d+ %4d= %4dC %6d+ %6d= %6dP\n",
+			    cbParent->cbBMin, cbLow->cbBMax, cbParent->cbBMax,
+			    cbLow->cbColorCount, cbHigh->cbColorCount,
+			    cbParent->cbColorCount,
+			    cbLow->cbPixelCount, cbHigh->cbPixelCount,
+			    cbParent->cbPixelCount );
+#	    endif
 	    }
 	}
 
-    he= histogram+ 1;
-    count= histogram[0].heCount;
-    for ( i= 1; i < colorCount- 1; he++, i++ )
-	{
-	if  ( count >= pixelCount/ 2 )
-	    { break;	}
-
-	count += he->heCount;
-	}
-
-    switch( *pComponent )
-	{
-	case CN_RED:	*pValue= he->heColor.rgb8Red;	break;
-	case CN_GREEN:	*pValue= he->heColor.rgb8Green;	break;
-	case CN_BLUE:	*pValue= he->heColor.rgb8Blue;	break;
-	default:	LDEB(*pComponent);		break; /* shame */
-	}
-
-    *pColorsLeft= i;
-    *pPixelsLeft= count;
-
     return;
     }
 
@@ -244,6 +531,7 @@ static int bmFindColors(	HistogramEntry *	histogram,
 				int			maxcolors )
     {
     ColorBox *			boxes;
+    const ColorBox *		cb;
     int				i;
     int				j;
     int				boxCount;
@@ -253,26 +541,25 @@ static int bmFindColors(	HistogramEntry *	histogram,
     if  ( ! boxes )
 	{ LLDEB(maxcolors,boxes); return -1; }
 
-    boxes[0].cbFirst= 0;
+    boxes[0].cbHistogramOffset= 0;
     boxes[0].cbColorCount= colorCount;
     boxes[0].cbPixelCount= pixelCount;
     boxes[0].cbCutNumber= 0;
     boxCount= 1;
 
     cutNodes[0].cnComponent= CN_LEAF;
-    cutNodes[0].cnValue= 0;		/*  Irrelevant !	*/
+    cutNodes[0].cnLeftMaximum= 0;	/*  Irrelevant !	*/
     cutNodes[0].cnLeft= 0;		/*  Both to this color.	*/
     cutNodes[0].cnRight= 0;		/*  Both to this color.	*/
     nodeCount= 1;
 
     while( boxCount < maxcolors )
 	{
-	int	biggestBox;
-	int	colorsLeft;
-	int	pixelsLeft;
+	int		biggestBox;
+	int		leftMaximum;
+	int		cutComponent;
 
-	int	cutComponent;
-	int	cutValue;
+	ColorBox	cbSplit;
 
 	for ( i= 0; i < boxCount; i++ )
 	    {
@@ -289,66 +576,75 @@ static int bmFindColors(	HistogramEntry *	histogram,
 		{ biggestBox= i;	}
 	    }
 
-	bmSplitBox( histogram+ boxes[biggestBox].cbFirst,
-				    boxes[biggestBox].cbColorCount,
-				    boxes[biggestBox].cbPixelCount,
-				    &colorsLeft, &pixelsLeft,
-				    &cutComponent, &cutValue );
+	bmDelimitColorBox( boxes+ biggestBox, histogram );
+
+	cbSplit= boxes[biggestBox];
+	bmSplitBox( histogram, &cbSplit, boxes+ biggestBox, boxes+ boxCount,
+						&leftMaximum, &cutComponent );
 
 	/****************/
 	/*  Cut Nodes	*/
 	/****************/
-	i= boxes[biggestBox].cbCutNumber;
+	i= cbSplit.cbCutNumber;
 	cutNodes[i].cnComponent= cutComponent;
-	cutNodes[i].cnValue= cutValue;
+	cutNodes[i].cnLeftMaximum= leftMaximum;
 	cutNodes[i].cnLeft= nodeCount;
 	cutNodes[i].cnRight= nodeCount+ 1;
 
 	cutNodes[nodeCount].cnComponent= CN_LEAF;
-	cutNodes[nodeCount].cnValue= 0;		/*  Irrelevant !	*/
+	cutNodes[nodeCount].cnLeftMaximum= 0;	/*  Irrelevant !	*/
 	cutNodes[nodeCount].cnLeft= biggestBox;	/*  Both to this color.	*/
 	cutNodes[nodeCount].cnRight= biggestBox;/*  Both to this color.	*/
 
 	cutNodes[nodeCount+1].cnComponent= CN_LEAF;
-	cutNodes[nodeCount+1].cnValue= 0;	/*  Irrelevant !	*/
+	cutNodes[nodeCount+1].cnLeftMaximum= 0;	/*  Irrelevant !	*/
 	cutNodes[nodeCount+1].cnLeft= boxCount;	/*  Both to this color.	*/
 	cutNodes[nodeCount+1].cnRight= boxCount;/*  Both to this color.	*/
 
 	/****************/
 	/*  Boxes	*/
 	/****************/
-	boxes[boxCount].cbFirst= boxes[biggestBox].cbFirst+ colorsLeft;
-	boxes[boxCount].cbColorCount=
-				boxes[biggestBox].cbColorCount- colorsLeft;
-	boxes[boxCount].cbPixelCount=
-				boxes[biggestBox].cbPixelCount- pixelsLeft;
-	boxes[boxCount].cbCutNumber= nodeCount+ 1;
-
-	boxes[biggestBox].cbColorCount= colorsLeft;
-	boxes[biggestBox].cbPixelCount= pixelsLeft;
 	boxes[biggestBox].cbCutNumber= nodeCount;
+	boxes[boxCount].cbCutNumber= nodeCount+ 1;
 
 	nodeCount += 2; boxCount++;
 	}
 
-    for ( i= 0; i < boxCount; i++ )
+    cb= boxes;
+    for ( i= 0; i < boxCount; cb++, i++ )
 	{
 	long			sR= 0;
 	long			sG= 0;
 	long			sB= 0;
 	long			N= 0;
-	HistogramEntry *	he= histogram+ boxes[i].cbFirst;
+
+	HistogramEntry *	he= histogram+ cb->cbHistogramOffset;
 
 	for ( j= 0; j < boxes[i].cbColorCount; j++, he++ )
 	    {
-	    sR += he->heCount* he->heColor.rgb8Red;
-	    sG += he->heCount* he->heColor.rgb8Green;
-	    sB += he->heCount* he->heColor.rgb8Blue;
+	    sR += he->heCount* he->heRed;
+	    sG += he->heCount* he->heGreen;
+	    sB += he->heCount* he->heBlue;
 	    N  += he->heCount;
 	    }
+
 	palette[i].rgb8Red= sR/ N;
 	palette[i].rgb8Green= sG/ N;
 	palette[i].rgb8Blue= sB/ N;
+
+#	if LOG_PALETTE
+	appDebug( "BOX %3d: "
+			"R %3d..%3d..%3d "
+			"G %3d..%3d..%3d "
+			"B %3d..%3d..%3d "
+			"%4dC %6ldP\n",
+
+			i,
+			cb->cbRMin, palette[i].rgb8Red, cb->cbRMax,
+			cb->cbGMin, palette[i].rgb8Green, cb->cbGMax,
+			cb->cbBMin, palette[i].rgb8Blue, cb->cbBMax,
+			boxes[i].cbColorCount, N );
+#	endif
 	}
 
     free( (char *)boxes );
@@ -396,15 +692,15 @@ static void bm24to8or4Cut(	int			pixelsHigh,
 		switch( cutNodes[i].cnComponent )
 		    {
 		    case CN_RED:
-			if  ( r < cutNodes[i].cnValue )
+			if  ( r <= cutNodes[i].cnLeftMaximum )
 			    { i= cutNodes[i].cnLeft;	continue; }
 			else{ i= cutNodes[i].cnRight;	continue; }
 		    case CN_GREEN:
-			if  ( g < cutNodes[i].cnValue )
+			if  ( g <= cutNodes[i].cnLeftMaximum )
 			    { i= cutNodes[i].cnLeft;	continue; }
 			else{ i= cutNodes[i].cnRight;	continue; }
 		    case CN_BLUE:
-			if  ( b < cutNodes[i].cnValue )
+			if  ( b <= cutNodes[i].cnLeftMaximum )
 			    { i= cutNodes[i].cnLeft;	continue; }
 			else{ i= cutNodes[i].cnRight;	continue; }
 		    case CN_LEAF:
